#///////////////////////////////////////////////////////////////////////
#NOTE: This script tests the performance difference between s2dverification::Ano,
#      s2dv::Ano, and s2dv::Ano with apply() instead of Apply() inside.
#///////////////////////////////////////////////////////////////////////
#CONCLUSION:
#///////////////////////////////////////////////////////////////////////

library(s2dv)
library(s2dverification)
library(rbenchmark)
library(multiApply)
library(compiler)

#___________ Data ____________
# Small
  set.seed(1)
  data1 <- array(rnorm(360), c(dat = 2, member = 3, sdate = 50, ftime = 12))
  set.seed(2)
  clim1 <- array(rnorm(72), c(dat = 2, member = 3, ftime = 12))

# Large
  set.seed(1)
  data2 <- array(rnorm(360), c(dat = 2, member = 3, sdate = 50, ftime = 12, lon = 360, lat = 180))
  set.seed(2)
  clim2 <- array(rnorm(72), c(dat = 2, member = 3, ftime = 12, lon = 360, lat = 180))


##___________ s2dv::Ano_apply ____________
#  Ano_apply <- function(data, clim, ncores = NULL) {
#
#  ###############################
#  # Calculate Ano
#
##  res <- Apply(list(data),
##               target_dims = names(dim(clim)),
##               output_dims = names(dim(clim)),
##               fun = .Ano,
##               clim = clim,
##               ncores = ncores)$output1
#
##  margin_dim_ind <- names
#  target_dim_ind <- match(names(dim(clim)), names(dim(data)))
#print(c(1:length(dim(data)))[-target_dim_ind])
#  res <- apply(data, c(1:length(dim(data)))[-target_dim_ind],
#               .Ano, clim = clim)
#
#    return(invisible(res))
#  }
#
#  .Ano <- function(data, clim) {
#print('dim(data)')
#print(dim(data))
#print('dim(clim)')
#print(dim(clim))
#    ano <- data - clim
#print('dim(ano)')
#print(dim(ano))
#    return(ano)
#  }


#___________ s2dv::Ano_nocheck ____________
  Ano_nocheck <- function(data, clim, ncores = NULL) {

  ###############################
  # Calculate Ano

  res <- Apply(list(data),
               target_dims = names(dim(clim)),
               output_dims = names(dim(clim)),
               fun = .Ano,
               clim = clim,
               ncores = ncores)$output1

    return(invisible(res))
  }

  .Ano <- function(data, clim) {
    ano <- data - clim
    return(ano)
  }

#___________ s2dv::Ano_compile ____________
  Ano_compile <- function(data, clim, ncores = NULL) {

  ###############################
  # Calculate Ano

  res <- Apply(list(data),
               target_dims = names(dim(clim)),
               output_dims = names(dim(clim)),
               fun = .Ano_compile,
               clim = clim,
               ncores = ncores)$output1

    return(invisible(res))
  }

  .Ano_compile <- cmpfun(function(data, clim) {
    ano <- data - clim
    return(ano)
  })


#__________ Test ___________
# Small data
benchmark("s2dverification::Ano" = {
            res_s2dverification <- s2dverification::Ano(data1, clim1)
          },

          "s2dv::Ano" = {
             res_s2dv <- s2dv::Ano(data1, clim1, ncores = 1)
          },

          "s2dv::Ano2" = {
             res_s2dv <- s2dv::Ano(data1, clim1, ncores = 2)
          },

#          "s2dv::Ano_nocheck" = {
#             res_s2dv_nocheck <- Ano_nocheck(data1, clim1)
#          },
          "s2dv::Ano_compile" = {
             res_s2dv_compile <- Ano_compile(data1, clim1)
          },
          replications = 50,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

#                  test replications elapsed relative user.self sys.self
#2            s2dv::Ano           50   0.188   10.444     0.188    0.000
#4    s2dv::Ano_compile           50   0.236   13.111     0.236    0.000
#3           s2dv::Ano2           50   1.316   73.111     0.240    0.916
#1 s2dverification::Ano           50   0.018    1.000     0.018    0.000

#///////////////////////////////////////////////////////////////////////
# Large data
# Killed!!!!!!
benchmark("s2dverification::Ano" = {
            res_s2dverification <- s2dverification::Ano(data2, clim2)
          },

          "s2dv::Ano" = {
             res_s2dv <- s2dv::Ano(data2, clim2, ncores = 1)
          },

          "s2dv::Ano2" = {
             res_s2dv <- s2dv::Ano(data2, clim2, ncores = 2)
          },

          "s2dv::Ano_nocheck" = {
             res_s2dv_nocheck <- Ano_nocheck(data2, clim2)
          },
          "s2dv::Ano_compile" = {
             res_s2dv_compile <- Ano_compile(data2, clim2)
          },
          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))
