# Compare s2dv::MeanDims and s2dverification::Mean1Dim/MeanListDims
library(s2dv)
library(s2dverification)
library(rbenchmark)

# Mean along 1 dim
benchmark("MeanDims" = {
            data <- array(rnorm(1:1000),  
                          dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
            res_MeanDims <- s2dv::MeanDims(data, 3)
          },

          "Mean1Dim" = {
            data <- array(rnorm(1:1000),
                          dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
             res_Mean1Dim <- s2dverification::Mean1Dim(data, 3)
          },

          "MeanListDim" = {
             data <- array(rnorm(1:1000),
                           dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
             res_MeanListDim <- s2dverification::MeanListDim(data, 3)
          },
          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

         test replications elapsed relative user.self sys.self
2    Mean1Dim           10  64.622    1.000    61.024    3.168
1    MeanDims           10  91.476    1.416    88.897    1.912
3 MeanListDim           10  93.579    1.448    90.890    2.045

# Mean along two dims
benchmark("MeanDims" = {
            data <- array(rnorm(1:1000), 
                          dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
            res_MeanDims <- s2dv::MeanDims(data, c(2, 3))
          },

          "Mean1Dim" = {
            data <- array(rnorm(1:1000),
                          dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
             res_Mean1Dim <- s2dverification::Mean1Dim(s2dverification::Mean1Dim(data, 2), 2)
          },
          
          "MeanListDim" = {
             data <- array(rnorm(1:1000),
                           dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
             res_MeanListDim <- s2dverification::MeanListDim(data, c(2, 3))
          },
          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

         test replications elapsed relative user.self sys.self
2    Mean1Dim           10  67.951    2.761    64.912    2.572
1    MeanDims           10  24.764    1.006    22.214    2.360
3 MeanListDim           10  24.608    1.000    23.121    1.308


# Mean along three dims
benchmark("MeanDims" = {
            data <- array(rnorm(1:1000),
                          dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
            res_MeanDims <- s2dv::MeanDims(data, c(2, 3, 4))
          },

          "Mean1Dim" = {
            data <- array(rnorm(1:1000),
                          dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
             res_Mean1Dim <- s2dverification::Mean1Dim(s2dverification::Mean1Dim(s2dverification::Mean1Dim(data, 2), 2), 2)
          },

          "MeanListDim" = {
             data <- array(rnorm(1:1000),
                           dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
             res_MeanListDim <- s2dverification::MeanListDim(data, c(2, 3, 4))
          },
          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

         test replications elapsed relative user.self sys.self
2    Mean1Dim           10  68.918    4.929    64.688    3.756
1    MeanDims           10  13.983    1.000    12.093    1.780
3 MeanListDim           10  16.335    1.168    12.725    3.457


#------------- Remove MeanDims check -----------------

MeanDims2 <- function(data, dims, na.rm = FALSE) {
# Calculate MeanDims

  ## Change character dims into indices
  if (is.character(dims)) {
    tmp <- rep(0, length(dims))
    for (i in 1:length(dims)) {
      tmp[i] <- which(names(dim(data)) == dims[i])
    }
  dims <- tmp
  }

  if (length(dim(data)) == 1) {
    res <- mean(data, na.rm = na.rm) 
  } else {

    margins <- setdiff(c(1:length(dim(data))), dims)
    res <- as.array(apply(data, margins, mean, na.rm = na.rm))
    if (!is.null(names(dim(data))[margins]) & is.null(names(dim(res)))) {
      names(dim(res)) <- names(dim(data))[margins]
    }
  }

  return(res)

}
# Mean along 1 dim
benchmark("MeanDims" = {
            data <- array(rnorm(1:1000),
                          dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
            res_MeanDims <- MeanDims2(data, 3)
          },

          "Mean1Dim" = {
            data <- array(rnorm(1:1000),
                          dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
             res_Mean1Dim <- s2dverification::Mean1Dim(data, 3)
          },

          "MeanListDim" = {
             data <- array(rnorm(1:1000),
                           dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
             res_MeanListDim <- s2dverification::MeanListDim(data, 3)
          },
          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

2    Mean1Dim           10  65.036    1.000    60.980    3.596
1    MeanDims           10  70.094    1.078    68.348    1.304
3 MeanListDim           10  92.020    1.415    89.706    1.716
