#//////////////////////////////////////////////////////
#NOTE: Run in RStudio
#//////////////////////////////////////////////////////

library(s2dv)
library(s2dverification)
library(profvis)
library(compiler)
library(multiApply)


#========================================================
# See results at https://earth.bsc.es/gitlab/ces/multiApply/-/issues/11

Season_apply_nocheck <- function(data, time_dim = 'ftime', monini, moninf, monsup,
                                 method = mean, na.rm = TRUE, ncores = NULL) {

  ###############################
  # Calculate Season

  # Correction need if monini is not January: 
  moninf <- moninf - monini + 1
  monsup <- monsup - monini + 1
  moninf <- ifelse(moninf <= 0, moninf + 12, moninf)
  monsup <- ifelse(monsup <= 0, monsup + 12, monsup)

  while (monsup < moninf) {
    monsup <- monsup + 12
  }

  #    res <- Apply(list(data), 
  #                 target_dims = time_dim, 
  #                 output_dims = time_dim,
  #                 fun = .Season, 
  #                 monini = monini, moninf = moninf, monsup = monsup,
  #                 method = method, na.rm = na.rm, ncores = ncores)$output1
  res <- apply(data, seq(1, length(dim(data)))[-match(time_dim, names(dim(data)))], s2dv:::.Season,
               monini = monini, moninf = moninf, monsup = monsup,
               method = method, na.rm = na.rm)
  return(res)
}


set.seed(1)
dat1 <- array(rnorm(144*3), dim = c(member = 2, sdate = 2, ftime = 12*3, lon = 3))

profvis({
  res_Season <- s2dv::Season(data = dat1, monini = 1, moninf = 1, monsup = 2)

})

profvis({
res_s2dverification <- s2dverification::Season(dat1, monini = 11, moninf = 12, monsup = 1, posdim = 3)
})

profvis({
  res_Season_apply <- Season_apply_nocheck(data = dat1, monini = 1, moninf = 1, monsup = 2)

})

#==========================================================
#==========================================================


set.seed(1)
dat1 <- array(rnorm(144*3), dim = c(dat = 1, member = 2, sdate = 2, ftime = 12*3, lon = 3))


profvis({
  
.Season2 <- cmpfun(s2dv:::.Season)
  
Season_outside <- function(data, time_dim = 'ftime', monini, moninf, monsup,
                           method = mean, na.rm = TRUE, ncores = NULL) {
  
  # Check inputs 
  ## data
  if (is.null(data)) {
    stop("Parameter 'data' cannot be NULL.")
  }
  if (!is.numeric(data)) {
    stop("Parameter 'data' must be a numeric array.")
  }
  if (is.null(dim(data))) {  #is vector
    dim(data) <- c(length(data))
    names(dim(data)) <- time_dim
  }
  if(any(is.null(names(dim(data))))| any(nchar(names(dim(data))) == 0)) {
    stop("Parameter 'data' must have dimension names.")
  }
  ## time_dim
  if (!is.character(time_dim) | length(time_dim) > 1) {
    stop("Parameter 'time_dim' must be a character string.")
  }
  if (!time_dim %in% names(dim(data))) {
    stop("Parameter 'time_dim' is not found in 'data' dimension.")
  }
  ## monini
  if (!is.numeric(monini)) {
    stop("Parameter 'monini' must be a positive integer between 1 and 12.")
  } else {
    if (monini %% 1 != 0 | monini < 1 | monini > 12 | length(monini) > 1) {
      stop("Parameter 'monini' must be a positive integer between 1 and 12.")
    }
  }
  ## moninf
  if (!is.numeric(moninf)) {
    stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
  } else {
    if (moninf %% 1 != 0 | moninf < 1 | moninf > 12 | length(moninf) > 1) {
      stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
    }
  }
  ## monsup
  if (!is.numeric(monsup)) {
    stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
  } else {
    if (monsup %% 1 != 0 | monsup < 1 | monsup > 12 | length(monsup) > 1) {
      stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
    }
  }
  ## method
  if (!is.function(method)) {
    stop("Parameter 'method' should be an existing R function, e.g., mean or sum.")
  }
  ## na.rm
  if (!is.logical(na.rm) | length(na.rm) > 1) {
    stop("Parameter 'na.rm' must be one logical value.")
  }
  ## ncores
  if (!is.null(ncores)) {
    if (!is.numeric(ncores) | ncores %% 1 != 0 | ncores < 0 |
        length(ncores) > 1) {
      stop("Parameter 'ncores' must be a positive integer.")
    }
  }
  
  ###############################
  # Calculate Season
  
  # Correction need if monini is not January: 
  moninf <- moninf - monini + 1
  monsup <- monsup - monini + 1
  moninf <- ifelse(moninf <= 0, moninf + 12, moninf)
  monsup <- ifelse(monsup <= 0, monsup + 12, monsup)
  
  while (monsup < moninf) {
    monsup <- monsup + 12
  }
  
  res <- Apply(list(data),
               target_dims = time_dim,
               output_dims = time_dim,
               fun = .Season2,
               monini = monini, moninf = moninf, monsup = monsup,
               method = method, na.rm = na.rm, ncores = ncores)$output1
  
  return(res)
}

res_Season <- Season_outside(data = dat1, monini = 1, moninf = 1, monsup = 2)

})

profvis({
Season_inside <- function(data, time_dim = 'ftime', monini, moninf, monsup,
                          method = mean, na.rm = TRUE, ncores = NULL) {
  
  # Check inputs 
  ## data
  if (is.null(data)) {
    stop("Parameter 'data' cannot be NULL.")
  }
  if (!is.numeric(data)) {
    stop("Parameter 'data' must be a numeric array.")
  }
  if (is.null(dim(data))) {  #is vector
    dim(data) <- c(length(data))
    names(dim(data)) <- time_dim
  }
  if(any(is.null(names(dim(data))))| any(nchar(names(dim(data))) == 0)) {
    stop("Parameter 'data' must have dimension names.")
  }
  ## time_dim
  if (!is.character(time_dim) | length(time_dim) > 1) {
    stop("Parameter 'time_dim' must be a character string.")
  }
  if (!time_dim %in% names(dim(data))) {
    stop("Parameter 'time_dim' is not found in 'data' dimension.")
  }
  ## monini
  if (!is.numeric(monini)) {
    stop("Parameter 'monini' must be a positive integer between 1 and 12.")
  } else {
    if (monini %% 1 != 0 | monini < 1 | monini > 12 | length(monini) > 1) {
      stop("Parameter 'monini' must be a positive integer between 1 and 12.")
    }
  }
  ## moninf
  if (!is.numeric(moninf)) {
    stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
  } else {
    if (moninf %% 1 != 0 | moninf < 1 | moninf > 12 | length(moninf) > 1) {
      stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
    }
  }
  ## monsup
  if (!is.numeric(monsup)) {
    stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
  } else {
    if (monsup %% 1 != 0 | monsup < 1 | monsup > 12 | length(monsup) > 1) {
      stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
    }
  }
  ## method
  if (!is.function(method)) {
    stop("Parameter 'method' should be an existing R function, e.g., mean or sum.")
  }
  ## na.rm
  if (!is.logical(na.rm) | length(na.rm) > 1) {
    stop("Parameter 'na.rm' must be one logical value.")
  }
  ## ncores
  if (!is.null(ncores)) {
    if (!is.numeric(ncores) | ncores %% 1 != 0 | ncores < 0 |
        length(ncores) > 1) {
      stop("Parameter 'ncores' must be a positive integer.")
    }
  }
  
  ###############################
  # Calculate Season
  
  # Correction need if monini is not January: 
  moninf <- moninf - monini + 1
  monsup <- monsup - monini + 1
  moninf <- ifelse(moninf <= 0, moninf + 12, moninf)
  monsup <- ifelse(monsup <= 0, monsup + 12, monsup)
  
  while (monsup < moninf) {
    monsup <- monsup + 12
  }
  
  res <- Apply(list(data),
               target_dims = time_dim,
               output_dims = time_dim,
               fun = cmpfun(s2dv:::.Season),
               monini = monini, moninf = moninf, monsup = monsup,
               method = method, na.rm = na.rm, ncores = ncores)$output1
  
  return(res)
}

  res_Season <- s2dv::Season(data = dat1, monini = 1, moninf = 1, monsup = 2)
  
})





profvis({
  res_Season <- Season_inside(data = dat1, monini = 1, moninf = 1, monsup = 2)
  
})

