library(startR)

# Define a region
lons.min <- 0
lons.max <- 359.99
lats.min <- -90
lats.max <- 90

# Declaration of data sources
exp <- Start(dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = '19811101',
             ensemble = indices(1),
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(), #Sort(decreasing = T),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(0, 360),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r100x50',
                                     method = 'conservative',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = T)

lon <- attr(exp, 'Variables')$dat1$longitude
lat <- attr(exp, 'Variables')$dat1$latitude

s2dv::PlotEquiMap(exp, lon = lon, lat = lat, filled.continents = F, 
                  contours = exp[1,1,1,1,1,,])
s2dv::PlotStereoMap(exp, lon = lon, lat = lat)

PlotStereoMap(exp, lon = lon, lat = lat, fileout = 'stereo_no_contour.png')
PlotStereoMap(exp, lon = lon, lat = lat, contours = exp, contour_color = 'green', 
              brks2 = 11, fileout = 'stereo_contour.png')

dots <- array(sample(c(TRUE, FALSE), size= 5000, replace = TRUE), c(lat = 50, lon = 100))
PlotStereoMap(exp, lon = lon, lat = lat, dots = dots, fileout = 'stereo_dots.png')
