library(startR)

# Define a region
lons.min <- 0
lons.max <- 359.99
lats.min <- 60 #-90
lats.max <- 90 #-50 #90
sdate <- paste0(1981:2011, '0101')
grid_type <- 'r360x180'

# Declaration of data sources
var <- Start(dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = sdate,
             ensemble = indices(1),
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(decreasing = F),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(0, 360),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = grid_type,
                                     method = 'bil',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = T)

lon <- attr(var, 'Variables')$dat1$longitude
lat <- attr(var, 'Variables')$dat1$latitude

# wind
uas <- Start(dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'uas',
             sdate = sdate,
             ensemble = indices(1),
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(decreasing = F),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(0, 360),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = grid_type,
                                     method = 'bil',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = T)

vas <- Start(dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'vas',
             sdate = sdate,
             ensemble = indices(1),
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(decreasing = F),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(0, 360),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = grid_type,
                                     method = 'bil',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = T)


  func <- function(x, y, z) {
    x <- mean(x)
    y <- mean(y)
    z <- mean(z)
    dim(x) <- c(sdate = 1)
    dim(y) <- c(sdate = 1)   
    dim(z) <- c(sdate = 1)   
    return(list(var = x, uas = y, vas = z))
  }
  step <- Step(func, target_dims = list(x = 'sdate', y = 'sdate', z = 'sdate'),
                     output_dims = list(var = 'sdate', uas = 'sdate', vas = 'sdate'))
  wf <- AddStep(list(var, uas, vas), step)

  res <- Compute(wf$var,
                 chunks = list(latitude = 2,
                               longitude = 2))

#=========================================================
lon <- attr(var, 'Variables')$dat1$longitude
lat <- attr(var, 'Variables')$dat1$latitude
var <- res$var
uas <- res$uas
vas <- res$vas

#-----------------------------------------------
var <- s2dv::MeanDims(var, 'sdate')
uas <- s2dv::MeanDims(uas, 'sdate')
vas <- s2dv::MeanDims(vas, 'sdate')
#=======================================================
s2dv::PlotEquiMap(drop(var), lon = lon, lat = lat, varu = drop(uas), varv = drop(vas), arr_subsamp = 5, arr_ref_len = 15, arr_scale_shaft = 0.5, filled.continents = F, fileout = 'equi_map_arrow.png')


PlotStereoMap(drop(var), lon = lon, lat = lat, varu = drop(uas), varv = drop(vas), 
              bar_limits = c(225, 280), #latlims = c(-90, -60), 
              units = 'K',
              arr_subsamp = 5, arr_scale_shaft_angle = 1.8, arr_scale_shaft = 0.25, arr_scale = 1.2,
              toptitle = "tas + 10m wind, 1981-2010 climatology", title_scale = 0.6,
              filled.continents = F, fileout = 'stereo_map_arrow.png')



set.seed(1)
dots <- array(sample(c(TRUE, FALSE), size= 40*360, replace = TRUE), c(lat = 40, lon = 360))

PlotStereoMap(drop(var), lon = lon, lat = lat,bar_limits = c(225, 280),
              dots = dots,filled.continents = F, fileout = 'stereo_map_dots.png')

