library(s2dv)
library(startR)


path_exp <- paste0('/esarchive/exp/ecmwf/system5_m1/daily_mean/',
                   '$var$_f24h/$var$_$sdate$.nc')
var <- 'tasmax'

# define sdates
firsty <- "1982"
lasty <- "1987"
months <- 1:12
firstd <- paste0(firsty,"0101")
sdates <- ymd(firstd) + months(0:11) + rep(years(0:(1987-1981)), 
          each = 12)
sdates <- format(sdates, "%Y%m%d")

# define region
lons.min <- 10
lons.max <- 20
lats.min <- -10
lats.max <- 10


data <- Start(dat = path_exp,
              var = var,
              ensemble = 'all',
              sdate = sdates,
              time = indices(1),
              latitude = values(list(lats.min, lats.max)),
              latitude_reorder = Sort(),
              longitude = values(list(lons.min, lons.max)),
              longitude_reorder = CircularSort(-180, 180),
              synonims = list(latitude=c('lat', 'latitude'),
                              longitude=c('lon', 'longitude')),
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = NULL),
              retrieve = F)

 fun <- function(x) {
   res <- s2dv::Trend(x, time_dim = 'sdate')
   return(res$detrended)
 }

  step <- Step(fun, target_dims = c('sdate'),
                    output_dims = c('sdate'))

  wf <- AddStep(data, step)


 res <- Compute(wf,
                chunks = list(latitude = 2,
                              longitude = 2))

