library(s2dverification)

pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')

obs_s2dv <- Load(var = 'tas',
                 obs = list(list(path = pobs)),
                 sdates = '20181101',
                 leadtimemax = 1,
                 output = 'lonlat',
                 storefreq = 'monthly')#,
#                 latmin = lats.min,
#                 latmax = lats.max,
#                 lonmin = lons.min,
#                 lonmax = lons.max)

# latitude(640): 89.8,...-89.78488
# longitude(1280): 0,0.281,...359.7188

lons <- as.vector(obs_s2dv$lon)
lats <- as.vector(obs_s2dv$lat)

# ------------grid -----------------
# lat(180): -90,90  lon(360):-180,180
grid <- '/esarchive/obs/lbl/berkeley-earth/daily_mean/tas/tas_196508.nc' 
# lat(181): -90,90  lon(360):0,359
#grid <- 'r360x181'
#-----------------------------------


#---- crop = TRUE/'preserve'/'tight'(this case) -------
obs_remap1 <- CDORemap(obs_s2dv$obs, lons, lats, grid, 'con', crop = TRUE)

str(obs_remap1)
List of 3
 $ data_array: num [1, 1, 1, 1, 1:179, 1:360] 236 237 237 236 237 ...
  ..- attr(*, "dimensions")= chr [1:6] "dataset" "member" "sdate" "ftime" ...
 $ lons      : num [1:360(1d)] 0 1 2 3 4 5 6 7 8 9 ...
 $ lats      : num [1:179(1d)] -89 -88 -87 -86 -85 -84 -83 -82 -81 -80 ...

##grid = file
List of 3
 $ data_array: num [1, 1, 1, 1, 1:180, 1:360] 236 236 237 236 236 ...
  ..- attr(*, "dimensions")= chr [1:6] "dataset" "member" "sdate" "ftime" ...
 $ lons      : num [1:360(1d)] 0.5 1.5 2.5 3.5 4.5 5.5 6.5 7.5 8.5 9.5 ...
 $ lats      : num [1:180(1d)] -89.5 -88.5 -87.5 -86.5 -85.5 -84.5 -83.5 -82.5 -81.5 -80.5 ...



#---- crop = FALSE -------
obs_remap2 <- CDORemap(obs_s2dv$obs, lons, lats, grid, 'con', crop = FALSE)

str(obs_remap2)
List of 3
 $ data_array: num [1, 1, 1, 1, 1:181, 1:360] 236 236 237 237 236 ...
  ..- attr(*, "dimensions")= chr [1:6] "dataset" "member" "sdate" "ftime" ...
 $ lons      : num [1:360(1d)] 0 1 2 3 4 5 6 7 8 9 ...
 $ lats      : num [1:181(1d)] -90 -89 -88 -87 -86 -85 -84 -83 -82 -81 ...

##grid = file
List of 3
 $ data_array: num [1, 1, 1, 1, 1:180, 1:360] 236 236 239 242 245 ...
  ..- attr(*, "dimensions")= chr [1:6] "dataset" "member" "sdate" "ftime" ...
 $ lons      : num [1:360(1d)] -180 -178 -178 -176 -176 ...
 $ lats      : num [1:180(1d)] -89.5 -88.5 -87.5 -86.5 -85.5 -84.5 -83.5 -82.5 -81.5 -80.5 ...


#---- crop = c() -------
obs_remap3 <- CDORemap(obs_s2dv$obs, lons, lats, grid, 'con', crop = c(lons[1], lons[1280], lats[1], lats[640]))
## grid = 'r360x181'
str(obs_remap3)
List of 3
 $ data_array: num [1, 1, 1, 1, 1:179, 1:360] 236 237 237 236 237 ...
  ..- attr(*, "dimensions")= chr [1:6] "dataset" "member" "sdate" "ftime" ...
 $ lons      : num [1:360(1d)] 0 1 2 3 4 5 6 7 8 9 ...
 $ lats      : num [1:179(1d)] -89 -88 -87 -86 -85 -84 -83 -82 -81 -80 ...
##grid = file
List of 3
 $ data_array: num [1, 1, 1, 1, 1:180, 1:360] 236 236 237 236 236 ...
  ..- attr(*, "dimensions")= chr [1:6] "dataset" "member" "sdate" "ftime" ...
 $ lons      : num [1:360(1d)] 0.5 1.5 2.5 3.5 4.5 5.5 6.5 7.5 8.5 9.5 ...
 $ lats      : num [1:180(1d)] -89.5 -88.5 -87.5 -86.5 -85.5 -84.5 -83.5 -82.5 -81.5 -80.5 ...



## cdo -remapcon,r360x181 tas_201204.nc tas_201204_1.nc
library(easyNCDF)
  path <- '/home/Earth/aho/aho-testtest/s2dv/issue/issue240/tas_201204_1.nc'
  fnc <- NcOpen(path)
  fnc_dims <- NcReadDims(fnc)
  cdo_lon <- NcToArray(fnc, vars_to_read = 'lon')  #lon: 0,1,2,...,359 (360)
  cdo_lat <- NcToArray(fnc, vars_to_read = 'lat')  #lat: -90,-89,...,89,90 (181)

##cdo -remapcon,/esarchive/obs/lbl/berkeley-earth/daily_mean/tas/tas_196508.nc tas_201204.nc tas_201204_3.nc
  path <- '/home/Earth/aho/aho-testtest/s2dv/issue/issue240/tas_201204_3.nc'
  fnc <- NcOpen(path)
  fnc_dims <- NcReadDims(fnc)
  cdo_lon <- NcToArray(fnc, vars_to_read = 'longitude') #lon: -179.5,-178.5,...,179.5 (360)
  cdo_lat <- NcToArray(fnc, vars_to_read = 'latitude')  #lat: -89.5,-88.5,...89.5 (180)


## cdo -sellonlatbox,0,360,-90,90 -remapcon,r360x181 tas_201204.nc tas_201204_4.nc
lon: 0,1,...,359 (360)
lat: -90,-89,...,90 (181)

## cdo -sellonlatbox,0,360,-90,90 -remapcon,/esarchive/obs/lbl/berkeley-earth/daily_mean/tas/tas_196508.nc tas_201204.nc tas_201204_5.nc
lon: 0.5,1.5,...,359.5 (360)
lat: -89.5,-88.5,...89.5 (180)


# interpolate within Load()
obs1 <- Load(var = 'tas',
             obs = list(list(path = pobs)),
             sdates = '20181101',
             leadtimemax = 1,
             output = 'lonlat',
             grid = grid,
             method = 'conservative',
             storefreq = 'monthly')

  lons2 <- as.vector(obs1$lon)  #0,1,...,359 (360)
  lats2 <- as.vector(obs1$lat)  #90,89,...-89,-90 (181)

