library(s2dverification)

pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')

lats.min <- -10
lats.max <- 10
lons.min <- 0
lons.max <- 10

obs_s2dv <- Load(var = 'tas',
                 obs = list(list(path = pobs)),
                 sdates = '20181101',
                 leadtimemax = 1,
                 output = 'lonlat',
                 storefreq = 'monthly',
                 latmin = lats.min,
                 latmax = lats.max,
                 lonmin = lons.min,
                 lonmax = lons.max)
#original data
# latitude: 90, -90
# longitude: 0, 360

lons <- as.vector(obs_s2dv$lon)
#str(lons)
# num [1:36] 0 0.281 0.562 0.844 1.125 ... 9.84375

lats <- as.vector(obs_s2dv$lat)
#str(lats)
# num [1:72] 9.98 9.7 9.41 9.13 8.85 ... -9.9765777


#---- crop = TRUE/'preserve'/'tight'(this case) -------
obs_remap1 <- CDORemap(obs_s2dv$obs, lons, lats, 'r360x181', 'con', crop = TRUE)

str(obs_remap1)
List of 3
 $ data_array: num [1, 1, 1, 1, 1:19, 1:10] 295 296 296 297 298 ...
  ..- attr(*, "dimensions")= chr [1:6] "dataset" "member" "sdate" "ftime" ...
 $ lons      : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
 $ lats      : num [1:19(1d)] -9 -8 -7 -6 -5 -4 -3 -2 -1 0 ...


#---- crop = FALSE -------
obs_remap2 <- CDORemap(obs_s2dv$obs, lons, lats, 'r360x181', 'con', crop = FALSE)

str(obs_remap2)
List of 3
 $ data_array: num [1, 1, 1, 1, 1:181, 1:360] NA NA NA NA NA NA NA NA NA NA ...
  ..- attr(*, "dimensions")= chr [1:6] "dataset" "member" "sdate" "ftime" ...
 $ lons      : num [1:360(1d)] 0 1 2 3 4 5 6 7 8 9 ...
 $ lats      : num [1:181(1d)] -90 -89 -88 -87 -86 -85 -84 -83 -82 -81 ...


# use cdo to create sub-region data (not remap yet. Same as Load without remap)
## cdo sellonlatbox,0,10,10,-10 tas_201204.nc tas_201204_sub.nc
# then use cdo to remap the sub-region
## cdo -remapcon,r360x181 tas_201204_sub.nc tas_201204_2.nc
library(easyNCDF)
  path <- '/home/Earth/aho/aho-testtest/s2dv/issue/issue240/tas_201204_2.nc'
  fnc <- NcOpen(path)
  fnc_dims <- NcReadDims(fnc)
  cdo_lon <- NcToArray(fnc, vars_to_read = 'lon')  #lon: 0,1,2,...,359 (360)
  cdo_lat <- NcToArray(fnc, vars_to_read = 'lat')  #lat: -90,-89,...,89,90 (181)
  cdo_tas <- NcToArray(fnc, vars_to_read = 'tas')

# interpolate within Load()
obs1 <- Load(var = 'tas',
             obs = list(list(path = pobs)),
             sdates = '20181101',
             leadtimemax = 1,
             output = 'lonlat',
             grid = 'r360x181',
             method = 'conservative',
             storefreq = 'monthly',
                 latmin = lats.min,
                 latmax = lats.max,
                 lonmin = lons.min,
                 lonmax = lons.max)

  lons2 <- as.vector(obs1$lon)  #0,1,...,10 (11)
  lats2 <- as.vector(obs1$lat)  #10,9,...,-10 (21)

