func_cal <- function(exp, proj, memb, table, grid, ver, sdate, type, cal_choice, var, dataInput, comp, rundate1, rundate2, cal_term) {
  print('in func_cal')
  continue = TRUE  #for stop
  
  #------decide how many chuncks needed--------
  fun_chunk <- function(data) {
    
    tmp <- as.vector((attr(data, "Dimensions")))
    data_size <- 1
    
    for (i in 1:length(tmp)){
      data_size = tmp[i] * data_size
    }
    data_size <- data_size * 8 / 1000000   # *8 bytes, byte to mb
    chunk_n <- round(data_size / 1000)  # 1GB per chunk
    if (chunk_n == 0) {
      chunk_lat <- 1
      chunk_lon <- 1
    } else if (chunk_n %% 2 == 1) {
      chunk_n <- chunk_n + 1
      chunk_lat <- chunk_n / 2
      chunk_lon <- 2
    } else {
      chunk_lat <- chunk_n / 2
      chunk_lon <- 2
    }
    chunk <- c(chunk_lat, chunk_lon)
    return(chunk)
  }  #fun_chunk
  
  
  #------define new path for group read-in----------
  opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                     exp, 'cmorfiles/',proj, '/EC-Earth-Consortium/'))
  opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                   exp, 'cmorfiles/',proj, '/EC-Earth-Consortium/',
                                   opt_Earth3))
  dir <- file.path('/esarchive/exp/ecearth',
                   exp, 'cmorfiles',proj, 'EC-Earth-Consortium',
                   opt_Earth3, opt_simu, memb, table, 
                   var, grid, ver)
  
  filename <- paste0(var, '_', table, '_', opt_Earth3, '_', opt_simu, 
                     '_', sdate, 
                     memb, '_', grid, '_', 
                     '$period$',
                     '.nc')
  path <- file.path(dir, filename)
  
  #-------------------------------------------
  chunk1 <- paste0(rundate1, '-', as.numeric(rundate1) + 11)
  chunk2 <- paste0(as.numeric(rundate2) - 11, '-', rundate2)
  years <- as.numeric(substr(chunk1, 1, 4)):as.numeric(substr(chunk2, 1, 4))
  chunks <- paste0(years, '01-', years, '12')
  
  #-------------------------------------------
  
  if (cal_choice != 'bias') {
    
    #--------Calculation function------------
    if (cal_choice == "clim" | cal_choice == "ano") {
      
      if (cal_term == "mth") {
        
        fun_cal <- function(x) {  #in: ['time', 'period']; out: ['time']
          a <- apply(x, c(which(names(dim(x)) != 'period')), mean)
          dim(a) <- c(time = length(a))
          return(a)
        }
        
      } else if (cal_term == "yr") {
        
        fun_cal <- function(x) { #in: ['var', 'time', 'period']; out: ['var', 'time']
          a <- apply(x, c(which(names(dim(x)) != 'period' & names(dim(x)) != 'time')), mean)
          dim(a) <- c(var = length(a))
          a <- Enlarge(a, 2)
          names(dim(a))[2] <- 'time'
          return(a)
        }
        
      }
      
    } else if (cal_choice == "std") {
      
      if (cal_term == "mth") {
        
        fun_cal <- function(x) {
          a <- apply(x, c(which(names(dim(x)) != 'period')), sd)
          dim(a) <- c(time = length(a))
          return(a)
        }
        
      } else if (cal_term == "yr") {
        
        fun_cal <- function(x) { #in: ['var', 'time', 'period']; out: ['var', 'time']
          a <- apply(x, c(which(names(dim(x)) != 'period' & names(dim(x)) != 'time')), mean)
          dim(a) <- c(var = length(a))
          a <- Enlarge(a, 2)
          names(dim(a))[2] <- 'time'
          return(a)
        }
        
      }
    }
    
    #-----------Start------------
    if (cal_choice == "none") {
      
      res <- NULL
      
    } else if (cal_choice == "clim" | cal_choice == "ano" | cal_choice == "std") {
      
      if (type$dim == 'latlon' | type$dim == 'ij') {
        
        if (type$dim == 'latlon') {
          
          data <- Start(dat = path,
                        var = var,
                        time = 'all',
                        period = values(chunks),  #'all',
                        lat = 'all',
                        lon = 'all',
                        return_vars = list(lon = NULL, lat = NULL),
                        synonims = list(lat = c('lat','latitude'),
                                        lon = c('lon','longitude')),
                        retrieve = FALSE)
          
        } else if (type$dim == 'ij') {
          
          lon_name <- 'lon'
          if ('longitude' %in% type$var_list) {
            lon_name <- 'longitude'
          }
          lat_name <- 'lat'
          if ('latitude' %in% type$var_list) {
            lat_name <- 'latitude'
          }
          
          data <- Start(dat = path,
                        period = values(chunks),  #'all',
                        var = var,
                        time = 'all',
                        j = indices(1 : type$length_j),
                        i = indices(1 : type$length_i),
                        j_var = lat_name,
                        i_var = lon_name,
                        return_vars = list(lat_name = NULL, lon_name = NULL),
                        retrieve = FALSE)
        }
        
        if (cal_term == "mth") {
          step <- Step(fun_cal,
                       target_dims = c('time', 'period'),
                       output_dims = c('time'))
        } else if (cal_term == "yr") {
          step <- Step(fun_cal,
                       target_dims = c('var', 'time', 'period'),
                       output_dims = c('var', 'time'))
        }
        
        wf <- AddStep(data, step)
        
        if (comp == 'fatnode') {
          
          # initial.ok <- 0
          
          #fatnode
          message("start Compute() on fatnodes")
          start_time <- Sys.time()
          
          res <- Compute(wf,
                         chunks = list(lat = fun_chunk(data)[1],
                                       lon = fun_chunk(data)[2]),#$output1
                         threads_load = 2,
                         threads_compute = 4,
                         cluster = list(queue_host = 'bsceslogin01.bsc.es',
                                        queue_type = 'slurm',
                                        cores_per_job = 2,
                                        job_wallclock = '00:10:00',
                                        max_jobs = 4,
                                        bidirectional = TRUE
                         ),
                         ecflow_suite_dir = '/home/Earth/aho/startR_local/',
                         wait = TRUE
          )$output1
          
          # if (initial.ok < input$ok) {
          #   initial.ok <<- initial.ok + 1
          #   stop("Interrupted")
          # }
          
          end_time <- Sys.time()
          time_reorder <- end_time - start_time
          message('The consuming time of Compute() on fatnodes: ', time_reorder)
          
        } else if (comp == 'ws') {
          
          #local
          message("start Compute() on ws")
          start_time <- Sys.time()
          
          res <- Compute(wf,
                         chunks = list(lat = fun_chunk(data)[1],
                                       lon = fun_chunk(data)[2])
          )$output1
          
          end_time <- Sys.time()
          time_reorder <- end_time - start_time
          message('The consuming time of Compute() on ws: ', time_reorder)
          
        } else if (comp == 'p9') {
          
          #Power9
          message("start Compute() on P9")
          start_time <- Sys.time()
          res <- Compute(wf,
                         chunks = list(lat = fun_chunk(data)[1],
                                       lon = fun_chunk(data)[2]),
                         threads_load = 2,
                         threads_compute = 4,
                         cluster = list(queue_host = 'p1',
                                        queue_type = 'slurm',
                                        temp_dir = '/gpfs/scratch/bsc32/bsc32734/startR_hpc/',
                                        #lib_dir = '/gpfs/projects/bsc32/share/R_libs/3.5/',
                                        r_module = 'R/3.5.0-foss-2018b',
                                        #r_module = 'startR/0.1.2-foss-2018b-R-3.5.0',
                                        job_wallclock = '00:10:00',
                                        cores_per_job = 4,
                                        max_jobs = 4,
                                        bidirectional = FALSE,
                                        polling_period = 50
                         ),
                         ecflow_suite_dir = '/home/Earth/aho/startR_local/',
                         wait = TRUE
          )$output1
          
          end_time <- Sys.time()
          time_reorder <- end_time - start_time
          message('The consuming time of Compute() on P9: ', time_reorder)
        }
        
        
        #dim_order <- names(dim(res))
        print(paste0('check!!! ', names(dim(res))))
        if (cal_term == "mth") {
          res <- multiApply:::.aperm2(res, c(2, 3, 1, 4, 5))  #dat var time lat lon
        } else if (cal_term == 'yr') {
          res <- multiApply:::.aperm2(res, c(3, 1, 2, 4, 5))  #dat var time lat lon
        }
        
      } else if (type$dim == 'time' | type$dim == 'time_region') {  # if latlon or ij
        
        if (type$dim == 'time') {
        
        data <- Start(dat = path,
                      period = values(chunks),
                      var = var,
                      time = 'all',
                      time_across = 'period',
                      #merge_across_dims = TRUE,
                      #region = 'all',
                      retrieve = FALSE)
        
        } else if (type$dim == 'time_region') {
          
          data <- Start(dat = path,
                        period = values(chunks),
                        var = var,
                        time = 'all',
                        time_across = 'period',
                        #merge_across_dims = TRUE,
                        region = 'all',
                        retrieve = FALSE)
        }
  
        if (cal_term == "mth") {
          step <- Step(fun_cal,
                       target_dims = c('time', 'period'),
                       output_dims = c('time'))
        } else if (cal_term == "yr") {
          step <- Step(fun_cal,
                       target_dims = c('var', 'time', 'period'),
                       output_dims = c('var', 'time'))
        }
        
        wf <- AddStep(data, step)
        
        if (comp == 'fatnode') {
          #fatnode
          res <- Compute(wf,
                         # chunks = list(lat = fun_chunk(data)[1],
                         #               lon = fun_chunk(data)[2]),#$output1
                         threads_load = 2,
                         threads_compute = 4,
                         cluster = list(queue_host = 'bsceslogin01.bsc.es',
                                        queue_type = 'slurm',
                                        cores_per_job = 2,
                                        job_wallclock = '00:10:00',
                                        max_jobs = 4,
                                        bidirectional = TRUE
                         ),
                         ecflow_suite_dir = '/home/Earth/aho/startR_local/',
                         wait = TRUE
          )$output1
          
        } else if (comp == 'ws') {
          
          #local
          res <- Compute(wf)$output1
          
        } else if (comp == 'p9') {
          
          #Power9
          res <- Compute(wf,
                         # chunks = list(lat = fun_chunk(data)[1],
                         #               lon = fun_chunk(data)[2]),
                         threads_load = 2,
                         threads_compute = 4,
                         cluster = list(queue_host = 'p1',
                                        queue_type = 'slurm',
                                        temp_dir = '/gpfs/scratch/bsc32/bsc32734/startR_hpc/',
                                        #lib_dir = '/gpfs/projects/bsc32/share/R_libs/3.5/',
                                        r_module = 'R/3.5.0-foss-2018b',
                                        #r_module = 'startR/0.1.2-foss-2018b-R-3.5.0',
                                        job_wallclock = '00:10:00',
                                        cores_per_job = 4,
                                        max_jobs = 4,
                                        bidirectional = FALSE,
                                        polling_period = 50
                         ),
                         ecflow_suite_dir = '/home/Earth/aho/startR_local/',
                         wait = TRUE
          )$output1
          
        }
        
        #dim_order <- names(dim(res))
        print('res=')
        print(dim(res))
        if (cal_term == "mth") {
          res <- multiApply:::.aperm2(res, c(2, 3, 1))  #dat var time
        } else if (cal_term == 'yr') {
          res <- multiApply:::.aperm2(res, c(3, 1, 2))  #dat var time
        }
        
        
      }  #type$dim == 'time'
      #clim finished here
      
      if (cal_choice == "ano") {
        clim_output <- res
        dup_array <- clim_output
        
        if (cal_term == "mth") {
          rep_time <- dim(dataInput)['time']/dim(clim_output)['time'] - 1
        } else if (cal_term == "yr") {
          if (type$dim == 'latlon' | type$dim == 'ij') {
            rep_time <- 11
          } else {
            rep_time <- dim(dataInput)['time']/dim(clim_output)['time'] - 1
          }
        }
        
        print(paste('rep_time=', rep_time))
        print(paste('clim_output=', dim(clim_output)))
        
        if (rep_time != 0) {
          for (n in 1: rep_time) {
            dup_array <- abind(dup_array, clim_output, along = which(names(dim(clim_output)) == 'time'))
          }
        }
        print('dup_array=')
        print(dim(dup_array))
        print('dataInput=')
        print(dim(dataInput))
        res <- dataInput - dup_array
      }
      #clim and ano end
    } 
    
  } else if (cal_choice == 'bias') {
    
    #exp data
    #type <- isolate(typeInput())
    
    ##obs/recon data
    #era40-erainterim/psl
    dir <- file.path('/esarchive/recon/ecmwf',
                     paste0('era40-erainterim/monthly_mean/', var, '_f6h'))
    filename <- paste0(var, '_', '$YEAR$$MONTH$.nc')
    
    ##ecmwf/orap5/tos (irregular grid, doesn't work)
    # dir <- file.path('/esarchive/recon/ecmwf',
    #                  paste0('orap5/monthly_mean/', var))
    # filename <- paste0(var, '_', '$YEAR$$MONTH$$DAY$.nc')
    
    path_obs <- file.path(dir, filename)
    
    if (input$sdate != '') {
      exp_path <- gsub(input$sdate, 's$START_DATE$-', type$path)
    } else {
      exp_path <- gsub(input$date1, '$YEAR$$MONTH$', type$path)
    }
    
    exp_list <- list(name = input$exp,
                     path = exp_path)
    obs_list <- list(name = 'obs',
                     path = path_obs)
    
    data_both <- Load(var,
                      exp = list(exp_list), obs = list(obs_list),
                      sdates = paste0(input$date1,'01'),
                      output = 'lonlat')
    
    dat_exp <- data_both$mod
    dat_obs <- data_both$obs
    
    res <- dat_exp - dat_obs
    names(dim(res)) <- names(dim(dat_exp))
    
  }  # else if == 'bias'
  
  print('res=')
  print(dim(res))
  print(head(res))
  
  res  #return result of all possibilities
}
