library(shiny)
library(shinyjs)
library(s2dverification)
library(multiApply)
library(startR)
library(ggplot2)
library(plyr)
library(abind)

source('func_type.R')
source('func_forward.R')
source('func_plot.R')
source('func_cal.R')
source('func_timestep.R')

shinyServer(function(input, output, session) {
  
  shinyjs::enable('reset')
  shinyjs::enable('playAni')
  
  observe({
    #both work, either put inside observe() or outside like above.
    #shinyjs::toggleState("playAni", !is.null(input$var) && input$var != "")
    shinyjs::toggleState("nextPlot", !is.null(input$var) && input$var != "")
  })
  
  
  #------------------dir finding-----------------------
  #filter out the workable exp ID
  output$exp <- renderUI({
    asd <- list.files(file.path('/esarchive/exp/ecearth/'))
    
    #remove the ones without cmorfiles/
    for (i in 1:length(asd)) {
      sdf <- list.files(file.path('/esarchive/exp/ecearth/', asd[i]))
      if (!'cmorfiles' %in% sdf) asd[i] <- 'remove'
    }
    asd <- asd[!asd %in% 'remove']
    
    #remove the ones without EC-Earth-Consortium
    for (i in 1:length(asd)) {
      sdf <- list.files(file.path('/esarchive/exp/ecearth/', asd[i], 'cmorfiles'))  #*MIP
      for (j in 1:length(sdf)) {
        qwe <- list.files(file.path('/esarchive/exp/ecearth/', asd[i], 'cmorfiles',
                                    sdf[j]))
        
        if (!'EC-Earth-Consortium' %in% qwe) sdf[j] <- 'remove'
      }
      if (all(sdf == 'remove')) asd[i] <- 'remove'
    }
    asd <- asd[!asd %in% 'remove']
    
    selectizeInput("exp", "Experiment ID:",
                   choices = asd,
                   selected = 'a1tr')
    
  })  #renderUI, output$exp
  
  #get *MIP
  observeEvent(input$exp, {
    
    opt_MIP <- list.files(file.path('/esarchive/exp/ecearth',
                                    input$exp, 'cmorfiles/'))
    if(length(opt_MIP) == 1) {
      output$proj <- renderUI(
        disabled(textInput("proj", "Project:", opt_MIP))
      )
    } else {
      output$proj <- renderUI(
        
        selectizeInput(inputId = "proj", label = "Project:",
                       choices = opt_MIP,
                       selected = opt_MIP[1])
      )
    }
  })
  
  #get the memb list under exp
  observeEvent({
    input$exp
    input$proj}, {
      
      #EC-Earth3*
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      #simulation (e.g., historical)
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      #member
      opt_memb <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3, opt_simu))
      output$memb <- renderUI(
        disabled(textInput("memb", "Member ID:", opt_memb))
      )
    })
  
  #get the table list under memb
  observeEvent({
    input$exp
    input$proj
    input$memb}, {
      
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      opt_table <- list.files(file.path('/esarchive/exp/ecearth',
                                        input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                        opt_Earth3, opt_simu, input$memb))
      output$table <- renderUI(
        selectizeInput("table", "Model realm:",
                       choices = opt_table,
                       selected = 'Amon')
      )
    })
  
  
  
  
  #get the variables under table
  observeEvent({
    input$exp
    input$proj
    input$memb
    input$table}, {
      
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      opt_var <- list.files(file.path('/esarchive/exp/ecearth',
                                      input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                      opt_Earth3, opt_simu, input$memb, input$table))
      output$var <- renderUI(
        selectizeInput("var", "Variable:",
                       choices = opt_var,
                       selected = opt_var[1])
      )
    })
  
  
  
  
  #get the grid type under variable
  observeEvent({
    input$exp
    input$proj
    input$memb
    input$table
    input$var}, {
      
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      opt_grid <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3, opt_simu, input$memb, input$table, input$var))
      
      if(length(opt_grid) == 1) {
        output$grid <- renderUI(
          disabled(textInput("grid", "Grid type:", opt_grid))
        )
      } else {
        output$grid <- renderUI(
          selectizeInput("grid", "Grid type:",
                         choices = opt_grid,
                         selected = opt_grid[1])
        )
      }
    })
  
  #get v*
  observeEvent({
    input$exp
    input$proj
    input$memb
    input$table
    input$var
    input$grid}, {
      
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      opt_ver <- list.files(file.path('/esarchive/exp/ecearth',
                                      input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                      opt_Earth3, opt_simu, input$memb, input$table, 
                                      input$var, input$grid))
      output$ver <- renderUI(
        disabled(textInput("ver", 'Version:', opt_ver))
      )
    })
  
  #-------end of dir finding------------
  
  #---------get start date (e.g., s19900101) if there is-------------
  observeEvent({
    input$exp
    input$proj
    input$memb
    input$table
    input$var
    input$grid
    input$ver}, {
      
      #get file names
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      opt <- list.files(file.path('/esarchive/exp/ecearth',
                                  input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                  opt_Earth3, opt_simu, input$memb, input$table, 
                                  input$var, input$grid, input$ver))
      if (grepl('*_*_*_*_s*-*', opt[1])) {
        opt_sdate <- sapply(strsplit(opt, opt_simu), "[", 2)
        opt_sdate <- sapply(strsplit(opt_sdate, input$memb), "[", 1)  #_s19900101-
        opt_sdate <- unique(substr(opt_sdate, 2, 11))  #s19900101- 
      }
      output$sdate <- renderUI(
        disabled(textInput("sdate", 'Simulation start date:', opt_sdate))
      )
    })
  
  substrRight <- function(x, n) {
    substr(x, nchar(x)-n+1, nchar(x))
  }
  
  #get whole chunk period
  observeEvent({
    input$exp
    input$proj
    input$memb
    input$table
    input$var
    input$grid
    input$ver
    input$sdate}, {
      
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      opt <- list.files(file.path('/esarchive/exp/ecearth',
                                  input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                  opt_Earth3, opt_simu, input$memb, input$table, 
                                  input$var, input$grid, input$ver))
      #first chunk
      asd1 <- substrRight(opt, 16)
      qwe1 <- substr(asd1, 1, 6)
      #last chunk
      asd2 <- substrRight(opt, 9)
      qwe2 <- substr(asd2, 1, 6)
      
      #output$date1 <- renderUI(
      output$date1 <- renderText(paste(qwe1[1], '-', tail(qwe2, 1)))
      #output$date1 <- renderText(paste('Whole chunk period:', qwe1[1], '-', tail(qwe2, 1)))
      #)
    })
  
  #set animation run start date
  observeEvent({
    input$exp
    input$proj
    input$memb
    input$table
    input$var
    input$grid
    input$ver
    input$sdate}, {
      
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      opt <- list.files(file.path('/esarchive/exp/ecearth',
                                  input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                  opt_Earth3, opt_simu, input$memb, input$table, 
                                  input$var, input$grid, input$ver))
      asd <- substrRight(opt, 16)
      qwe <- substr(asd, 1, 6)
      
      output$rundate1 <- renderUI(
        selectizeInput("rundate1", "Run animation/Calculate climatology from:",
                       choices = unique(qwe),
                       selected = qwe[1])
      )
    })
  
  #in order to tell pathInput which file to read
  num <- reactive({
    input$cal
    input$comp
    input$term
    opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
    opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                     input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                     opt_Earth3))
    opt <- list.files(file.path('/esarchive/exp/ecearth',
                                input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                opt_Earth3, opt_simu, input$memb, input$table,
                                input$var, input$grid, input$ver))
    asd <- substrRight(opt, 16)
    qwe <- substr(asd, 1, 4)
    as.numeric(substr(input$rundate1, 1, 4)) - as.numeric(qwe[1]) + session$i
  })
  
  #set animation run end date
  observeEvent({
    input$exp
    input$proj
    input$memb
    input$table
    input$var
    input$grid
    input$ver
    input$sdate
    input$rundate1}, {
      
      opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                         input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
      opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                       opt_Earth3))
      opt <- list.files(file.path('/esarchive/exp/ecearth',
                                  input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                  opt_Earth3, opt_simu, input$memb, input$table, 
                                  input$var, input$grid, input$ver))
      asd <- substrRight(opt, 9)
      qwe <- substr(asd, 1, 6)
      qwe <- qwe[as.integer(qwe) >= as.integer(input$rundate1)]
      
      output$rundate2 <- renderUI(
        selectizeInput("rundate2", "Run animation/Calculate climatology to:",
                       choices = unique(qwe),
                       selected = tail(qwe, 1))
      )
    })
  
  #----------------filename set----------------
  
  #--------------reactiveValues--------------
  session <- reactiveValues()
  if (!exists('session$i')) session$i <- 1
  if (!exists('session$ii')) session$ii <- 0
  if (!exists('session$timer')) session$timer <- 1000
  if (!exists('session$run')) session$run <- FALSE

  observe({
      session$timer <- reactiveTimer(as.numeric(input$freq)*1000)
  })
  
  #---------reset time step when changing data--------------
  observeEvent({
    input$reset
    input$exp
    input$proj
    input$memb
    input$table
    input$var
    input$grid
    input$ver
    input$sdate
    input$rundate1
    input$rundate2
    input$cal
    input$term}, {
      
      session$i <- 1
      session$ii <- 0
      
    })
  
  #-------------reactive and function part---------------
  
  pathInput <- reactive({
    print('in pathInput')
    
    opt_Earth3 <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/'))
    opt_simu <- list.files(file.path('/esarchive/exp/ecearth',
                                     input$exp, 'cmorfiles/',input$proj, '/EC-Earth-Consortium/',
                                     opt_Earth3))
    dir <- file.path('/esarchive/exp/ecearth',
                     input$exp, 'cmorfiles',input$proj, 'EC-Earth-Consortium',
                     opt_Earth3, opt_simu, input$memb, input$table,
                     input$var, input$grid, input$ver)
    
    files <- list.files(dir) 
    filename <- files[num()]
    
    path <- file.path(dir, filename)
    path
  })
  
  typeInput <- reactive({
    func_type(pathInput(), input$var)    
  })  #typeInput
  
  
  output$bigdimvar <- reactive({
    
    type <- typeInput()
    asd <- ""
    if (type$length > 4) {
      asd <- paste0("The variable has additional dimension besides lat, lon, and time. ",
                    "Unable to plot.")
      #shinyjs::disable('playAni')
      #shinyjs::disable('nextPlot')
    }
    if (type$latlondim > 2) {
      asd <- paste0("The grid type of this variable is irregualr. ",
                    "Unable to plot.")
    }
    if (type$dim == 'time_region' & type$length > 3) {
      asd <- "The data format is not available to plot by this app."
    }
    if (type$dim == 'time' & type$length > 2) {
      asd <- "The data format is not available to plot by this app."
    }
    
    asd
  })
  
  
  #-------Read in single original file---------
  dataInput <- reactive({
    print('in dataInput')
    
    type <- typeInput()
    
    if(type$dim == 'latlon') {
      
      data <- Start(dat = type$path,
                    var = input$var,
                    time = 'all',
                    lat = 'all',
                    lon = 'all',
                    return_vars = list(lon = NULL, lat = NULL),
                    synonims = list(lat = c('lat','latitude'),
                                    lon = c('lon','longitude')),
                    retrieve = TRUE)
      
    } else if (type$dim == 'ij') {
      
      lon_name <- 'lon'
      if ('longitude' %in% type$var_list) {
        lon_name <- 'longitude'
      }
      lat_name <- 'lat'
      if ('latitude' %in% type$var_list) {
        lat_name <- 'latitude'
      }
      
      data <- Start(dat = type$path,
                    var = input$var,
                    time = 'all',
                    j = indices(1 : type$length_j),
                    i = indices(1 : type$length_i),
                    j_var = lat_name,
                    i_var = lon_name,
                    #return_vars = list(lat_name = NULL, lon_name = NULL),
                    retrieve = TRUE)
      
      
    } else if (type$dim == 'time') {
      
      chunk1 <- paste0(input$rundate1, '-', as.numeric(input$rundate1) + 11)
      chunk2 <- paste0(as.numeric(input$rundate2) - 11, '-', input$rundate2)
      years <- as.numeric(substr(chunk1, 1, 4)):as.numeric(substr(chunk2, 1, 4))
      chunks <- paste0(years, '01-', years, '12')
      
      #re-make path, read in all periods
      tmp <- gregexpr(pattern = input$grid, type$path)[[1]]
      new_path <- paste0(substr(type$path, 1, tmp[length(tmp)] + 2), '$periods$.nc')
      print(paste('new path: ', new_path))
      data <- Start(dat = new_path,
                    var = input$var,
                    periods = values(chunks),  #'all',
                    time = 'all',
                    #region = 'all',
                    time_across = 'periods',
                    merge_across_dims = TRUE,
                    retrieve = TRUE)
      print(dim(data))
      
    } else if (type$dim == 'time_region') {
      
      #re-make path, read in all periods
      tmp <- gregexpr(pattern = input$grid, type$path)[[1]]
      new_path <- paste0(substr(type$path, 1, tmp[length(tmp)] + 2), '$periods$.nc')
      print(paste('new path: ', new_path))
      data <- Start(dat = new_path,
                    var = input$var,
                    periods = 'all',
                    time = 'all',
                    region = 'all',
                    time_across = 'periods',
                    merge_across_dims = TRUE,
                    retrieve = TRUE)
    }
    
    data   #return single original data
    
  })  #dataInput <- reactive
  
  #-----------------------Calculation-------------------------
  CalInput <- reactive({
    func_cal(input$exp, input$proj, input$memb, input$table, input$grid, input$ver, input$sdate, 
             typeInput(), input$cal, input$var, dataInput(), input$comp, input$rundate1, input$rundate2, input$term)
  })  #CalInput
  
  tt <- reactive({
    print('in tt')
    if (input$cal == 'none') {
      yr <- as.character(as.integer(substr(input$rundate1, 1, 4)) + session$i - 1)
      paste(input$exp, input$var, yr)
    } else if (input$cal == 'clim' | input$cal == 'std') {
      paste(input$exp, input$var, input$cal, input$rundate1, '-', input$rundate2)
    } else if (input$cal == 'ano' | input$cal == 'bias') {
      paste(input$exp, input$var, input$cal, 'based on', input$rundate1, '-', input$rundate2)
    }
    
  })
  
  #----------------reactive and function over---------------  
  
  data_use <- eventReactive(num(), {
    print('data_use eventReactive')
    isolate(dataInput())
  })
  cal_use <- eventReactive(num(), {
    print('cal_use eventReactive')

    cal <- isolate(CalInput())
    if(is.null(cal)) {
      data <- isolate(dataInput())
      cal <- data
    }
    cal
  })
  t_use <- eventReactive(num(), {
    print('t_use eventReactive')
    isolate(tt())
  })  

  
  #---------------nextPlot--------------------
  observeEvent(input$nextPlot,{
    
    shinyjs::enable('playAni')
    shinyjs::enable('stopAni')
    
    #---------change i and ii for the next plot--------------
    session$i <- func_timestep(cal_use(), input$cal, session$ii, session$i, input$rundate1, input$rundate2, input$term)[[1]]
    session$ii <- func_timestep(cal_use(), input$cal, session$ii, session$i, input$rundate1, input$rundate2, input$term)[[2]]
    
    
    #------------a list for plot input-----------
    plot_arg <- func_forward(cal_use(), t_use(), input$cal, input$var, data_use())
    
    output$map1 <- renderPlot({
      print('renderPlot')
      print('ii =')
      print(session$ii)
      print('i =')
      print(session$i)
      
      if (plot_arg[[2]]) {  #is_map

        func_plot(plot_arg[[1]], plot_arg[[2]], plot_arg[[3]], plot_arg[[4]], plot_arg[[5]], plot_arg[[6]],
                  plot_arg[[7]], plot_arg[[8]], session$ii, session$i, input$rundate1, input$rundate2, input$cal, input$term)[[1]]
        
      } else {  #time series
        func_plot(plot_arg[[1]], plot_arg[[2]], plot_arg[[3]], plot_arg[[4]], plot_arg[[5]], plot_arg[[6]],
                  plot_arg[[7]], plot_arg[[8]], session$ii, session$i, input$rundate1, input$rundate2, input$cal, input$term)#[[1]]
        
      }
    },
    height = 400*(log(6)/log(1 + 5)), width = 600
    )
    
  })  #observeEvent(input$nextPlot
  
  
  #---------------anim--play or stop-----------------
  observeEvent(input$playAni,{
    
    shinyjs::disable('playAni')
    shinyjs::disable('nextPlot')
    shinyjs::enable('stopAni')
    
    session$run <- TRUE
  }) 
  
  observeEvent(input$stopAni, {
    
    shinyjs::disable('stopAni')
    shinyjs::enable('playAni')
    shinyjs::enable('nextPlot')
    
    session$run <- FALSE
  })
  

  observeEvent(session$timer(), {
    
    if (isolate(session$run)) {
      
      #---------change i and ii for the next plot--------------
      session$i <- func_timestep(cal_use(), input$cal, session$ii, session$i, input$rundate1, input$rundate2, input$term)[[1]]
      session$ii <- func_timestep(cal_use(), input$cal, session$ii, session$i, input$rundate1, input$rundate2, input$term)[[2]]
      
      #---------a list for plot input------------
      plot_arg <- func_forward(cal_use(), t_use(), input$cal, input$var, data_use())
      
      output$map1 <- renderPlot({
        print('renderPlot')
        print('ii =')
        print(session$ii)
        
        if (plot_arg[[2]]) {  #is_map
          
          func_plot(plot_arg[[1]], plot_arg[[2]], plot_arg[[3]], plot_arg[[4]], plot_arg[[5]], plot_arg[[6]],
                    plot_arg[[7]], plot_arg[[8]], session$ii, session$i, input$rundate1, input$rundate2, input$cal, input$term)[[1]]
          
          
        } else {  #time series
          func_plot(plot_arg[[1]], plot_arg[[2]], plot_arg[[3]], plot_arg[[4]], plot_arg[[5]], plot_arg[[6]],
                    plot_arg[[7]], plot_arg[[8]], session$ii, session$i, input$rundate1, input$rundate2, input$cal, input$term)#[[1]]
          
        }
      },
      height = 400*(log(6)/log(1 + 5)), width = 600 
      )
    } #if
  }) #observeEvent(timer)
  


  
  #the message below plotting
  output$cal_info <- renderText({
    
    if (input$cal != 'none') {
      paste0("Calculation takes several minutes. ",
             "Don't refresh the page unless you wanna cancel the calculation.")
    } else {
      ""
    }
  })
  
  
})  #all end
