library(shiny)

# Define UI for application

shinyUI(fluidPage(
  shinyjs::useShinyjs(),
  
  # Application title
  titlePanel("ECEARTH experiment visualization"),
  p(paste0("Only experiments with path '/esarchive/exp/ecearth/.../cmorfiles/.../",
           "EC-Earth-Consortium/...' are available.")),
  
  sidebarLayout(
    
    sidebarPanel(
      
      fluidRow(
        column(7, wellPanel(
          uiOutput("exp"),
          uiOutput("proj"),
          uiOutput("table"),
          uiOutput("var"),
          h5(tags$b("Whole chunk period:")),
          tags$i(uiOutput("date1")),
          br(),
          uiOutput("rundate1"),
          uiOutput("rundate2"),
          hr(),
          uiOutput("memb"),
          uiOutput("grid"),
          uiOutput("ver"),
          uiOutput("sdate")
          
          #checkboxInput("allchunks", "Run through all chunks", FALSE)
        )),
        
        column(5, wellPanel(
          radioButtons("cal", "Calculation:",
                       c("None" = "none",
                         "Climatology" = "clim",
                         "Anomaly" = "ano",
                         "Standard deviation" = "std",
                         "Bias (unavailable)" = "bias")),
          conditionalPanel(
            condition = "input.cal != 'none'",
            radioButtons("term", "Term:",
                         c("Monthly" = "mth",
                           "Annual" = "yr")
                         )
          ),
          br(),
          radioButtons("comp", "Compute on:", 
                       c("bscearth200" = "ws",
                         "fatnodes" = "fatnode",
                         "Power 9" = "p9")),
          
          br(),
          h5(tags$b("Plot options:")),
          
          
          h5(tags$em("Animation")),
          actionButton("playAni", "Play",
                       style = "color: #fef9e7; background-color: #f1c40f; border: none),
                       #padding:5px; font-size:100%"),
          actionButton("stopAni", "Stop",
                       style = "color: #fef5e7; background-color: #eb984e; border: none),
                       #padding:6px; font-size:100%"),
          textInput("freq",
                    h6("Frequency (sec):"),
                    value = "1"),
          
          h5(tags$em("One by one")),
          actionButton("nextPlot", "Next time step",
                       style = "color: #fef9e7; background-color: #f1c40f; border: none),
                                #padding: 10px 24px; font-size:100%"),
          
          hr(),
          actionButton("reset", "Reset time step",  
                       style = "color: #e8f8f5; background-color: #48c9b0; border: none),
                                #padding:12px 28px; font-size:100%")
          
        ))
        
      ),
      
      includeScript('shiny.js')
      
    ),
    
    mainPanel(
      verbatimTextOutput('bigdimvar'),
      hr(),
      plotOutput('map1'),
      hr(),
      verbatimTextOutput('cal_info')
      
    )
  ),
  
  #popup box
  ##The "Modal" part:
  div(
    id = 'busyModal', class = 'modal', role = 'dialog', 'data-backdrop' = 'static',
    div(
      class = 'modal-dialog modal-sm',
      
      ##The "Modal content" part:
      div(
        class = 'modal-content',
        div(class = 'modal-header', 
            h5(class = 'modal-title', 'Connecting to the server...')),
        div(class = 'modal-body', 
            p(paste('This dialog box will disappear',
                    'automatically after shiny is ready.')))
      )
    )
  )
  
))
