func_plot <- function(cal, kk, is_map, lons, lats, units, t, brks, cbar, ii, i, rundate1, rundate2, subdata) {
  
  validate(
    need(ii != 0, "Time step has been reset or data has been changed. Click plot button again.")
  )
  
  if (is_map) {
    
    tmp <- cal[1, kk, ii, , ]
    tmp_title <- paste0(t, ' month = ', ii)
    
    PlotEquiMap(tmp,
                lons, lats,
                toptitle = tmp_title,
                brks = brks, color_fun = clim.palette(palette = cbar),
                triangle_ends = c(TRUE, TRUE),
                units = units,
                intylat = 10, intxlon = 10,
                fill = FALSE)
    
  } else {   #is_map = FALSE, time series, #a1tr/thetaosum, tossum, tosmean; a1py/tosmean
    
    
    #library(ggplot2)
    #source('https://earth.bsc.es/gitlab/es/s2dverification/raw/develop-debug-plot-ts/R/PlotTimeSeries.R')
    #turn array into data frame
    
    time_id <- grep('time', names(dim(cal)))
    region_id <- grep('region', names(dim(cal)))
    
    # region_name <- type$regions[region_id]
    data_df <- adply(Subset(cal, 'var', kk), c(time_id, region_id))
    #rename
    n <- length(names(data_df))
    
    type_dim <- subdata[[2]][[1]]$dim
    
    
    if(type_dim == 'time_region') {
      names(data_df)[1] <- 'time'
      names(data_df)[2] <- 'region'
      names(data_df)[3] <- 'var' #input$var
      
      reg_name <- data.frame(    
        region = c(1:length(unique(data_df$region))),    
        reg_name = subdata[[2]][[kk]]$regions)
      # reg_name = type[[kk]]$regions)
      
      data_df <- merge(data_df, reg_name, by.x = "region")
      
    } else {
      names(data_df)[1] <- 'time'
      names(data_df)[2] <- 'var' 
    }
    
    titlename <- paste0(t, ' - ', substr(rundate2, 1, 4))
    
    breaks <- seq(1, (as.numeric(substr(rundate2, 1, 4), 1) - as.numeric(substr(rundate1, 1, 4), 1) + 1) * 12, 12)
    labels <- as.character(rep(seq(as.numeric(substr(rundate1, 1, 4)), as.numeric(substr(rundate2, 1, 4), 1))))
    
    
    if(type_dim == 'time_region') {
      
      ggplot(data_df, aes(as.numeric(time), var, color = reg_name)) + 
        geom_point(alpha = 0.6, size = 0.5) +
        geom_line(size = 0.3) +
        labs(title = paste0(t, ' - ', substr(rundate2, 1, 4)),
             x = 'time', y = units, color = "Region") +
        scale_x_continuous(breaks = breaks, 
                           labels = labels) +
        theme(axis.text.x = element_text(angle = 45, hjust = 1))
      
    } else {
      
      ggplot(data_df, aes(as.numeric(time), var)) +
        geom_point(alpha = 0.6, size = 0.5) +
        geom_line(size = 0.3) +
        labs(title = titlename,
             x = 'time', y = units) +
        scale_x_continuous(breaks = breaks,
                           labels = labels) +
        theme(axis.text.x = element_text(angle = 45, hjust = 1))
    }
    
  }  #is_map = FALSE
  
}  #ends

