library(shiny)
library(shinyjs)
library(s2dverification)
library(startR)
library(ggplot2)
library(plyr)
library(abind)
##library(reticulate)
# 
source('func_type.R')
source('func_forward.R')
source('func_plot.R')
source('func_cal.R')
source('func_timestep.R')

shinyServer(function(input, output, session) {
  
  # shinyjs::enable('reset')
  # shinyjs::enable('playAni')
  
  #only original data
  shinyjs::disable("comp")
  shinyjs::disable("cal")
  
  
  observe({
    #both work, either put inside observe() or outside like above.
    #shinyjs::toggleState("playAni", !is.null(input$var) && input$var != "")
    shinyjs::toggleState("nextPlot", !is.null(input$var_grid) && input$var_grid != "")
  })
  
  
  #------------------dir finding-----------------------
  
  #------NEW!!! Choose between diags or cmorfiles/-------
  cmordiags <- reactive({
    if (input$cmordiags == 'cmorfiles') {
      cmordiags <- 'cmorfiles'
    } else {
      cmordiags <- 'diags'
    }
    cmordiags
  })
  
  
  #-------filter out the workable exp ID-----------
  observeEvent(input$cmordiags,{
    
    output$exp <- renderUI({
      
      asd <- list.files(file.path('/esarchive/exp/ecearth/'))  #all the exp name
      # create a list of lists recording the full path for each exp
      asd_fullpath <- lapply(asd, function(x) paste0('/esarchive/exp/ecearth/', x, '/'))
      
      #------remove the ones without cmordiags/-------
      
      # a list of lists containing all folders under each exp
      qwe <- lapply(asd_fullpath, function(x) list.files(x))
      # assign the exp which contains cmordiags 1, without cmordiags 0. qwe is a vector
      qwe <- sapply(qwe, function(x) {if (!(cmordiags() %in% x)) {x <- 0} else {x <- 1} })
      # only keep the exp which contains cmordiags 
      asd <- asd[which(qwe == 1)]
      # create a list of lists recording the full path for each exp
      asd_fullpath <- asd_fullpath[which(qwe == 1)]
      
      
      
      #-----remove the ones without EC-Earth-Consortium------
      
      # a list of lists containing all folders under each exp/cmordiags
      qwe <- lapply(asd_fullpath, function(x) list.files(file.path(x, cmordiags())))
      qwe <- lapply(1:length(asd),  function(x) list.files(file.path(asd_fullpath[[x]], cmordiags(), qwe[[x]])))
      qwe <- sapply(qwe, function(x) {if (!'EC-Earth-Consortium' %in% x) {x <- 0} else {x <- 1} })
      asd <- asd[which(qwe == 1)]
      # modify the list of lists recording the full path for each exp
      asd_fullpath <- asd_fullpath[which(qwe == 1)]
      
      # finally, we get the usable exp id!!
      selectizeInput("exp", "Experiment ID:",
                     choices = asd,
                     selected = 'a1tr')
      
    })  #renderUI, output$exp
  })  #  observeEvent(input$cmordiags,{
  
  
  #-------get *MIP-------
  observeEvent({
    input$cmordiags
    input$exp}, {
      
      opt_MIP <- list.files(file.path('/esarchive/exp/ecearth',
                                      input$exp, cmordiags()))
      if(length(opt_MIP) == 1) {
        output$proj <- renderUI(
          disabled(textInput("proj", "Project:", opt_MIP))
        )
      } else {
        output$proj <- renderUI(
          
          selectizeInput(inputId = "proj", label = "Project:",
                         choices = opt_MIP,
                         selected = opt_MIP[1])
        )
      }
    })
  
  
  #-----get EC-Earth3*-----
  opt_Earth3 <- eventReactive({
    cmordiags()
    input$exp
    input$proj}, {
      #EC-Earth3*
      list.files(file.path('/esarchive/exp/ecearth',
                           input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium'))
    })  
  
  #-----get simulation-----
  opt_simu <- eventReactive({
    cmordiags()
    input$exp
    input$proj
    opt_Earth3()}, {
      #simulation (e.g., historical)
      list.files(file.path('/esarchive/exp/ecearth',
                           input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                           opt_Earth3()))
    })  
  
  
  #-----get version-----
  opt_ver <- eventReactive({
    cmordiags()
    input$exp
    input$proj
    opt_Earth3()
    opt_simu()
    input$memb
    input$table
    var()
    grid()}, {
      # get the version
      tail(
        list.files(file.path('/esarchive/exp/ecearth',
                             input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                             opt_Earth3(), opt_simu(), input$memb, input$table, 
                             var()[1], grid()[1])),
        n = 1)  #assume all files in this exp has same ver; use first var as represntation; choose the newest ver
      
    })  
  
  
  
  #-----get the memb list under exp-----
  observeEvent({
    input$cmordiags
    input$exp
    input$proj}, {
      
      #member
      opt_memb <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                                       opt_Earth3(), opt_simu()))
      
      if(length(opt_memb) == 1) {
        output$memb <- renderUI(
          disabled(textInput("memb", "Member ID:", opt_memb))
        )
      } else {
        output$memb <- renderUI(
          
          selectizeInput(inputId = "memb", label = "Member ID:",
                         choices = opt_memb,
                         selected = opt_memb[1]))
      }
      
    })
  
  #------get the table list under memb--------
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb}, {
      
      opt_table <- list.files(file.path('/esarchive/exp/ecearth',
                                        input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                                        opt_Earth3(), opt_simu(), input$memb))
      
      output$table <- renderUI(
        selectizeInput("table", "Model realm:",
                       choices = opt_table,
                       selected = 'Omon'))
    })
  
  
  #------get the var/grid under table---------
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table}, {
      
      opt_var <- list.files(file.path('/esarchive/exp/ecearth',
                                      input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                                      opt_Earth3(), opt_simu(), input$memb, input$table))
      path <- file.path('/esarchive/exp/ecearth',
                        input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                        opt_Earth3(), opt_simu(), input$memb, input$table)
      
      grid <- lapply(file.path(path, opt_var), list.files)
      
      var_grid <- unlist(lapply(1:length(opt_var), function(x) file.path(opt_var[x], grid[[x]])))
      
      output$var_grid <- renderUI(
        checkboxGroupInput("var_grid", "var/grid:",
                           choices = var_grid,
                           selected = var_grid[2:3])
      )

    })
  
  #-----separate var and grid from input$var_grid-----
  var <- eventReactive(input$var_grid, {
    
    tmp <- lapply(input$var_grid, function(x) strsplit(x, '/'))
    var <- unlist(lapply(lapply(tmp, '[[',1), '[[', 1))
    var
    
  })
  
  grid <- eventReactive(input$var_grid, {
    
    tmp <- lapply(input$var_grid, function(x) strsplit(x, '/'))
    grid <- unlist(lapply(lapply(tmp, '[[',1), '[[', 2))
    grid
    
  })
 
  
  #-------end of dir finding------------
  
  
  substrRight <- function(x, n) {
    substr(x, nchar(x)- n + 1, nchar(x))
  }
  
  #-------get whole chunk period---------
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    var()
    grid()}, {
      
      opt <- list.files(file.path('/esarchive/exp/ecearth',
                                  input$exp, isolate(cmordiags()), input$proj, 'EC-Earth-Consortium',
                                  opt_Earth3(), opt_simu(), input$memb, input$table, 
                                  var()[1], grid()[1], opt_ver())) #get all files
      
      #first chunk
      opt_1 <- opt[1] 
      opt_1 <- substrRight(opt_1, 16)
      opt_1 <- substr(opt_1, 1, 6)
      #last chunk
      opt_2 <- opt[length(opt)]
      opt_2 <- substrRight(opt_2, 9)
      opt_2 <- substr(opt_2, 1, 6)
      
      output$date1 <- renderText(paste(opt_1, '-', opt_2))
      
    })
  
  #set animation run start date
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    var()
    grid()}, {
      
      opt <- list.files(file.path('/esarchive/exp/ecearth',
                                  input$exp, isolate(cmordiags()), input$proj, 'EC-Earth-Consortium',
                                  opt_Earth3(), opt_simu(), input$memb, input$table, 
                                  var()[1], grid()[1], opt_ver())) 
      asd <- substrRight(opt, 16)
      qwe <- substr(asd, 1, 6)
      
      output$rundate1 <- renderUI(
        selectizeInput("rundate1", "Time period starts from:",
                       choices = unique(qwe),
                       selected = qwe[1])
      )
    })
  
  #in order to tell pathInput which file to read
  num <- reactive({
    # input$cal
    # input$comp
    # input$term
    
    opt <- list.files(file.path('/esarchive/exp/ecearth',
                                input$exp, isolate(cmordiags()), input$proj, 'EC-Earth-Consortium',
                                opt_Earth3(), opt_simu(), input$memb, input$table,
                                var()[1], grid()[1], opt_ver()))[1]  #beginning of the whole chunk
    asd <- substrRight(opt, 16)
    qwe <- substr(asd, 1, 4)
    as.numeric(substr(input$rundate1, 1, 4)) - as.numeric(qwe) + session$i
  })
  
  #set animation run end date
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    var()
    grid()
    input$rundate1}, {
      
      opt <- list.files(file.path('/esarchive/exp/ecearth',
                                  input$exp, isolate(cmordiags()), input$proj, 'EC-Earth-Consortium',
                                  opt_Earth3(), opt_simu(), input$memb, input$table, 
                                  var()[1], grid()[1], opt_ver()))
      asd <- substrRight(opt, 9)
      qwe <- substr(asd, 1, 6)
      qwe <- qwe[as.integer(qwe) >= as.integer(input$rundate1)]
      
      output$rundate2 <- renderUI(
        selectizeInput("rundate2", "Time period ends at:",
                       choices = unique(qwe),
                       selected = tail(qwe, 1))
      )
    })
  
  #----------------filename set----------------
  
  ###################################################
  
  #--------------reactiveValues--------------
  session <- reactiveValues()
  if (!exists('session$i')) session$i <- 1
  if (!exists('session$ii')) session$ii <- 0
  if (!exists('session$timer')) session$timer <- 1000
  if (!exists('session$run')) session$run <- FALSE
  
  observe({
    session$timer <- reactiveTimer(as.numeric(input$freq)*1000)
  })
  
  #---------reset time step when changing data--------------
  observeEvent({
    input$cmordiags
    input$reset
    input$exp
    input$proj
    input$memb
    input$table
    var()
    grid()
    input$rundate1
    input$rundate2
    input$cal
    input$term}, {
      
      session$i <- 1
      session$ii <- 0
      
    })
  
  #-------------reactive and function part---------------
  
  pathInput <- reactive({
    print('in pathInput')
    
    dir <- lapply(1:length(var()), function(kk) file.path('/esarchive/exp/ecearth',
                                                              input$exp, isolate(cmordiags()),input$proj, 'EC-Earth-Consortium',
                                                              opt_Earth3(), opt_simu(), input$memb, input$table,
                                                              var()[kk], grid()[kk], opt_ver()) )   #BUG: grid should be gr or grid()[1]????
    
    #####
    path <- lapply(1:length(var()), function(kk) file.path(dir[[kk]], list.files(dir[[kk]])[num()]) )
    path <- lapply(1:length(var()), function(kk) gsub(var()[kk], '$var$', path[[kk]]))
    
    path  #lists of a vector of character
  })
  
  
  xx <- reactive({
    paste(input$nextPlot , input$playAni)
  })
  
  typeInput <- eventReactive(xx(), {
    
    validate(
      need(length(grid()) == length(var()), "Grid choices doesn't match correponding variables.")
    )
    
    func_type(pathInput(), var())    
  })  #typeInput, list (var) of list (type$xxx)
  
  
  output$bigdimvar <- eventReactive(typeInput(), {
    
    type <- typeInput()
    asd <- ""
    #if (type$length > 4) {
    if (any(sapply(type, `[[`, 'length')) > 4) {
      asd <- paste0("The variable has additional dimension besides lat, lon, and time. ",
                    "Unable to plot.")
      
    }
    if (any(sapply(type, `[[`, 'latlondim')) > 2) {
      asd <- paste0("The grid type of this variable is irregualr. ",
                    "Unable to plot.")
    }
    
    if (any(sapply(type, `[[`, 'dim') == 'time_region' & sapply(type, `[[`, 'length') > 3)) {
      asd <- "The data format is not available to plot by this app."
    }
    if (any(sapply(type, `[[`, 'dim') == 'time' & sapply(type, `[[`, 'length') > 2)) {
      asd <- "The data format is not available to plot by this app."
    }
    
    asd
  })
  
  #---------NEW!!!---------- classify data into groups---------
    subdata <- eventReactive(xx(), {
    print('in subdata')
    type <- typeInput()
    path <- pathInput()
    
    ind <- c(1:length(var()))
    
    if (length(ind) > 1) {
      for (jj in 1:(length(var()) - 1)) {
        for (kk in (jj + 1):length(var())) {
          if (path[[kk]] == path[[jj]] & type[[kk]]$dim == type[[jj]]$dim)
            ind[kk] <- ind[jj]
        }
      }
    }
    
    tmp <- unique(sort(ind)) 
    
    if (length(tmp) > 1) {
      for (n in 1:(length(tmp) - 1)) {  
        if (tmp[n + 1] != (tmp[n] + 1)) {
          ind[which(ind == tmp[n + 1])] <- tmp[n] + 1
        }
      }
    }
    
    subdata <- vector(mode = "list", length = length(unique(ind)))
    subdata <- lapply(1:length(subdata), function(kk) subdata[[kk]] <- list(unlist(path[which(ind == kk)][1]), type[which(ind == kk)]))
    #subdata <- lapply(1:length(subdata), function(kk) names(subdata[[kk]]) <- c('path', 'type'))
    print('subdata')
    print(str(subdata))
    subdata
    
  })
  
  
  #-----------------------Calculation-------------------------
  #CalInput <- reactive({
    CalInput <- eventReactive(xx(), {
    func_cal(typeInput(), pathInput(), 
             input$rundate1, input$rundate2, subdata())   #BUG: typeInput() should be removed. wait for (i, j)
  })
  
  tt <- reactive({
    print('in tt')
    if (input$cal == 'none') {
      yr <- as.character(as.integer(substr(input$rundate1, 1, 4)) + session$i - 1)
      paste(input$exp, var(), yr)
    } else if (input$cal == 'clim' | input$cal == 'std') {
      paste(input$exp, var(), input$cal, input$rundate1, '-', input$rundate2)
    } else if (input$cal == 'ano' | input$cal == 'bias') {
      paste(input$exp, var(), input$cal, 'based on', input$rundate1, '-', input$rundate2)
    }
    
  })
  
  #----------------reactive and function over---------------  
  
  cal_use <- eventReactive(num(), {
    print('cal_use eventReactive')
    isolate(CalInput())
  })
  t_use <- eventReactive(num(), {
    print('t_use eventReactive')
    isolate(tt())
  })  
  
  
  #---------------nextPlot--------------------
  observeEvent(input$nextPlot,{
    shinyjs::enable('playAni')
    shinyjs::enable('stopAni')
    
    #---------change i and ii for the next plot--------------
    #BUG?? use cal_use() first list to represent all
    tmp <- func_timestep(cal_use(),  session$ii, session$i, input$rundate1, input$rundate2)
    session$i <- tmp[[1]]
    session$ii <- tmp[[2]]
    
    
    #------------a list for plot input-----------
    plot_arg <- func_forward(cal_use(), t_use(), subdata())
    
    
    #--------dynamically create the right number of plotOutput---------
    
    func_name <- function(y, x) {
      plotname <- vector(mode = "list", length = length(x))
      for (nn in 1: length(x)) {
        plotname[[nn]] <- paste0('plot', nn, '_', 1:length(names(y[[nn]][[2]])))
      }
      unlist(plotname)
    }
    
    output$plots <- renderUI({
      # plot_output_list <- lapply(1:length(var()), function(i) {
      #   plotname <- paste0("plot", i)
      #   plotOutput(plotname)
      # })
      plotname <- func_name(subdata(), subdata())  #a vector
      plot_output_list <- lapply(1:length(plotname), function(i) {plotOutput(plotname[i])} )
      tagList(plot_output_list)
    }) 
    
    
    #------Call renderPlot for each one-----------
    for (nn in 1:length(cal_use())) {
      
      for (i in 1:length(names(subdata()[[nn]][[2]]))) {
        
        local({
          my_nn <- nn
          my_i <- i
          plotname <- paste0('plot', my_nn, '_', my_i)
          
          output[[plotname]] <- renderPlot({

            print('renderPlot:')
            print(plotname)
            print('ii =')
            print(session$ii)
            print('i =')
            print(session$i)

            if (plot_arg[[my_nn]][[my_i]]$is_map) {  #is_map

              func_plot(cal_use()[[my_nn]], kk = my_i, plot_arg[[my_nn]][[my_i]]$is_map,
                        plot_arg[[my_nn]][[my_i]]$lons, plot_arg[[my_nn]][[my_i]]$lats,
                        plot_arg[[my_nn]][[my_i]]$units, plot_arg[[my_nn]][[my_i]]$title,
                        plot_arg[[my_nn]][[my_i]]$brks, plot_arg[[my_nn]][[my_i]]$cbar,
                        session$ii, session$i, input$rundate1, input$rundate2, subdata()[[my_nn]])[[1]]

            } else {  #time series

              func_plot(cal_use()[[my_nn]], kk = my_i, plot_arg[[my_nn]][[my_i]]$is_map,
                        plot_arg[[my_nn]][[my_i]]$lons, plot_arg[[my_nn]][[my_i]]$lats,
                        plot_arg[[my_nn]][[my_i]]$units, plot_arg[[my_nn]][[my_i]]$title,
                        plot_arg[[my_nn]][[my_i]]$brks, plot_arg[[my_nn]][[my_i]]$cbar,
                        session$ii, session$i, input$rundate1, input$rundate2, subdata()[[my_nn]])#[[1]]
            }
          },
          height = 400*(log(6)/log(1 + 5)), width = 600
          )
          
          ##################
          #### func_plot2()
          # func_plot2(session$ii, subdata())
          # output[[plotname]] <- renderImage({
          #   
          #   list(src = paste0('/home/Earth/aho/aho-testtest/shiny/output_', nn, '_', i, '_', 
          #                     sprintf("%02d", as.numeric(session$ii)-1), '.png'),
          #        contentType = 'png',
          #        width = 600,
          #        height = 400)
          #        #alt = "This is alternate text")
          #   
          # }, deleteFile = TRUE)
          ##################
          
          
        })
      }
      
    } #for loop nn
    
  })  #observeEvent(input$nextPlot
  
  
  #---------------anim--play or stop-----------------
  observeEvent(input$playAni,{
    
    shinyjs::disable('playAni')
    shinyjs::disable('nextPlot')
    shinyjs::enable('stopAni')
    
    session$run <- TRUE
  }) 
  
  observeEvent(input$stopAni, {
    
    shinyjs::disable('stopAni')
    shinyjs::enable('playAni')
    shinyjs::enable('nextPlot')
    
    session$run <- FALSE
  })
  
  
  observeEvent(session$timer(), {
    
    if (isolate(session$run)) {
      
      #---------change i and ii for the next plot--------------
      tmp <- func_timestep(cal_use(), session$ii, session$i, input$rundate1, input$rundate2)
      session$i <- tmp[[1]]
      session$ii <- tmp[[2]]
      
      #---------a list for plot input------------
      plot_arg <- func_forward(cal_use(), t_use(), subdata())
      
      
      #--------dynamically create the right number of plotOutput---------
      
      func_name <- function(y, x) {
        plotname <- vector(mode = "list", length = length(x))
        for (nn in 1: length(x)) {
          plotname[[nn]] <- paste0('plot', nn, '_', 1:length(names(y[[nn]][[2]])))
        }
        unlist(plotname)
      }
      
      output$plots <- renderUI({
        
        plotname <- func_name(subdata(), subdata())  #a vector
        plot_output_list <- lapply(1:length(plotname), function(i) {plotOutput(plotname[i])} )
        tagList(plot_output_list)
      }) 
      
      
      #------Call renderPlot for each one-----------
      for (nn in 1:length(cal_use())) {
        
        for (i in 1:length(names(subdata()[[nn]][[2]]))) {
          
          local({
            my_nn <- nn
            my_i <- i
            plotname <- paste0('plot', my_nn, '_', my_i)
            
            output[[plotname]] <- renderPlot({
              
              print('renderPlot:')
              print(plotname)
              print('ii =')
              print(session$ii)
              print('i =')
              print(session$i)
              
              if (plot_arg[[my_nn]][[my_i]]$is_map) {  #is_map
                
                func_plot(cal_use()[[my_nn]], kk = my_i, plot_arg[[my_nn]][[my_i]]$is_map, 
                          plot_arg[[my_nn]][[my_i]]$lons, plot_arg[[my_nn]][[my_i]]$lats, 
                          plot_arg[[my_nn]][[my_i]]$units, plot_arg[[my_nn]][[my_i]]$title,
                          plot_arg[[my_nn]][[my_i]]$brks, plot_arg[[my_nn]][[my_i]]$cbar, 
                          session$ii, session$i, input$rundate1, input$rundate2, subdata()[[my_nn]])[[1]]
                
              } else {  #time series
                
                func_plot(cal_use()[[my_nn]], kk = my_i, plot_arg[[my_nn]][[my_i]]$is_map, 
                          plot_arg[[my_nn]][[my_i]]$lons, plot_arg[[my_nn]][[my_i]]$lats, 
                          plot_arg[[my_nn]][[my_i]]$units, plot_arg[[my_nn]][[my_i]]$title,
                          plot_arg[[my_nn]][[my_i]]$brks, plot_arg[[my_nn]][[my_i]]$cbar, 
                          session$ii, session$i, input$rundate1, input$rundate2, subdata()[[my_nn]])#[[1]]
              }
            },
            height = 400*(log(6)/log(1 + 5)), width = 600
            )
            
          })
        }
        
      } #for loop nn      
      
      
    } #if
  }) #observeEvent(timer)
  
  
  #--------the message below plotting--------
  output$cal_info <- renderText({
    
    if (input$cal != 'none') {
      paste0("Calculation takes several minutes. ",
             "Don't refresh the page unless you wanna cancel the calculation.")
    } else {
      ""
    }
  })
  
  
})  #all end
