func_forward <- function(cal, tt, subdata, lev) {
  arg <- vector(mode = "list", length = length(cal))
  names(arg) <- names(cal)
  cal_ori <- cal  #keep the original cal
  
  for (kk in 1:length(cal_ori)) {   #run through each var

# NOTE: The method for irregular can also be used for others. Which method is faster?

    if (subdata[[kk]][[2]]$dim == 'ij') {  #irregular
      file <- easyNCDF::NcOpen(subdata[[kk]][[2]]$path)
    lev_names <- c('lev', 'level', 'height', 'depth', 'plev')
      if (!any(lev_names %in% subdata[[kk]][[2]]$dimnames)) {
      cal <- easyNCDF::NcToArray(file, vars_to_read = names(subdata)[kk])
      } else {  #lev
        lev_name <- subdata[[kk]][[2]]$dimnames[which(subdata[[kk]][[2]]$dimnames %in% lev_names)]
        dim_indices_input <- list(as.numeric(lev))
        names(dim_indices_input) <- lev_name
        cal <- easyNCDF::NcToArray(file, vars_to_read = names(subdata)[kk],
                                   dim_indices = dim_indices_input)
     }

    #---------define colorbar-------------
    # Turn land value into NAs
    asd <- sort(unique(as.vector(cal)))
    if ((asd[2] - asd[1]) > 10^10 & (asd[3] - asd[2]) < 10^10) {
      asd[1] <- NA
    } else if ((asd[length(asd)] - asd[length(asd) - 1]) > 10^10 &
               (asd[length(asd) - 1] - asd[length(asd) - 2]) < 10^10) {
      asd[length(asd)] <- NA
    }
    asd <- na.omit(asd)

    if (!all(asd >= 0) && !all(asd <= 0)) {
      cbar <- c("seismic")
      #define brks
      bound <- max(abs(min(asd)), max(asd))
      brks <- seq(-bound + 0.0001,
                  bound - 0.0001,
                  length.out = 101)
    } else {
      cbar <- c("YlOrRd")
      #define brks
      brks <- seq(min(asd, na.rm = TRUE) + 0.0001,
                  max(asd, na.rm = TRUE) - 0.0001,
                  length.out = 101)
    }

    #----lon and lat, and units---------
    if ('latitude' %in% subdata[[kk]][[2]]$var_list) {
      lat_name <- 'latitude'
      lon_name <- 'longitude'
    } else if ('lat' %in% subdata[[kk]][[2]]$var_list) {
      lat_name <- 'lat'
      lon_name <- 'lon'
    }
    lats <- c(min(as.vector(easyNCDF::NcToArray(file, vars_to_read = lat_name))),
              max(as.vector(easyNCDF::NcToArray(file, vars_to_read = lat_name))))
    lons <- c(min(as.vector(easyNCDF::NcToArray(file, vars_to_read = lon_name))),
              max(as.vector(easyNCDF::NcToArray(file, vars_to_read = lon_name))))

    units <- try(ncdf4::ncatt_get(file, names(subdata)[kk], attname = 'units')[[2]])
    is_map <- TRUE

    arg[[kk]] <- list(is_map = is_map, lons = lons, lats = lats,
                      units = units, title = tt[kk], brks = brks, cbar = cbar)

    } else { # old script
    
    cal <- cal_ori[[kk]]  # kk_th var
    
    #---------define colorbar-------------
    
    asd <- na.omit(as.vector(cal))
    if (!all(asd >= 0) && !all(asd <= 0)) {
      cbar <- c("bluered")
      #define brks
      bound <- max(abs(min(asd)), max(asd))
      brks <- seq(-bound + 0.0001,
                  bound - 0.0001,
                  length.out = 101)
    } else {
      cbar <- c("yellowred")
      #define brks
      brks <- seq(min(cal, na.rm = TRUE) + 0.0001,
                  max(cal, na.rm = TRUE) - 0.0001,
                  length.out = 101)
    }
    
    
    #--------retrieve lat, lon, and units for plotting-----------
    
    lon_name <- 'lon'
    if ('longitude' %in% names(attr(cal, 'Variables')$common)) {
      lon_name <- 'longitude'
    } 
    
    lat_name <- 'lat'
    if ('latitude' %in% names(attr(cal, 'Variables')$common)) {
      lat_name <- 'latitude'
    } 
    
    lons <- unique(as.vector(attr(cal, 'Variables')$common[[lon_name]]))
    lats <- unique(as.vector(attr(cal, 'Variables')$common[[lat_name]]))
    units <- try(attr(cal, 'Variables')$common[[names(cal_ori)[kk]]]$units)   #BUG: NULL for non-first var, NULL for all var if retrieve = FALSE
    
    
    #-------Find out 2D Map or 1D time series--------
    if (any(c('lat', 'latitude', 'j') %in% names(attr(cal, 'Variables')$common))
        && any(c('lon', 'longitude', 'i') %in% names(attr(cal, 'Variables')$common))) {
      is_map <- TRUE
    } else {
      is_map <- FALSE
    }
    
    
    arg[[kk]] <- list(is_map = is_map, lons = lons, lats = lats, 
                      units = units, title = tt[kk], brks = brks, cbar = cbar)
    
    } # if irregular

  } #for loop kk
  
  return(arg)
  
}  #forward
