func_plot <- function(cal, is_map, lons, lats, units, tt, brks, cbar, ii, i, rundate1, rundate2, subdata, lev, var_name) {
  print('in func_plot!!!')
  
  validate(
    need(ii != 0, "Time step has been reset or data has been changed. Click plot button again.")
  )
  
  if (is_map) {
    if (!is.null(dim(cal))) {  #old, regular grid

    tmp <- try(cal[1, 1, ii, , ], silent = TRUE)
    if (class(tmp) == 'try-error') {
      tmp <- cal[1, 1, ii, , , 1]  # last dim is 'lev'
    }
    
    if (length(dim(cal)) == 5) {  # 2D map
      tmp_title <- paste0(tt, ' month = ', ii)
    } else if (length(dim(cal)) == 6) {  # with 'lev'
      lev_names <- c('lev', 'level', 'height', 'depth', 'plev')
      lev_name <- lev_names[which(lev_names %in% subdata[[2]]$dimnames)]
      lev_units <- try(ncdf4::ncatt_get(ncdf4::nc_open(subdata[[2]]$path),
                                        lev_name, attname = 'units')$value)
      if (class(lev_units) == 'try-error') {
        lev_units <- ''
      }
      lev_value <- try(as.vector(easyNCDF::NcToArray(subdata[[2]]$path,
                                                     vars_to_read = lev_name))[as.numeric(lev)])
      if (class(lev_value) == 'try-error') {
        lev_value <- ''
      } else {
        # round the number
        lev_value <- round(lev_value, digits = 2)
      }

      # Adjust 'Pa' to 'hPa'
      if (lev_units == 'Pa') {
        lev_units <- 'hPa'
        lev_value <- as.numeric(lev_value)/100
      }
      tmp_title <- paste0(tt, ' month = ', ii, ', ', lev_name, ' = ', lev,
                          ' (', lev_value, ' ', lev_units, ')')
    }

      # add long name in title if it has
      long_name <- ncdf4::ncatt_get(ncdf4::nc_open(subdata[[2]]$path),
                                    var_name, attname = 'long_name')$value
      if (long_name != 0) {
        tmp_title <- paste0(tmp_title, '\n(', long_name, ')')
        title_scale <- 0.7  #shrink the title scale
      } else {
        title_scale <- 1
      }

    res_plot <- PlotEquiMap(tmp,
                            lons, lats,
                            toptitle = tmp_title,
                            title_scale  = title_scale,
			    brks = brks, 
			    color_fun = clim.palette(palette = cbar),
                            triangle_ends = c(TRUE, TRUE),
                            units = units,
                            intylat = 10, intxlon = 10,
                            fill = FALSE)

    } else { #irregular grid

library(reticulate)
plotmap <- import("mapgenerator.plotting.plotmap")
pm <- plotmap$PlotMap()

# TODO: lev
lons <- seq(floor(lons[1]), ceiling(lons[2]), 30)
lats <- seq(floor(lats[1]), ceiling(lats[2]), 30)
ticks <- seq(floor(min(brks)), ceiling(max(brks)), length.out = 5)
tmp_title <- paste0(tt, ' month = ', ii, ' [', units, ']')

lev_names <- c('lev', 'level', 'height', 'depth', 'plev')
      if (!any(lev_names %in% subdata[[2]]$dimnames)) {  # no lev

      # add long name in title if it has
      long_name <- ncdf4::ncatt_get(ncdf4::nc_open(subdata[[2]]$path),
                                    var_name, attname = 'long_name')$value
      if (long_name != 0) {
        tmp_title <- paste0(tmp_title, '\n(', long_name, ')')
        fontsize <- 8  #shrink the title scale
      } else {
        fontsize <- 12
      }

res_plot <- pm$plot(srcfiles = subdata[[2]]$path, srcvars = var_name,
                    timesteps = ii - 1,
                    lon = lons, lat = lats,
                    colors = cbar,
                    bounds = brks, ticks = ticks,
                    extend = "both",
                    title = tmp_title,
                    fontsize = fontsize,
		    img_template = paste0(var_name, "_%(step)s"),
                    outdir = "/srv/shiny-server/es/mapgenerator_plot")  # read from nc file

      } else {  #lev
              print('irregular with level!!!!')
      lev_name <- lev_names[which(lev_names %in% subdata[[2]]$dimnames)]
      lev_units <- try(ncdf4::ncatt_get(ncdf4::nc_open(subdata[[2]]$path),
                                        lev_name, attname = 'units')$value)
      if (class(lev_units) == 'try-error') {
        lev_units <- ''
      }
      lev_value <- try(as.vector(easyNCDF::NcToArray(subdata[[2]]$path,
                                                     vars_to_read = lev_name))[as.numeric(lev)])
      if (class(lev_value) == 'try-error') {
        lev_value <- ''
      } else {
        # round the number
        lev_value <- round(lev_value, digits = 2)
      }

      tmp_title <- paste0(tmp_title, ' ', lev_name, ' = ', lev,
                          ' (', lev_value, ' ', lev_units, ')')
      # add long name in title if it has
      long_name <- ncdf4::ncatt_get(ncdf4::nc_open(subdata[[2]]$path),
                                    var_name, attname = 'long_name')$value
      if (long_name != 0) {
        tmp_title <- paste0(tmp_title, '\n(', long_name, ')')
        fontsize <- 8  #shrink the title scale
      } else {
        fontsize <- 12
      }
      lev_input <- paste0(lev_name, ",", as.numeric(lev) - 1)  #python starts from 0

res_plot <- pm$plot(srcfiles = subdata[[2]]$path,
                    srcvars = var_name,
#                    timesteps = 0, #ii - 1,
                    dimension = lev_input,
                    lon = lons, lat = lats,
                    colors = cbar,
                    bounds = brks, ticks = ticks,
                    extend = "both",
                    title = tmp_title,
                    fontsize = fontsize,
		    img_template = paste0(var_name, "_%(step)s"),
                    outdir = "/srv/shiny-server/es/mapgenerator_plot")
      }
    }  # irre if loop

  } else {   #is_map = FALSE, time series
    dimname <- subdata[[2]]$dimnames
    possible_spatial_names <- c('lev', 'level', 'plev','lon', 'lat',
                                'longitude', 'latitude', 'height', 'depth')
   if (any(dimname %in% possible_spatial_names)) {  # with lev (time series at one level)
     # Subset (remove) the lev dim
     spatial_dim_name <- dimname[which(dimname %in% possible_spatial_names)]
     cal <- Subset(cal, spatial_dim_name, 1, drop = 'selected')
   }

    # running mean (> 10 years)
    time_dim <- which(names(dim(cal)) == 'time')
    
    if (dim(cal)[time_dim] > 120) {
      smooth <- TRUE
    } else {
      smooth <- FALSE
    }
    
    yr_diff <- as.numeric(substr(rundate2, 1, 4)) - as.numeric(substr(rundate1, 1, 4)) + 1
    
    if (smooth) {
      if (yr_diff <= 100) {
        cal_smooth <- s2dverification::Smoothing(cal, 12, time_dim) 
      } else {
        cal_smooth <- s2dverification::Smoothing(cal, 120, time_dim)
        cal <- s2dverification::Smoothing(cal, 12, time_dim)
      }
    }
    
    time_id <- grep('time', names(dim(cal)))
    region_id <- grep('region', names(dim(cal)))
    data_df <- adply(Subset(cal, 'var', 1), c(time_id, region_id))
    if (smooth) {
      data_df_smooth <- adply(Subset(cal_smooth, 'var', 1), c(time_id, region_id))
    }
    
    #rename
    n <- length(names(data_df))
    
    type_dim <- subdata[[2]]$dim
    
    if(type_dim == 'time_region') {
      names(data_df)[1] <- 'time'
      names(data_df)[2] <- 'region'
      names(data_df)[3] <- 'var' #input$var
      if (smooth) {
        names(data_df_smooth)[1] <- 'time'
        names(data_df_smooth)[2] <- 'region'
        names(data_df_smooth)[3] <- 'var2' 
      }
      
      reg_name <- data.frame(    
        region = c(1:length(unique(data_df$region))),    
        reg_name = subdata[[2]]$regions)
      
      data_df <- merge(data_df, reg_name, by.x = "region")
      if (smooth) {
        data_df_smooth <- merge(data_df_smooth, reg_name, by.x = "region")
        data_df <- merge(data_df_smooth, data_df)
      }
      
    } else {
      names(data_df)[1] <- 'time'
      names(data_df)[2] <- 'var' 
      
      if (smooth) {
        names(data_df_smooth)[1] <- 'time'
        names(data_df_smooth)[2] <- 'var2'
        
        data_df <- merge(data_df_smooth, data_df)
      }
      
    }

    # find lev units and values
    if (exists('spatial_dim_name')) {  # have lev
      lev_names <- c('lev', 'level', 'height', 'depth', 'plev')
      lev_name <- lev_names[which(lev_names %in% subdata[[2]]$dimnames)]
      lev_units <- try(ncdf4::ncatt_get(ncdf4::nc_open(subdata[[2]]$path),
                                        lev_name, attname = 'units')$value)
      if (class(lev_units) == 'try-error') {
        lev_units <- ''
      }
      print('lev_units')
      print(lev_units)
      lev_value <- try(as.vector(easyNCDF::NcToArray(subdata[[2]]$path,
                                                     vars_to_read = lev_name))[as.numeric(lev)])
      if (class(lev_value) == 'try-error') {
        lev_value <- ''
      } else {
        # round the number
        lev_value <- round(lev_value, digits = 2)
      }
      # Adjust 'Pa' to 'hPa'
      if (lev_units == 'Pa') {
        lev_units <- 'hPa'
        lev_value <- as.numeric(lev_value)/100
      }
    }

    # Adjust title
    tt <- substr(tt, 1, nchar(tt) - 4)
    if (smooth) {
      if (exists('spatial_dim_name')) {
        tt <- paste0(tt, 'at ', spatial_dim_name, ' = ', lev,
                     ' (', lev_value, ' ', lev_units, ')')
      }
      if (yr_diff <= 100) {
        titlename <- paste0(tt, ', ', substr(rundate1, 1, 4), ' - ', substr(rundate2, 1, 4), ', ', ' + 120-month running mean')
      } else {
        titlename <- paste0(tt, ' 12-month running mean, ', substr(rundate1, 1, 4), ' - ', substr(rundate2, 1, 4), ', ', ' + 120-month running mean')
      }

    } else {
      if (exists('spatial_dim_name')) {
        tt <- paste0(tt, 'at ', spatial_dim_name, ' = ', lev,
                     ' (', lev_value, ' ', lev_units, ')')
      }
      titlename <- paste0(tt, ', ', substr(rundate1, 1, 4), ' - ', substr(rundate2, 1, 4))
    }

    # add long name in title if it has
    long_name <- ncdf4::ncatt_get(ncdf4::nc_open(subdata[[2]]$path),
                                  var_name, attname = 'long_name')$value
    if (long_name != 0) {
      titlename <- paste0(titlename, '\n(', long_name, ')')
#      title_scale <- 0.7  #shrink the title scale
    } else {
#      title_scale <- 1
    }

    # Adjust x axis and label
    if (yr_diff <= 20) { # 1-20yr; 1yr spacing
      
      breaks <- seq(1, (as.numeric(substr(rundate2, 1, 4), 1) - as.numeric(substr(rundate1, 1, 4), 1) + 1) * 12, 12)
      labels <- as.character(rep(seq(as.numeric(substr(rundate1, 1, 4)), as.numeric(substr(rundate2, 1, 4)),1)))
      
    } else if (yr_diff > 20 & yr_diff <= 50) { # 21-50yr; 5yr spacing
      breaks <- seq(1, (as.numeric(substr(rundate2, 1, 4)) - as.numeric(substr(rundate1, 1, 4)) + 1) * 12, 60)
      labels <- as.character(seq(as.numeric(substr(rundate1, 1, 4)), as.numeric(substr(rundate2, 1, 4)), 5))
      
    } else if (yr_diff > 50 & yr_diff <= 100) { # 51-100yr; 10yr spacing
      breaks <- seq(1, (as.numeric(substr(rundate2, 1, 4)) - as.numeric(substr(rundate1, 1, 4)) + 1) * 12, 120)
      labels <- as.character(seq(as.numeric(substr(rundate1, 1, 4)), as.numeric(substr(rundate2, 1, 4)), 10))
      
    } else if (yr_diff > 100 & yr_diff <= 500) { # 101-500yr; 20yr spacing
      # push the starting point to the next 10s
      ori_yr1 <- as.numeric(substr(rundate1, 1, 4))
      round <- ori_yr1%%10
      if (round != 0) {
        new_yr1 <- ori_yr1 - round + 10  #e.g., 1953 - 3 + 10 = 1960
      } else {
        new_yr1 <- ori_yr1
      }
      padding <- (new_yr1 - ori_yr1) * 12
      breaks <- seq(1 + padding, (as.numeric(substr(rundate2, 1, 4)) - as.numeric(substr(rundate1, 1, 4)) + 1) * 12, 240)
      labels <- as.character(seq(new_yr1, as.numeric(substr(rundate2, 1, 4)), 20))
      labels <- paste0(labels, 's')
      
    } else if (yr_diff > 500) { # 501-yr; 50yr spacing
      # push the starting point to the next 100s
      ori_yr1 <- as.numeric(substr(rundate1, 1, 4))
      round <- ori_yr1%%50
      if (round != 0) {
        new_yr1 <- ori_yr1 - round + 50  #e.g., 1903 - 3 + 50 = 1950
      } else {
        new_yr1 <- ori_yr1
      }
      padding <- (new_yr1 - ori_yr1) * 12
      breaks <- seq(1 + padding, (as.numeric(substr(rundate2, 1, 4)) - as.numeric(substr(rundate1, 1, 4)) + 1) * 12, 600)
      labels <- as.character(seq(new_yr1, as.numeric(substr(rundate2, 1, 4)), 50))
      labels <- paste0(labels, 's')
    }
    
    
    if(type_dim == 'time_region') {
      
      if (smooth) {
        res_plot <- ggplot(data_df, aes(as.numeric(time), color = reg_name)) +
          geom_point(aes(y = var2), alpha = 0, size = 0.3) +
          geom_point(aes(y = var), alpha = 0, size = 0.1) +
          geom_line(aes(y = var2), alpha = 0.9, size = 0.5) +
          geom_line(aes(y = var), alpha = 0.5, size = 0.3) +
          labs(title = titlename,
               x = 'time', y = units, color = "Region") +
          scale_x_continuous(breaks = breaks,
                             labels = labels) +
          theme(axis.text.x = element_text(angle = 45, hjust = 1))
        
      } else {
        res_plot <- ggplot(data_df, aes(as.numeric(time), var, color = reg_name)) + 
          geom_point(alpha = 0.6, size = 0.5) +
          geom_line(size = 0.3) +
          labs(title = titlename,
               x = 'time', y = units, color = "Region") +
          scale_x_continuous(breaks = breaks, 
                             labels = labels) +
          theme(axis.text.x = element_text(angle = 45, hjust = 1))
      }
      
    } else {
      if (smooth) {
        
        res_plot <- ggplot(data_df, aes(as.numeric(time))) +
          geom_point(aes(y = var2), alpha = 0.6, size = 0.3) +
          geom_point(aes(y = var), alpha = 0.3, size = 0.2) +
          geom_line(aes(y = var2), alpha = 0.4, size = 0.2) +
          geom_line(aes(y = var), alpha = 0.2, size = 0.1) +
          labs(title = titlename,
               x = 'time', y = units) +
          scale_x_continuous(breaks = breaks,
                             labels = labels) +
          theme(axis.text.x = element_text(angle = 45, hjust = 1))
        
      } else {
        
        res_plot <- ggplot(data_df, aes(as.numeric(time), var)) +
          geom_point(alpha = 0.6, size = 0.5) +
          geom_line(size = 0.3) +
          labs(title = titlename,
               x = 'time', y = units) +
          scale_x_continuous(breaks = breaks,
                             labels = labels) +
          theme(axis.text.x = element_text(angle = 45, hjust = 1))
      }
      
    }
    
  }  #is_map = FALSE
  
  
  return(res_plot)
  
}  #ends

