library(shiny)
library(shinyjs)
library(s2dverification)
library(startR)
library(ggplot2)
library(plyr)
library(abind)
#library(reticulate)
# 
source('func_type.R')
source('func_forward.R')
source('func_plot.R')
#source('func_plot2.R')
source('func_cal.R')
source('func_timestep.R')

file('/var/log/shiny-server/server-v100.txt', open = 'a')
sink('/var/log/shiny-server/server-v100.txt')
print('hello in server-v100.txt!!!!!!!!!!!')

shinyServer(function(input, output, session) {
  
  # only original data
  shinyjs::disable("comp")
  shinyjs::disable("cal")
  
  # shinyjs::disable('nextPlot')
  # shinyjs::disable('playAni')
  # shinyjs::disable('stopAni')
  # 
  observeEvent({
    input$playAni
    input$stopAni
    input$nextPlot
    1} , {
      shinyjs::disable('sure')
    })
  
  observeEvent(input$sure, {
    shinyjs::enable('stopAni')
    shinyjs::enable('playAni')
    shinyjs::enable('nextPlot')
  })
  
  # no work
  # observeEvent(!session$canplot, {
  #   shinyjs::enable('stopAni')
  #   shinyjs::enable('nextPlot')
  #   shinyjs::enable('playAni')
  # })
  
  observe({
    #both work, either put inside observe() or outside like above.
    shinyjs::toggleState("sure", !is.null(input$var_grid) && input$var_grid != "")
  })
  
  
  #------------------dir finding-----------------------
  
  #------Choose between diags or cmorfiles/-------
  cmordiags <- reactive({
    if (input$cmordiags == 'cmorfiles') {
      cmordiags <- 'cmorfiles'
    } else {
      cmordiags <- 'diags'
    }
    cmordiags
  })
  
  
  #-------filter out the workable exp ID-----------
  observeEvent(input$cmordiags,{
    req(!is.null(input$cmordiags))  

      asd <- list.files(file.path('/esarchive/exp/ecearth/'))  #all the exp name
      # create a list of lists recording the full path for each exp
      asd_fullpath <- lapply(asd, function(x) paste0('/esarchive/exp/ecearth/', x, '/'))
      
      #------remove the ones without cmordiags/-------
      
      # a list of lists containing all folders under each exp
      qwe <- lapply(asd_fullpath, function(x) list.files(x))
      # assign the exp which contains cmordiags 1, without cmordiags 0. qwe is a vector
      qwe <- sapply(qwe, function(x) {if (!(cmordiags() %in% x)) {x <- 0} else {x <- 1} })
      # only keep the exp which contains cmordiags 
      asd <- asd[which(qwe == 1)]
      # create a list of lists recording the full path for each exp
      asd_fullpath <- asd_fullpath[which(qwe == 1)]
      
      
      
      #-----remove the ones without EC-Earth-Consortium------
      
      # a list of lists containing all folders under each exp/cmordiags
      qwe <- lapply(asd_fullpath, function(x) list.files(file.path(x, cmordiags())))
      qwe <- lapply(1:length(asd),  function(x) list.files(file.path(asd_fullpath[[x]], cmordiags(), qwe[[x]])))
      qwe <- sapply(qwe, function(x) {if (!'EC-Earth-Consortium' %in% x) {x <- 0} else {x <- 1} })
      asd <- asd[which(qwe == 1)]
      # modify the list of lists recording the full path for each exp
      asd_fullpath <- asd_fullpath[which(qwe == 1)]
      
      # finally, we get the usable exp id!!
      output$exp <- renderUI({
	      selectizeInput("exp", "Experiment ID:",
                     choices = asd,
                     selected = 'a1om')
      
    })  #renderUI, output$exp
  })  #  observeEvent(input$cmordiags,{
  
  
  #-------get *MIP-------
  observeEvent({
    input$cmordiags
    input$exp
    1}, {
      req(!is.null(input$exp))

    opt_MIP <- list.files(file.path('/esarchive/exp/ecearth',
                                      input$exp, cmordiags()))
      if(length(opt_MIP) == 1) {
        output$proj <- renderUI(
          disabled(textInput("proj", "Project:", opt_MIP))
        )
      } else {
        output$proj <- renderUI(
          
          selectizeInput(inputId = "proj", label = "Project:",
                         choices = opt_MIP,
                         selected = opt_MIP[1])
        )
      }
    })
  
  #-----get EC-Earth3*-----
  opt_Earth3 <- eventReactive({
    cmordiags()
    input$exp
    input$proj
    1}, {
      req(!is.null(input$proj))
      #EC-Earth3*
      list.files(file.path('/esarchive/exp/ecearth',
                           input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium'))
    })  
  
  #-----get simulation-----
  opt_simu <- eventReactive({
    cmordiags()
    input$exp
    input$proj
    opt_Earth3()
    1}, {
      req(!identical(opt_Earth3(), character(0)))
      #simulation (e.g., historical)
      list.files(file.path('/esarchive/exp/ecearth',
                           input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                           opt_Earth3()))[1]  #BUG: Have more than one choice. Turn it into selection list
    })  
  
  #-----get version-----
  opt_ver <- eventReactive({
    cmordiags()
    input$exp
    input$proj
    opt_Earth3()
    opt_simu()
    input$memb
    input$table
    var()
    grid()
    1}, {
      req(!identical(grid(), character(0)))
      # get the version
      tail(
        list.files(file.path('/esarchive/exp/ecearth',
                             input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                             opt_Earth3(), opt_simu(), input$memb, input$table, 
                             var()[1], grid()[1])),
        n = 1)  #assume all files in this exp has same ver; use first var as represntation; choose the newest ver
      
    })  
  
  #-----get the memb list under exp-----
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    1}, {
      req(!identical(opt_simu(), character(0)))
      #member
      opt_memb <- list.files(file.path('/esarchive/exp/ecearth',
                                       input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                                       opt_Earth3(), opt_simu()))
      
      if(length(opt_memb) == 1) {
        output$memb <- renderUI(
          disabled(textInput("memb", "Member ID:", opt_memb))
        )
      } else {
        output$memb <- renderUI(
          selectizeInput(inputId = "memb", label = "Member ID:",
                         choices = opt_memb,
                         selected = opt_memb[1]))
      }
      
    })
  
  #------get the table list under memb--------
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    1}, {
      req(!is.null(input$memb))
      opt_table <- list.files(file.path('/esarchive/exp/ecearth',
                                        input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                                        opt_Earth3(), opt_simu(), input$memb))
      # define the selected table. If there is no Omon, use the first table
      if ('Omon' %in% opt_table) {
        selected <- 'Omon'
      } else {
        selected <- opt_table[1]
      }
      output$table <- renderUI(
        selectizeInput("table", "CMOR table:",
                       choices = opt_table,
                       selected = selected))
    })
  
  #------get the var/grid under table---------
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    1}, {
      req(!is.null(input$table))
      opt_var <- list.files(file.path('/esarchive/exp/ecearth',
                                      input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                                      opt_Earth3(), opt_simu(), input$memb, input$table))
      path <- file.path('/esarchive/exp/ecearth',
                        input$exp, cmordiags(), input$proj, 'EC-Earth-Consortium',
                        opt_Earth3(), opt_simu(), input$memb, input$table)
      
      grid <- lapply(file.path(path, opt_var), list.files)
      var_grid <- try(unlist(lapply(1:length(opt_var), function(x) file.path(opt_var[x], grid[[x]]))), silent = TRUE)
      
      output$var_grid <- renderUI(
        checkboxGroupInput("var_grid", "var/grid:",
                           choices = var_grid,
                           selected = var_grid[1])
      )
      
    })
  
  #-----separate var and grid from input$var_grid-----
  var <- reactive({
    req(!is.null(input$var_grid))
    tmp <- lapply(input$var_grid, function(x) strsplit(x, '/'))
    var <- unlist(lapply(lapply(tmp, '[[',1), '[[', 1))
    return(var)
  })
  
  grid <- eventReactive(input$var_grid, {
    req(!is.null(input$var_grid))
    tmp <- lapply(input$var_grid, function(x) strsplit(x, '/'))
    grid <- try(unlist(lapply(lapply(tmp, '[[',1), '[[', 2)), silent = TRUE)
    return(grid)
  })
  
  #-------end of dir finding------------
  
  substrRight <- function(x, n) {
    substr(x, nchar(x)- n + 1, nchar(x))
  }
  
  #-------get whole chunk period---------
#  observeEvent({
#    input$cmordiags
#    input$exp
#    input$proj
#    input$memb
#    input$table
#    input$var_grid
#    # var()
#    grid()
#    1}, {
#      
#      opt <- list.files(file.path('/esarchive/exp/ecearth',
#                                  input$exp, isolate(cmordiags()), input$proj, 'EC-Earth-Consortium',
#                                  opt_Earth3(), opt_simu(), input$memb, input$table, 
#                                  var()[1], grid()[1], opt_ver())) #get all files
#      
#      #first chunk
#      opt_1 <- substrRight(opt[1], 16)
#      opt_1 <- substr(opt_1, 1, 6)
#      #last chunk
#      opt_2 <- substrRight(opt[length(opt)], 9)
#      opt_2 <- substr(opt_2, 1, 6)
#      
#      output$date1 <- renderText(paste(opt_1, '-', opt_2))
#      
#    })
  
  #find sdate
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    input$var_grid
    # var()
    grid()
    opt_ver()
    1}, {
    # all the files
      all_files <- lapply(1:length(var()), function(kk) list.files(file.path('/esarchive/exp/ecearth',
                                                                             input$exp, isolate(cmordiags()),input$proj, 'EC-Earth-Consortium',
                                                                             opt_Earth3(), opt_simu(), input$memb, input$table,
                                                                             var()[kk], grid()[kk], opt_ver())))
      req(all(unlist(sapply(all_files, substrRight, 3)) == '.nc'))

      dir_in_file_name <- paste0(opt_simu(), '_', input$memb)
      
      if (grepl(dir_in_file_name, all_files[[1]][1])) {   #no sdate
        # output$sdate <- ""
        opt_sdate <- ""
        output$sdate <- renderUI(
          disabled(textInput("sdate", "sdate:", opt_sdate))
        )
        
      } else {  #there is sdate
        
        
        all_files <- unlist(all_files)
        tmp <- lapply(strsplit(all_files, opt_simu()), '[[', 2)
        tmp <- (substr(tmp, 2, nchar(tmp[1])))
        tmp <- substr(tmp, 1, (regexpr(input$memb, tmp)[1] - 1 - 1))
        opt_sdate <- unique(tmp)
        
        if(length(opt_sdate) == 1) {
          output$sdate <- renderUI(
            disabled(textInput("sdate", "sdate:", opt_sdate))
          )
        } else {
          output$sdate <- renderUI(
            selectizeInput(inputId = "sdate", label = "sdate:",
                           choices = opt_sdate,
                           selected = opt_sdate[1]))
        }
      }
      
    })
  
  
  #set animation run start date
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    input$var_grid
    # var()
    grid()
    opt_ver()
    input$sdate
    1}, {
      
      req(!is.null(input$sdate))
      all_files <- lapply(1:length(var()), 
			  function(kk) list.files(file.path('/esarchive/exp/ecearth',
                                                  input$exp, isolate(cmordiags()),input$proj, 'EC-Earth-Consortium',
                                                  opt_Earth3(), opt_simu(), input$memb, input$table,
                                                  var()[kk], grid()[kk], opt_ver()))) 
      # pick up the files with selected sdate
      all_files <- unlist(all_files)[which(grepl(input$sdate, unlist(all_files)))]

      date_of_each_var <- try(sapply(all_files, substrRight, 16), silent = TRUE)
      date_of_each_var <- try(sapply(date_of_each_var, substr, 1, 6), silent = TRUE)
      all_date <- sort(as.numeric(unique(unlist(date_of_each_var))))
            
      output$rundate1 <- renderUI(
        selectizeInput("rundate1", "Time period starts from:",
                       choices = all_date, 
                       selected = all_date[1])
      )
    })
  
  #in order to tell pathInput which file to read
  num <- reactive({
    # run only when input$rundate1 is not NULL
    req(input$rundate1)
    req(!is.null(input$sdate))
    
    if (input$sdate == "") {
      opt <- list.files(file.path('/esarchive/exp/ecearth',
                                  input$exp, isolate(cmordiags()), input$proj, 'EC-Earth-Consortium',
                                  opt_Earth3(), opt_simu(), input$memb, input$table,
                                  var()[1], grid()[1], opt_ver()))[1]  #beginning of the whole chunk
      asd <- substrRight(opt, 16)
      asd <- substr(asd, 1, 4)
      num <- as.numeric(substr(input$rundate1, 1, 4)) - as.numeric(asd) + session$i  #BUG?? Assume each rundate only has one file & sdate are continuous
      
      return(num)
      
    } else {
      all_files <- lapply(1:length(var()), function(kk) list.files(file.path('/esarchive/exp/ecearth',
                                                                             input$exp, isolate(cmordiags()),input$proj, 'EC-Earth-Consortium',
                                                                             opt_Earth3(), opt_simu(), input$memb, input$table,
                                                                             var()[kk], grid()[kk], opt_ver())))  # all the files
      # find the number of sdate
      all_files <- all_files[[1]]  #BUG?? Use 1st var to represent 
      tmp <- regexpr(input$sdate, all_files)
      asd <- which(tmp != -1)[1]
      # find rundate1 is which number within this sdate
      qwe <- as.numeric(substr(input$sdate, 2, nchar(input$sdate)))  # 's1962' -> c(1962)
      num <- asd + (as.numeric(substr(input$rundate1, 1, 4)) - qwe) - 1 + session$i
      
      return(num)
    }
    
  })
  
  #set animation run end date
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    input$var_grid
    # var()
    grid()
    input$sdate
    input$rundate1
    1}, {
      
      req(!is.null(input$sdate))
      all_files <- lapply(1:length(var()),
                          function(kk) list.files(file.path('/esarchive/exp/ecearth',
                                                  input$exp, isolate(cmordiags()),input$proj, 'EC-Earth-Consortium',
                                                  opt_Earth3(), opt_simu(), input$memb, input$table,
                                                  var()[kk], grid()[kk], opt_ver())))
      # remove the files of other sdate
      all_files <- all_files[which(grepl(input$sdate, all_files))]

      date_of_each_var <- try(sapply(unlist(all_files), substrRight, 9), silent = TRUE)
      date_of_each_var <- try(sapply(date_of_each_var, substr, 1, 6), silent = TRUE)
      all_date <- sort(as.numeric(unique(unlist(date_of_each_var))))
      all_date <- all_date[all_date >= as.numeric(input$rundate1)]

      output$rundate2 <- renderUI(
        selectizeInput("rundate2", "Time period ends at:",
                       choices = all_date,
                       selected = tail(all_date, 1)) 
      )      
      
    })
  
  #----------------filename set----------------
  
  ###################################################
  
  #--------------reactiveValues--------------
  session <- reactiveValues()
  if (!exists('session$i')) session$i <- 1
  if (!exists('session$ii')) session$ii <- 0
  if (!exists('session$timer')) session$timer <- 1000
  if (!exists('session$run')) session$run <- FALSE
  if (!exists('session$canplot')) session$canplot <- FALSE
  observe({
    session$timer <- reactiveTimer(as.numeric(input$freq)*1000)
  })
  
  #---------reset time step when changing data--------------
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    input$var_grid
    # var()
    grid()
    input$sdate
    input$rundate1
    input$rundate2
    input$cal
    input$term
    input$lev
    1}, {
      shinyjs::disable('stopAni')
      shinyjs::disable('nextPlot')
      shinyjs::disable('playAni')
      shinyjs::enable('sure')
    })
  
  observeEvent({
    input$sure
    input$reset
    1}, {
      session$i <- 1
      session$ii <- 0
      print('session$i reset!!')
      print(session$i)
    })
  
  # observe canplot
  observeEvent({
    input$nextPlot
    input$playAni
    session$run
    1}, {
      session$canplot <- TRUE
    })
  
  observeEvent({
    input$reset
    input$sure
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    input$var_grid
    # var()
    grid()
    input$sdate
    input$rundate1
    input$rundate2
    input$cal
    input$term
    input$lev
    1}, {
      session$canplot <- FALSE
    })
  
  
  #-------------reactive and function part---------------
  
  pathInput <- reactive({
    print('in pathInput')
    print('var() in pathInput')
  print(var())  
    dir <- lapply(1:length(var()), function(kk) file.path('/esarchive/exp/ecearth',
                                                          input$exp, isolate(cmordiags()),input$proj, 'EC-Earth-Consortium',
                                                          opt_Earth3(), opt_simu(), input$memb, input$table,
                                                          var()[kk], grid()[kk], opt_ver()) )   #BUG: grid should be gr or grid()[1]????
   print('dir in pathInput')
  print(dir)
  print('num()')
 print(num()) 
    #####
    path <- lapply(1:length(var()), function(kk) file.path(dir[[kk]], list.files(dir[[kk]])[num()]))
  print('path 1')
  print(path)
    path <- lapply(1:length(var()), function(kk) gsub(var()[kk], '$var$', path[[kk]]))
  print('path 2')
    print(path)
    print('pathInput over')
    
    return(path)  #lists of a vector of character
  })
  
  typeInput <- reactive({
    validate(
      need(length(grid()) == length(var()), "Grid choices doesn't match correponding variables.")
    )
    req(!identical(unlist(pathInput()), character(0)))
    req(!is.null(var()))
    func_type(pathInput(), var())    
  })  #typeInput, list (var) of list (type$xxx)
  
  
  # After func_type, we get type$dimnames, which has the info of lev.
  observeEvent({
    input$cmordiags
    input$exp
    input$proj
    input$memb
    input$table
    input$var_grid
    typeInput()
    1}, {
    req(!is.null(typeInput()))
    req(!any(unlist(lapply(typeInput(), is.na))))

    dims <- lapply(typeInput(), '[[', 'dimensions')
    dimnames <- lapply(typeInput(), '[[', 'dimnames')
    
    lev_name <- c('lev', 'level', 'height', 'depth', 'plev')
    if (any(unlist(dimnames) %in% lev_name)) {
      # retrieve lev choice
      ind <- which(unlist(dimnames) %in% lev_name)
      min_lev_number <- min(unlist(dims)[ind])
      
      output$lev <- renderUI(
        selectizeInput("lev", "Level:",
                       choices = c(1:min_lev_number),
                       selected = 1)
      )
    } else {
      output$lev <- NULL
    }
  })
  
  output$bigdimvar <- eventReactive(typeInput(), {
    type <- typeInput()
    asd <- ""
   if (any(unlist(lapply(type, is.na)))) {
      asd <- "Certian problems exist in the netCDF files or the file path. Cannot find the correct variable."
    } else {

      if (any(sapply(type, '[[', 'latlondim') > 2 & sapply(lapply(type, '[[', 'dimnames'), length) > 4 )) { # irre + lev
      	      asd <- paste0("The grid type of this variable is irregular and it is 3-dimensional. ",
                      "Unable to plot.")
      }
      possible_dims <- c('lon', 'longitude', 'lat', 'latitude')
      if (any(sapply(type, '[[', 'dim') == 'time_region' & any(sapply(type, '[[', 'dimnames') %in% possible_dims))) {
        asd <- paste0("The time series variable has lon/lat dimensions. ",
                      "Not supported now.")
      }
      if (any(sapply(type, '[[', 'dim') == 'time' & any(sapply(type, '[[', 'dimnames') %in% possible_dims))) {
        asd <- paste0("The time series variable has lon/lat dimensions. ",
                      "Not supported now.")
      }

    }    
    return(asd)
  })
  
  #--------- classify data into groups---------
  subdata <- reactive({
    print('in subdata')
    type <- typeInput()
    path <- pathInput()
    
    subdata <- vector(mode = "list", length = length(var()))
    subdata <- lapply(1:length(subdata), function(kk) subdata[[kk]] <- list(path[[kk]], type[[kk]]))
    names(subdata) <- names(type)
    print("names(subdata)")
    print(names(subdata))
    print('subdata over')
    
    return(subdata)
  })
  
  
  #----------------------Calculation------------------------
  cal_use <- eventReactive({
    session$i
    input$sure}, {
      func_cal(pathInput(), 
               input$rundate1, input$rundate2, subdata(), input$lev) 
    })
  
  t_use <- reactive({
#    if (input$cal == 'none') {
      yr <- as.character(as.integer(substr(input$rundate1, 1, 4)) + session$i - 1)
      tt <- paste(input$exp, var(), yr)
#    } else if (input$cal == 'clim' | input$cal == 'std') {
#      tt <- paste(input$exp, var(), input$cal, input$rundate1, '-', input$rundate2)
#    } else if (input$cal == 'ano' | input$cal == 'bias') {
#      tt <- paste(input$exp, var(), input$cal, 'based on', input$rundate1, '-', input$rundate2)
#    }
    tt
  })
  
  #------------a list for plot input-----------
  plot_arg <- eventReactive(cal_use(), {
    # TODO: t_use() is not used in func_forward. Can be removed.
    func_forward(cal_use(), t_use(), subdata(), input$lev)
  })
  
  #----------------reactive and function over---------------  
  
  
  #---------------nextPlot--------------------
  observeEvent(input$nextPlot,{
    print('we are in nextplot...')

    withProgress(message = 'Plotting ...', value = 0, {

    session$canplot <- TRUE
    
    #---------change i and ii for the next plot--------------
    tmp <- func_timestep(session$ii, session$i, input$rundate1, input$rundate2)
    session$i <- tmp[[1]]
    session$ii <- tmp[[2]]
    print('session$i in nextPlot')
    print(session$i)
    print('session$ii in nextPlot')
    print(session$ii)
    print('session$canplot in nextPlot')
    print(session$canplot)
    
    #--------dynamically create the right number of plotOutput---------
    
    func_name <- function(x) {
      plotname <- vector(mode = "list", length = length(x))
      for (nn in 1: length(x)) {
        plotname[[nn]] <- paste0('plot', nn)
      }
      unlist(plotname)
    }
    
    output$plots <- renderUI({
      plotname <- func_name(subdata())  # a vector
      plot_output_list <- lapply(1:length(plotname), function(i) {plotOutput(plotname[i])})
      tagList(plot_output_list)
    }) 
    
    #------Call renderPlot for each one-----------
    for (nn in 1:length(cal_use())) {
 
      # Increment the progress bar, and update the detail text.
      incProgress(1/(length(cal_use()) + 1), 
		  detail = paste0("variable ", nn, "of", length(cal_use())))

      local({
        my_nn <- nn
        plotname <- paste0('plot', my_nn)
     
        if (!plot_arg()[[my_nn]]$is_map) {  #NOT is_map
   
        output[[plotname]] <- renderPlot({
          
          print('canplot in renderPlot')
          print(session$canplot)
          
          req(session$canplot)
          
#          if (plot_arg()[[my_nn]]$is_map) {  #is_map
#            
#            func_plot(cal_use()[[my_nn]], plot_arg()[[my_nn]]$is_map,
#                      plot_arg()[[my_nn]]$lons, plot_arg()[[my_nn]]$lats,
#                      plot_arg()[[my_nn]]$units, plot_arg()[[my_nn]]$title,
#                      plot_arg()[[my_nn]]$brks, plot_arg()[[my_nn]]$cbar,
#                      session$ii, session$i, input$rundate1, input$rundate2, 
#                      subdata()[[my_nn]], input$lev)[[1]]
#            
#          } else {  #time series
            
            func_plot(cal_use()[[my_nn]], plot_arg()[[my_nn]]$is_map,
                      plot_arg()[[my_nn]]$lons, plot_arg()[[my_nn]]$lats,
                      plot_arg()[[my_nn]]$units, plot_arg()[[my_nn]]$title,
                      plot_arg()[[my_nn]]$brks, plot_arg()[[my_nn]]$cbar,
                      session$ii, session$i, input$rundate1, input$rundate2, 
                      subdata()[[my_nn]], input$lev, names(subdata())[[my_nn]])#[[1]]
            
#          }
        },
        height = 400, width = 640
        )

      } else { #is_map
        if (!is.null(dim((cal_use()[[my_nn]])))) { #old, regular grid

          output[[plotname]] <- renderPlot({

          print('canplot in renderPlot')
          print(session$canplot)

          req(session$canplot)

            func_plot(cal_use()[[my_nn]], plot_arg()[[my_nn]]$is_map,
                      plot_arg()[[my_nn]]$lons, plot_arg()[[my_nn]]$lats,
                      plot_arg()[[my_nn]]$units, plot_arg()[[my_nn]]$title,
                      plot_arg()[[my_nn]]$brks, plot_arg()[[my_nn]]$cbar,
                      session$ii, session$i, input$rundate1, input$rundate2,
                      subdata()[[my_nn]], input$lev, names(subdata())[[my_nn]])[[1]]
        },
        height = 400, width = 640
        )

        } else {   # irregular, use mapgenerator

         # generate the figure file
         func_plot(cal_use()[[my_nn]], plot_arg()[[my_nn]]$is_map,
                      plot_arg()[[my_nn]]$lons, plot_arg()[[my_nn]]$lats,
                      plot_arg()[[my_nn]]$units, plot_arg()[[my_nn]]$title,
                      plot_arg()[[my_nn]]$brks, plot_arg()[[my_nn]]$cbar,
                      session$ii, session$i, input$rundate1, input$rundate2,
                      subdata()[[my_nn]], input$lev, names(subdata())[[my_nn]])[[1]]

        output[[plotname]] <- renderImage({

          print('canplot in renderImage')
          print(session$canplot)

          req(session$canplot)

           list(src = paste0('/srv/shiny-server/es/mapgenerator_plot/',
                             names(subdata())[[my_nn]], '_',  sprintf("%02d", session$ii - 1),
                             '.png'),
                width = 780, height = 400)
        }, deleteFile = TRUE)
        }  # if irregular
     }
        
        
        ##################
        ### func_plot2()
        # func_plot2(session$ii, subdata(), tt(), plot_arg[[my_nn]][[my_i]]$brks, plot_arg[[my_nn]][[my_i]]$cbar)
        # output[[plotname]] <- renderImage({
        # 
        #   list(src = paste0('/home/Earth/aho/aho-testtest/shiny/output_', nn, '_', i, '_',
        #                     sprintf("%02d", as.numeric(session$ii)-1), '.png'),
        #        contentType = 'png',
        #        width = 600,
        #        height = 400)
        #        #alt = "This is alternate text")
        # 
        # }, deleteFile = TRUE)
        ##################
        
      })
      
    } #for loop nn

    incProgress(1/(length(cal_use()) + 1),
                detail = "finished.")
   })  #progress

  })  #observeEvent(input$nextPlot
  
  
  #---------------anim--play or stop-----------------
  observeEvent(input$playAni,{
    session$canplot <- TRUE
    session$run <- TRUE
  }) 
  
  observeEvent(input$stopAni, {
    session$run <- FALSE
  })
  
  
  observeEvent(session$timer(), {
    
    if (isolate(session$run)) {

      #---------change i and ii for the next plot--------------
      tmp <- func_timestep(session$ii, session$i, input$rundate1, input$rundate2)
      session$i <- tmp[[1]]
      session$ii <- tmp[[2]]
      
      
      #--------dynamically create the right number of plotOutput---------
      
      func_name <- function(x) {
        plotname <- vector(mode = "list", length = length(x))
        for (nn in 1: length(x)) {
          plotname[[nn]] <- paste0('plot', nn)
        }
        unlist(plotname)
      }
      
      output$plots <- renderUI({
        plotname <- func_name(subdata())  # a vector
        plot_output_list <- lapply(1:length(plotname), function(i) {plotOutput(plotname[i])})
        tagList(plot_output_list)
      }) 
      
      
      #------Call renderPlot for each one-----------
      for (nn in 1:length(cal_use())) {
        
        local({
          my_nn <- nn
          plotname <- paste0('plot', my_nn)
          
          output[[plotname]] <- renderPlot({
            print('canplot in renderPlot')
            print(session$canplot)
            
            req(session$canplot)
            
            if (plot_arg()[[my_nn]]$is_map) {  #is_map
              
              func_plot(cal_use()[[my_nn]], plot_arg()[[my_nn]]$is_map,
                        plot_arg()[[my_nn]]$lons, plot_arg()[[my_nn]]$lats,
                        plot_arg()[[my_nn]]$units, plot_arg()[[my_nn]]$title,
                        plot_arg()[[my_nn]]$brks, plot_arg()[[my_nn]]$cbar,
                        session$ii, session$i, input$rundate1, input$rundate2, 
                        subdata()[[my_nn]], input$lev, names(subdata())[[my_nn]])[[1]]
              
            } else {  #time series
              
              func_plot(cal_use()[[my_nn]], plot_arg()[[my_nn]]$is_map,
                        plot_arg()[[my_nn]]$lons, plot_arg()[[my_nn]]$lats,
                        plot_arg()[[my_nn]]$units, plot_arg()[[my_nn]]$title,
                        plot_arg()[[my_nn]]$brks, plot_arg()[[my_nn]]$cbar,
                        session$ii, session$i, input$rundate1, input$rundate2, 
                        subdata()[[my_nn]], input$lev, names(subdata())[[my_nn]])#[[1]]
            }
          },
          height = 400, width = 640
          )
          
        })
        
      } #for loop nn      
      
    } #if
  }) #observeEvent(timer)
  
  
})  #all end
