#-----------------------------------
# Script to load ECV, build model and compute Corr
# ECMWF ERA5 

# Chung
# June 2021
#-----------------------------------
rm(list = ls())
library(startR)
library(CSTools)
library(s2dv)

path.output <- '/esarchive/scratch/cchou/FOCUS/teleconnection/output/'
path.exp <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_s0-24h/$var$_$sdate$.nc'
path.obs <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$sdate$.nc'

var <- 'prlr'
y1 <- 1981
y2 <- 2016
months <- 12:1
seasons <- c('DJF', 'NDJ', 'OND', 'SON', 'ASO', 'JAS',
	     'JJA', 'MJJ', 'AMJ', 'MAM', 'FMA', 'JFM')
leadtimemin <- 1; leadtimemax <- 3

# SA domain

lons.min <- 0
lons.max <- 60
lats.min <- -40
lats.max <- 10

season <- 1

ini_month <- sprintf("%02d", months[season])
sdate <- paste0(y1:y2, ini_month, '01')

exp <- Start(data = path.exp,
	     var = var,
	     member = 'all',
	     sdate = sdate,
	     time = leadtimemin:leadtimemax,
	     latitude = values(list(lats.min, lats.max)),
	     latitude_reorder = Sort(decreasing = TRUE),
	     longitude = values(list(lons.min, lons.max)),
	     synonims = list(longitude = c('lon', 'longitude'),
			     latitude = c('lat', 'latitude'),
			     member = c('member', 'ensemble')),
	     return_vars = list(longitude = NULL,
				latitude = NULL,
				time = 'sdate'),
	     transform = CDORemapper,
	     transform_params = list(grid = 'r1440x721',
				     method = 'bicubic',
				     crop = c(lons.min, lons.max, lats.min, lats.max)),
	     transform_vars = c('latitude', 'longitude'),
	     retrieve = FALSE)

lons <- attr(exp, 'Variables')$common$longitude
lats <- attr(exp, 'Variables')$common$latitude
dates <- attr(exp, 'Variables')$common$time


obs <- Start(data = path.obs,
	     var = var,
	     sdate = unique(format(dates, '%Y%m')),
	     time = values(dates),
	     latitude = values(lats),
	     longitude = values(lons),
	     time_across = 'sdate',
	     merge_across_dims = TRUE,
	     split_multiselected_dims = TRUE,
	     synonims = list(longitude = c('lon', 'longitude'),
			     latitude = c('lat', 'latitude')),
	     return_vars = list(longitude = NULL,
				latitude = NULL,
				time = 'sdate'),
	     retrieve = FALSE)

# change 'sdate' to 'date' is fine. It returns an error:
#"Size of selector file dimensions must mach size of requested variable dimensions."
