# This script compares the SEAS5 prlr outputs from CST_Load and startR

rm(list = ls())
library(startR)
library(CSTools)
library(s2dv)


# startR

path.exp <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_s0-24h/$var$_$sdate$.nc'
path.exp <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc'
y1 <- 1981
y2 <- 2016
mon <- c(12, 3, 6, 8)
seasons <- c('DJF', 'MAM', 'JJA', 'SON')
leadtimemin <- 1; leadtimemax <- 3
season <- 1
var <- 'prlr'
ini_month <- sprintf("%02d", mon[season])
sdate <- paste0(y1:y2, ini_month, '01')

# SA domain
lons.min <- 0
lons.max <- 359.99 #60
lats.min <- -90 #-40
lats.max <- 90 #10

grid <- 'r1440x721'
grid <- 'r100x50'
method <- 'bicubic'
method <- 'bilinear'
method <- 'conservative'

data <- Start(data = path.exp, var = var, sdate = sdate[1],
              member = 1, time = indices(1), #leadtimemin:leadtimemax,
              longitude = values(list(lons.min, lons.max)),
              latitude = values(list(lats.min, lats.max)),
              return_vars = list(latitude = 'data', longitude = 'data', time = 'sdate'),
              synonims = list(latitude = c('lat', 'latitude'), longitude = c('lon', 'longitude'),
                              member = c('member', 'ensemble')),
              transform = CDORemapper,
              transform_params = list(grid = grid,
                                      method = method,
                                      crop = c(lons.min, lons.max, lats.min, lats.max)),
              transform_vars = c('latitude', 'longitude'),
              retrieve = TRUE)

##############################################################

# CST_Load

path.exp2 <- c('/esarchive/exp/ecmwf/system5c3s/monthly_mean/$VAR_NAME$_s0-24h/$VAR_NAME$_$YEAR$$MONTH$01.nc')
path.exp2 <- c('/esarchive/exp/ecmwf/system5c3s/monthly_mean/$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$01.nc')
data1 <- CST_Load(var = var, exp = list(list(path = path.exp2)),
                 sdates = sdate[1], nmember = 1,
                 leadtimemin = 1, leadtimemax = 1, output = 'lonlat',
                 storefreq = 'monthly',
                 latmin = lats.min, latmax = lats.max,
                 lonmin = lons.min,  lonmax = lons.max,
                 grid = grid, method = method)


# check
lon <- as.numeric(attributes(data)$Variables$dat1$longitude)
lat <- as.numeric(attributes(data)$Variables$dat1$latitude)
#data[1,1,,1,1,1,1]

lon1 <- as.vector(data1$lon)
lat1 <- as.vector(data1$lat)
#data1$data[1,1,,1,1,1]

#WRONG!!!! lat is reverse!!!!!!!!!!!!----------------------------
summary(as.vector(t(drop(data)) - drop(data1$data)))
## bicubic, r1440x721
#      Min.    1st Qu.     Median       Mean    3rd Qu.       Max. 
#-1.684e-07 -2.078e-08 -1.700e-13 -3.210e-11  2.075e-08  1.684e-07 
## bicubic, r100x50
#      Min.    1st Qu.     Median       Mean    3rd Qu.       Max. 
#-1.470e-07 -1.968e-08 -3.100e-13  3.000e-14  1.968e-08  1.470e-07 
## bilinear, r1440x721
#      Min.    1st Qu.     Median       Mean    3rd Qu.       Max. 
#-1.658e-07 -2.057e-08 -1.000e-14  0.000e+00  2.057e-08  1.658e-07 
## bilinear, r100x50
#      Min.    1st Qu.     Median       Mean    3rd Qu.       Max. 
#-1.425e-07 -1.924e-08  1.000e-14 -3.000e-14  1.924e-08  1.425e-07 
#------------------------------------------------------------------

s2dv::PlotEquiMap(t(drop(data)), lon = lon, lat = lat, 
#                  bar_limits = c(-8.5e-9, 2.2e-7),
                  toptitle = "startR", fileout = '~/issue105_startR.png')
s2dv::PlotEquiMap(drop(data1$data), lon = lon1, lat = lat1, 
#                  bar_limits = c(-8.5e-9, 2.2e-7),
                  toptitle = "CS_Load", fileout = '~/issue105_CSLoad.png')
tmp <- (t(drop(data)) - drop(data1$data)[14:1 , ])/drop(data1$data)[14:1 , ]*100
tmp[!is.finite(tmp)] <- NA
s2dv::PlotEquiMap(t(drop(data)) - drop(data1$data)[14:1, ],
                  lon = lon, lat = lat,
#                  bar_limits = c(-1.77e-7, 1.77e-7), 
                  toptitle = "(startR - CS_Load)/CS_Load*100", fileout = '~/issue105_diff.png')


#==================================
vari <- 'tos'
path_obs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/', vari, '_f1h-r1440x721cds/$var$_$year$.nc')
#path_obs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/', vari, '/$var$_$year$$month$.nc')

#startdates_obs  <- paste0(c(1980)) 
lon_min <- 0
lon_max <- 359.9 #60
lat_min <- -90 #-40
lat_max <- 90 #10

data_obs <- Start(dat = path_obs,
                                   var = vari,
                                   year = '198001',
                                   time = indices(1), #'all', 
                                   lat = values(list(lat_min, lat_max)),
                                   lon = values(list(lon_min, lon_max)),
                                   transform = CDORemapper,
                                   transform_extra_cells = 2,
                                   transform_params = list(grid = 'r100x50',
                                                           method = 'bil', #'bicubic', #'con', 
                                                           crop = F),
#                                                           crop = c(lon_min, lon_max, 
#                                                                    lat_min, lat_max)),
                                   transform_vars = c('lat', 'lon'),
                                   synonims = list(lon = c('lon', 'longitude'),
                                                   lat = c('lat', 'latitude')),
                                   return_vars = list(lat = NULL, lon = NULL,
                                                      time = 'year'),
                                   lon_reorder = startR::CircularSort(0, 360),
                                   lat_reorder = startR::Sort(decreasing = T),
                                   retrieve = TRUE)

library(s2dv)
obs_era <- list(name = 'era5',
                 path = file.path('/esarchive/recon/ecmwf/era5/monthly_mean',
                                  '$VAR_NAME$_f1h-r1440x721cds/$VAR_NAME$_$YEAR$$MONTH$.nc'))
data_obs2 <- Load(var = vari,
             exp = NULL,
             obs = list(obs_era),
             sdates= '19800101', #startdates_obs,
             nmember=1,
             leadtimemin = 1,
             leadtimemax = 1,
             latmin = lat_min, latmax = lat_max,
             lonmin = lon_min,  lonmax = lon_max,
             output = 'lonlat',
             grid = 'r100x50',
             method = 'bilinear', 
i#             method = 'bicubic', #'conservative',
             storefreq = 'monthly',
             nprocs= 1)

lon <- as.numeric(attributes(data_obs)$Variables$common$lon)
lat <- as.numeric(attributes(data_obs)$Variables$common$lat)

lon1 <- as.vector(data_obs2$lon)
lat1 <- as.vector(data_obs2$lat)



s2dv::PlotEquiMap(drop(data_obs), lon = lon, lat = lat,
                  bar_limits = c(288, 303),
                  toptitle = "startR", fileout = '~/issue103_startR.png')
s2dv::PlotEquiMap(drop(data_obs2$obs), lon = lon1, lat = lat1,
                  bar_limits = c(288, 303),
                  toptitle = "Load", fileout = '~/issue103_Load.png')

s2dv::PlotEquiMap((drop(data_obs) - drop(data_obs2$obs))/drop(data_obs2$obs)*100,
                  lon = lon, lat = lat,
                  bar_limits = c(-0.4, 0.4),
                  toptitle = "(startR - Load)/Load*100", fileout = '~/issue103_diff.png')

