library(startR)

path.obs <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$date$.nc'
var <- 'tos'
y1 <- 1981
y2 <- 2016
months <- 12:1
seasons <- c('DJF', 'NDJ', 'OND', 'SON', 'ASO', 'JAS',
             'JJA', 'MJJ', 'AMJ', 'MAM', 'FMA', 'JFM')

indices <- c('NINO34', 'ALT3')

# Domains: NINO34, ALT3 

lons.min <- c(360 - 170, 360 - 20)
lons.max <- c(360 - 120, 360)
lats.min <- c(-5, -3)
lats.max <- c(5, 3)


index <- 1

season = 3
ini_month <- sprintf("%02d", months[season])

dates.obs <- c(paste0(y1:y2, ini_month, '01'),
                      paste0(y1:y2, months[season] + 1, '01'),
                      paste0(y1:y2, months[season] + 2, '01'))

time.obs <- as.POSIXct(dates.obs, "%Y%m%d",
                       origin = paste0(y1,'-', ini_month), tz = 'UTC')
dim(time.obs) <- c(sdate = 36, time = 3)

# Load observation

obs <- Start(data = path.obs,
             var = var,
             date = unique(format(time.obs, '%Y%m')),
             time = values(time.obs),
             latitude = values(list(lats.min[index], lats.max[index])),
             latitude_reorder = Sort(decreasing = TRUE),
             longitude = values(list(lons.min[index], lons.max[index])),
             time_across = 'date',
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude')),
             return_vars = list(longitude = NULL,
                                latitude = NULL,
                                time = 'date'),
             retrieve = TRUE)

dates.obs <- attr(obs, 'Variables')$common$time
print(dates.obs)
length(dates.obs)
length(time.obs)

# netCDF file tos_200412.nc has "2004-12-16 11:30"
difftime(as.POSIXct("2004-12-16 11:30", tz = 'UTC'), as.POSIXct("2004-12-01", tz = 'UTC'))
#Time difference of 15.47917 days

# netCDF file tos_200411.nc has "2004-11-15 23:30"
difftime(as.POSIXct("2004-11-15 23:30", tz = 'UTC'), as.POSIXct("2004-12-01", tz = 'UTC'))
#Time difference of -15.02083 days

#----------

# netCDF file tos_198111.nc has "1981-11-14 23:30"
difftime(as.POSIXct("1981-11-14 23:30", tz = 'UTC'), as.POSIXct("1981-11-01", tz = 'UTC'))
#Time difference of 13.97917 days

# netCDF file tos_198110.nc has "1981-10-15 23:30"
difftime(as.POSIXct("1981-10-15 23:30", tz = 'UTC'), as.POSIXct("1981-11-01", tz = 'UTC'))
#Time difference of -16.02083 days
