rm(list = ls())
library(startR)
library(s2dv)
library(ClimProjDiags)

path.output <- '/esarchive/scratch/cchou/FOCUS/teleconnection/output/'
path.exp <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc'
path.obs <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$date$.nc'

var <- 'tos'
y1 <- 1981
y2 <- 2016
months <- 12:1
seasons <- c('DJF', 'NDJ', 'OND', 'SON', 'ASO', 'JAS',
             'JJA', 'MJJ', 'AMJ', 'MAM', 'FMA', 'JFM')
leadtimemin <- 1; leadtimemax <- 3

indices <- c('NINO34', 'ALT3')

# Domains: NINO34, ALT3 

lons.min <- c(360 - 170, 360 - 20)
lons.max <- c(360 - 120, 360)
lats.min <- c(-5, -3)
lats.max <- c(5, 3)

season <- 1
index <- 1

ini_month <- sprintf("%02d", months[season])
sdate <- paste0(y1:y2, ini_month, '01')

# Load prediction

exp <- Start(data = path.exp,
             var = var,
             member = 'all',
             sdate = sdate,
             time = leadtimemin:leadtimemax,
             latitude = values(list(lats.min[index], lats.max[index])),
             latitude_reorder = Sort(decreasing = TRUE),
             longitude = values(list(lons.min[index], lons.max[index])),
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude'),
                             member = c('member', 'ensemble')),
             return_vars = list(longitude = NULL,
                                latitude = NULL,
                                time = 'sdate'),
             retrieve = FALSE)

lons.exp <- attr(exp, 'Variables')$common$longitude
lats.exp <- attr(exp, 'Variables')$common$latitude
dates.exp <- attr(exp, 'Variables')$common$time

# Manually create date/ime

dates.obs <- c(paste0(y1:y2, ini_month, '01'),
               paste0((y1+1):(y2+1), '01', '01'),
               paste0((y1+1):(y2+1), '02', '01'))
time.obs <- as.POSIXct(dates.obs, "%Y%m%d",
                       origin = "1981-12", tz = 'UTC')
dim(time.obs) <- c(dim(dates.exp)['sdate'], dim(dates.exp)['time'])

# Load observation

obs <- Start(data = path.obs,
             var = var,
             date = unique(format(time.obs, '%Y%m')),
             time = values(time.obs),
             latitude = values(list(lats.min[index], lats.max[index])),
             latitude_reorder = Sort(decreasing = TRUE),
             longitude = values(list(lons.min[index], lons.max[index])),
             time_across = 'date',
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude')),
             return_vars = list(longitude = NULL,
                                latitude = NULL,
                                time = 'date'),
             retrieve = FALSE)

lons.obs <- attr(obs, 'Variables')$common$longitude
lats.obs <- attr(obs, 'Variables')$common$latitude
dates.obs <- attr(obs, 'Variables')$common$time


lons_exp <- as.vector(lons.exp)
lats_exp <- as.vector(lats.exp)
lons_obs <- as.vector(lons.obs)
lats_obs <- as.vector(lats.obs)

# Compute the index

fun <- function(exp, obs, path.output, y1, y2, seasons, indices,
                season, index, lons_exp = lons_exp, lats_exp = lats_exp,
                lons_obs = lons_obs, lats_obs = lats_obs) {

        e <- MeanDims(drop(exp), c('member','time'))
        sst.e <- WeightedMean(e, lons_exp, lats_exp,
                              londim = which(names(dim(e)) == 'longitude'),
                              latdim = which(names(dim(e)) == 'latitude'))
        index.exp <- (sst.e - mean(sst.e))/sd(sst.e)

        lons.obs <- as.vector(attr(obs, 'Variables')$common$longitude)
        lats.obs <- as.vector(attr(obs, 'Variables')$common$latitude)
        o <- MeanDims(drop(obs), 'time')
        sst.o <- WeightedMean(o, lons_obs, lats_obs,
                              londim = which(names(dim(o)) == 'longitude'),
                              latdim = which(names(dim(o)) == 'latitude'))
        index.obs <- (sst.o - mean(sst.o))/sd(sst.o)

        # give dim name
        dim(index.exp) <- c(sdate = length(index.exp))
        dim(index.obs) <- c(sdate = length(index.obs))

        return(list(ind_exp = index.exp, ind_obs = index.obs))

}

step <- Step(fun,
             target_dims = list(exp = c('member', 'sdate', 'time', 'latitude', 'longitude'),
                                obs = c('sdate', 'time', 'latitude', 'longitude')),
             output_dims = list(ind_exp = 'sdate', ind_obs = 'sdate'),
             use_libraries = c('CSTools', 's2dv', 'ClimProjDiags'))

workflow <- AddStep(list(exp = exp, obs = obs), step,
                    path.output = path.output, y1 = y1, y2 = y2,
                    seasons = seasons, indices = indices,
                    season = season, index = index, lons_exp = lons_exp, lats_exp = lats_exp,
                    lons_obs = lons_obs, lats_obs = lats_obs)

res <- Compute(workflow$ind_exp,
               chunks = list(var = 1),
               threads_load = 2,
               threads_compute = 4)


# Apply() can accept different length of lat and lon of exp and obs.
exp1 <- eval(exp)
obs1 <- eval(obs)
asd <- Apply(list(exp1, obs1), fun = fun, 
             target_dims = list(exp = c('member', 'sdate', 'time', 'latitude', 'longitude'),
                                obs = c('sdate', 'time', 'latitude', 'longitude')), 
             path.output = '~/', y1 = y1, y2 = y2, seasons = seasons, indices = indices,
             season = season, index = index, lons_exp = lons_exp, lats_exp = lats_exp,
             lons_obs = lons_obs, lats_obs = lats_obs)

