#library(lubridate)
library(startR)
library(s2dverification)
#library(multiApply)

source('/home/Earth/aho/aho-testtest/startR/issue/issue37/Start.R')
source('/home/Earth/aho/aho-testtest/startR/issue/issue37/Load.R')

#Start, fine res
obs_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc'  #fine res
var_name <- 'sfcWind'
sdate <- '201811'
#Load, fine res
pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')
sdates <- '20181101'
#Start, coarse res
obs_path <- '/esarchive/recon/ecmwf/era20c/monthly_mean/$var$_f3h/$var$_$sdate$.nc'  #coase res
var_name <- 'tas'
sdate <- '201011'
#Load, coarse res
pobs <- paste0('/esarchive/recon/ecmwf/era20c/monthly_mean/',
               '$VAR_NAME$_f3h/$VAR_NAME$_$YEAR$$MONTH$.nc')
sdates <- '20101101'

y1 <- 1993
y2 <- 1995

lons.min <- 10
lons.max <- 20 
lats.min <- 0
lats.max <- 10 

#-------------------------------
# Interpolation
#-------------------------------

obs <- Start(dat = obs_path,
             var = var_name,
#             sdate = paste0(y1:y2, '12'),
             sdate = sdate,
             time = 'all',
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),  #obs3 without this line
             longitude = values(list(lons.min, lons.max)),
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude')),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r360x181',
                                     method = 'conservative',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)), #obs2 without this line
             transform_vars = c('latitude', 'longitude'),
             return_vars = list(time = NULL,
                                latitude = 'dat',
                                longitude = 'dat'),
             split_multiselected_dims = TRUE,
             merge_across_dims = TRUE,
             #num_procs = 4,
             retrieve = T)

# load + interpolate data s2dverification
obs1 <- Load(var = var_name,
             obs = list(list(path = pobs)),
             sdates = sdates,
#             sdates = paste0(y1:y2, '1201'),
             leadtimemin = 1,
             leadtimemax = 1,
             output = 'lonlat',
             grid = 'r360x181',
             method = 'conservative',
             storefreq = 'monthly',
             latmin = lats.min,
             latmax = lats.max,
             lonmin = lons.min,
             lonmax = lons.max)


## use: cdo sellonlatbox,0,10,0,10 -remapcon,r360x181 sfcWind_201811_regrid.nc sfcWind_201811_regrid_sub2.nc

library(easyNCDF)
  path <- '/home/Earth/aho/aho-testtest/startR/issue/issue37/sfcWind_201811_regrid_sub2.nc'
  fnc <- NcOpen(path)
  fnc_dims <- NcReadDims(fnc)
  fnc_arr <- NcToArray(fnc, vars_to_read = var_name)


##Compare
#--Start--obs--------------------------------------------------
dim(obs)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        11        11 
range(obs)
#[1] 1.090472 4.987585
str(attr(obs, 'Variables')$common[[var_name]]$dim[[1]])
# $ name         : chr "longitude"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
str(attr(obs, 'Variables')$common[[var_name]]$dim[[2]])
# $ name         : chr "latitude"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
obs[1,1,1,1,,1]
# [1] 4.987585 4.299388 4.007287 3.758497 3.673881 3.908989 2.224407 1.774090
# [9] 1.800998 1.718733 1.816074


#--Start--obs2-------------------------------------------------
dim(obs2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        10        10 
range(obs2)
#[1] 1.177485 4.280102
str(attr(obs2, 'Variables')$common[[var_name]]$dim[[1]])
# $ name         : chr "longitude"
# $ vals         : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
str(attr(obs2, 'Variables')$common[[var_name]]$dim[[2]])
# $ name         : chr "latitude"
# $ vals         : num [1:10(1d)] 1 2 3 4 5 6 7 8 9 10
obs2[1,1,1,1,,1]
# [1] 4.280102 4.002453 3.771909 3.705606 3.982399 2.499812 1.827304 1.793729
# [9] 1.684370 1.790771  #consistent with obs, just miss lat=0

#--Start--obs3-------------------------------------------------
dim(obs3)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        11        11 
range(obs3)
#[1] 0.9431505 4.9703207
str(attr(obs3, 'Variables')$common[[var_name]]$dim[[1]])
# $ name         : chr "longitude"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
str(attr(obs3, 'Variables')$common[[var_name]]$dim[[2]])
# $ name         : chr "latitude"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
obs3[1,1,1,1,,1]
# [1] 4.970321 4.280102 4.002453 3.771909 3.705606 3.982399 2.499812 1.827304
# [9] 1.793729 1.684370 1.789749  #lat=0 and lat=11 different from obs

#--Load--------------------------------------------------------
dim(obs1$obs)
#dataset  member   sdate   ftime     lat     lon 
#      1       1       1       1      11      11 
range(obs1$obs)
#[1] 1.0905 4.9876
str(obs1)
# $ lon            : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
# $ lat            : num [1:11(1d)] 10 9 8 7 6 5 4 3 2 1 ...
obs1$obs[1,1,1,1,,1]  #when lon = 0
# [1] 1.8161 1.7187 1.8010 1.7741 2.2244 3.9090 3.6739 3.7585 4.0073 4.2994
#[11] 4.9876


#--cdo----------------------------------------------------------
dim(fnc_arr)
# var  lon  lat time 
#   1   11   11    1 
range(fnc_arr)
#[1] 1.090472 4.987585
str(attr(fnc_arr, 'variables')[[var_name]]$dim[[1]])
# $ name         : chr "lon"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
str(attr(fnc_arr, 'variables')[[var_name]]$dim[[2]])
# $ name         : chr "lat"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
fnc_arr[1,1,,1]
# [1] 4.987585 4.299388 4.007286 3.758497 3.673881 3.908989 2.224407 1.774090
# [9] 1.800998 1.718733 1.816074

##diff
diff <- drop(obs)[11:1, ] - drop(obs1$obs)  #lat order is different
range(diff)
#[1] -4.918404e-05  4.693756e-05

fnc_arr2 <- s2dverification:::.aperm2(fnc_arr, c(4,3,2,1))
diff2 <- drop(fnc_arr2) - drop(obs)
range(diff2)
#[1] -0.8776584  0.2157096

diff3 <- drop(fnc_arr2)[11:1, ] - drop(obs1$obs)
range(diff3)
#[1] -4.910541e-05  4.688785e-05

obs1$obs[1,1,1,1,8:4,4:8]
#       [,1]   [,2]   [,3]   [,4]   [,5]
#[1,] 3.6679 3.5429 3.4524 3.4784 3.4559
#[2,] 3.6263 3.5246 3.3739 3.1945 3.3999
#[3,] 3.6761 3.5613 3.3384 1.6401 1.4778
#[4,] 3.7732 3.4240 2.2569 1.3835 1.5695
#[5,] 1.6333 1.4643 1.3805 1.6768 1.6859
obs[1,1,1,1,4:8,4:8]
#         [,1]     [,2]     [,3]     [,4]     [,5]
#[1,] 3.667865 3.542937 3.452411 3.478388 3.455891
#[2,] 3.626259 3.524614 3.373920 3.194497 3.399945
#[3,] 3.676072 3.561281 3.338372 1.640094 1.477809
#[4,] 3.773196 3.423993 2.256853 1.383469 1.569521
#[5,] 1.633261 1.464259 1.380528 1.676817 1.685917
range(obs[1,1,1,1,4:8,4:8] - obs1$obs[1,1,1,1,8:4,4:8])
#[1] -4.665794e-05  4.525909e-05
range(obs[1,1,1,1,11:1,] - obs1$obs[1,1,1,1,,])
#[1] -0.2157251  0.8776323


#============================================================

#-----------------------------------------
# Read in data first, then s2dv::CDORemap
#-----------------------------------------

obs_start <- Start(dat = obs_path,
                   var = var_name,
                   sdate = '201811',
#                   sdate = paste0(y1:y2, '12'),
                   time = 'all',
                   latitude = values(list(lats.min, lats.max)),
                   latitude_reorder = Sort(),
                   longitude = values(list(lons.min, lons.max)),
                   synonims = list(longitude = c('lon', 'longitude'),
                                   latitude = c('lat', 'latitude')),
                   return_vars = list(time = NULL,
                                      latitude = 'dat',
                                      longitude = 'dat'),
                   split_multiselected_dims = TRUE,
                   merge_across_dims = TRUE,
                   num_procs = 4,
                   retrieve = T)

#use s2dv::CDORemap to interpolate
lons <- as.vector((attr(obs_start, 'Variables')$dat1$longitude))
lats <- as.vector((attr(obs_start, 'Variables')$dat1$latitude))
obs_start_2 <- s2dverification::CDORemap(obs_start, lons, lats, 'r360x181', 'con', TRUE)

#--Start--obs--result--------------------------------------------
dim(obs_start_2$data_array)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        10        10 
range(obs_start_2$data_array)
#[1] 1.177485 4.280102
str(obs_start_2)
# $ lons      : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
# $ lats      : num [1:10(1d)] 1 2 3 4 5 6 7 8 9 10
obs_start_2$data_array[1,1,1,1,,1]
# [1] 4.280102 4.002453 3.771909 3.705606 3.982399 2.499812 1.827304 1.793729
# [9] 1.684370 1.790771  #same as obs (using CDORemapper in Start())

#============================================================

pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')

obs_s2dv <- Load(var = var_name,
                 obs = list(list(path = pobs)),
#                 sdates = paste0(y1:y2, '1201'),
                 sdates = '20181101',
                 leadtimemin = 1,
                 leadtimemax = 1,
                 output = 'lonlat',
                 storefreq = 'monthly',
                 latmin = lats.min,
                 latmax = lats.max,
                 lonmin = lons.min,
                 lonmax = lons.max)

#use s2dv::CDORemap to interpolate
lons <- as.vector(obs_s2dv$lon)
lats <- as.vector(obs_s2dv$lat)  #from 9.9765777 to 0.1405152!! just like Start()
obs_s2dv_2 <- s2dverification::CDORemap(obs_s2dv$obs, lons, lats, 'r360x181', 'con', TRUE)

#--Load--result-------------------------------------------------
dim(obs_s2dv_2$data_array)
#dataset  member   sdate   ftime     lat     lon 
#      1       1       1       1       9      10 
range(obs_s2dv_2$data_array)
#[1] 1.177490 4.280105
str(obs_s2dv_2)
# $ lons      : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
# $ lats      : num [1:9(1d)] 1 2 3 4 5 6 7 8 9
obs_s2dv_2$data_array[1,1,1,1,,1]  #when lon = 0
# [1] 4.280105 4.002460 3.771910 3.705595 3.982394 2.499798 1.827297 1.793722
# [9] 1.684371  ##different from others

#============================================================

obs_s2dv_3 <- s2dverification::CDORemap(obs_s2dv$obs, lons, rev(lats), 'r360x181', 'con', TRUE)  ##reverse lats in CDORemap() i.e., make lat from small to large

#--Load--result------------------------------------------------
dim(obs_s2dv_3$data_array)
#dataset  member   sdate   ftime     lat     lon 
#      1       1       1       1      10      10 
range(obs_s2dv_3$data_array)
#[1] 1.174429 4.728291
str(obs_s2dv_3)
# $ lons      : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
# $ lats      : num [1:10(1d)] 1 2 3 4 5 6 7 8 9 10
obs_s2dv_3$data_array[1,1,1,1,,1]  #when lon = 0
# [1] 1.703802 1.760790 1.825117 2.210783 4.055306 3.708123 3.760025 3.971781
# [9] 4.228549 4.728291

#============================================================

tmp <- Subset(obs_s2dv$obs, 'lat', list(36:1))  #reverse the order of lat back to normal
obs_s2dv_4 <- s2dverification::CDORemap(tmp, lons, lats, 'r360x181', 'con', TRUE)

#--Load--result------------------------------------------------
dim(obs_s2dv_4$data_array)
#dataset  member   sdate   ftime     lat     lon 
#      1       1       1       1      10      10 
range(obs_s2dv_4$data_array)
#[1] 1.177490 4.280105
str(obs_s2dv_4)
# $ lons      : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
# $ lats      : num [1:10(1d)] 1 2 3 4 5 6 7 8 9 10
range(obs_s2dv_4$data_array - obs_start_2$data_array)
#[1] -1.502037e-05  1.740456e-05


