# Load startR
library(startR)

# Define a region
lons.min <- 0
lons.max <- 10
lats.min <- 0
lats.max <- 10

# Declaration of data sources
exp <- Start(dat = '/esarchive/exp/ecmwf/system4_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = paste0(2000:2010, '0101'),
             ensemble = 'all',
             time = indices(1),
#             latitude = values(list(lats.min, lats.max)),
#             latitude_reorder = Sort(),
#             longitude = values(list(lons.min, lons.max)),
             latitude = indices(129:142),
             longitude = indices(1:15),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = FALSE)

obs <- Start(dat = '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = paste0(2000:2010, '02'),
             time = indices(1),
#             latitude = values(list(lats.min, lats.max)),
#             latitude_reorder = Sort(),
#             longitude = values(list(lons.min, lons.max)),
             latitude = indices(129:142),
             longitude = indices(1:15),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             split_multiselected_dims = TRUE,
             merge_across_dims = TRUE,
             retrieve = FALSE)

# Define of the workflow

# Function
wrap_cal <- function(obs, exp) {
    obs <- s2dverification::InsertDim(obs, 1, 1)
    names(dim(obs))<- c('member', 'sdate')
    exp <- t(exp)
    names(dim(exp))<- c('member', 'sdate')
    calibrated <- CSTools:::.cal(var_obs = obs, var_exp = exp) # CSTools version 1.0.1 or earlier
    # calibrated <- CSTools:::.cal(mod = exp, obs = obs,
    #                             cal.method = "mse_min",
    #                             eval.method = "leave-one-out",
    #                             multi.model = FALSE)
    return(calibrated)
}

step <- Step(wrap_cal,
             target_dims = list(obs = c('sdate'), exp = c('sdate', 'ensemble')),
             output_dims = c('ensemble', 'sdate'))

# workflow of operations
wf <- AddStep(list(obs = obs, exp = exp), step)


# Execution
res1 <- Compute(wf, chunks = list(latitude = 2, longitude = 2))
#               threads_load = 2, threads_compute = 4)
