
#-----Test the number rule for range--------
lons.min <- 10
lons.max <- 20
lats.min <- 50
lats.max <- 70

#--------------------
#  Start
#--------------------
obs_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc'
var_name <- 'tas'#'sfcWind'
sdate <- '201811'

obs_start <- Start(dat = obs_path,
                   var = var_name,
                   sdate = '201811',
#                   sdate = paste0(y1:y2, '12'),
                   time = 'all',
                   latitude = values(list(lats.min, lats.max)),
                   #latitude = indices(180:225),
                   latitude_reorder = Sort(),
                   longitude = values(list(lons.min, lons.max)),
                   #longitude_reorder = Sort(),
                   #longitude_reorder = CircularSort(lons.max,lons.min),
                   synonims = list(longitude = c('lon', 'longitude'),
                                   latitude = c('lat', 'latitude')),
                   return_vars = list(time = NULL,
                                      latitude = 'dat',
                                      longitude = 'dat'),
                   split_multiselected_dims = TRUE,
                   merge_across_dims = TRUE,
                   num_procs = 4,
                   retrieve = T)


#--------------------
#  Load
#--------------------
pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')
sdates <- '20181101'

obs_s2dv <- Load(var = var_name,
                 obs = list(list(path = pobs)),
#                 sdates = paste0(y1:y2, '1201'),
                 sdates = '20181101',
                 leadtimemin = 1,
                 leadtimemax = 1,
                 output = 'lonlat',
                 storefreq = 'monthly',
                 latmin = lats.min,
                 latmax = lats.max,
                 lonmin = lons.min,
                 lonmax = lons.max)


#------------------
#  Results
#------------------

#---------------------About lat-----------------------
#---------------------Workable cases------------------
## 1 (no prob)
lons.min <- 10 #0
lons.max <- 20
lats.min <- 0
lats.max <- 10

range(as.vector(attr(obs_start, 'Variables')$dat1$latitude))
#[1] 0.1405152 9.9765777
str(attr(attr(obs_start, 'Variables')$dat1$latitude, 'variables')$latitude$dim[[1]]$vals)
# num [1:640(1d)] 89.8 89.5 89.2 88.9 88.7 ...

range(as.vector(obs_s2dv$lat))
#[1] 0.1405152 9.9765777


## 2 (no prob)
lons.min <- 0
lons.max <- 20
lats.min <- -10
lats.max <- 10

range(as.vector(attr(obs_start, 'Variables')$dat1$latitude))
#[1] -9.976578  9.976578

range(as.vector(obs_s2dv$lat))
#[1] -9.976578  9.976578


## 3 (no prob)
lons.min <- 0
lons.max <- 10
lats.min <- 0
lats.max <- 90

range(as.vector(attr(obs_start, 'Variables')$dat1$latitude))
#[1]  0.1405152 89.7848769
range(as.vector(obs_s2dv$lat))
#[1]  0.1405152 89.7848769

#---------------------Prob lat-----------------------
## 4 (prob lat: lats.max has no upper bound)
lons.min <- 0
lons.max <- 10
lats.min <- 0
lats.max <- 91 #any number > 90, unlimited!!!! the min value will be 90 anyway

range(as.vector(attr(obs_start, 'Variables')$dat1$latitude))
#[1]  0.1405152 89.7848769

#Error in try({ : Error: 'latmax' must be in the interval [-90, 90].


## 5 (prob lat: lats.min has no lower bound)
lons.min <- 0
lons.max <- 10
lats.min <- -91 #any number < -90, unlimited!!!! the min value will be -90 anyway
lats.max <- 0

range(as.vector(attr(obs_start, 'Variables')$dat1$latitude))
#[1] -89.7848769  -0.1405152

#Error in try({ : Error: 'latmax' must be in the interval [-90, 90].



## 6 (prob lat: lat.min cannot >= 90, lat.max cannot <= -90)
lons.min <- 0
lons.max <- 10
lats.min <- 90  #any number >= 90
lats.max <- 100
#OR
lons.min <- 0
lons.max <- 10
lats.min <- -1000  
lats.max <- -91  #any number <= -90

#Error in sub_array_of_indices[[1]]:sub_array_of_indices[[2]] : 
#  NA/NaN argument


## 7 (prob(?) lat: lat can reverse the order)
lons.min <- 0
lons.max <- 10
lats.min <- 80
lats.max <- 70  ##the order will be opposite!!!!

range(as.vector(attr(obs_start, 'Variables')$dat1$latitude))
#[1] 70.11705 79.95306
which(asd == obs_start[1,1,1,1,36,])
#[1] 1
which(asd == obs_start[1,1,1,1,1,])
#[1] 1296

#Error in try({ : 
#  Error: No found files for any dataset. Check carefully the file patterns and correct either the pattern or the provided parameters:
#/esarchive/recon/ecmwf/era5/monthly_mean/sfcWind_f1h/sfcWind_$YEAR$$MONTH$.nc
#
#In addition: Warning messages:
#1: In min(lat_indices) : no non-missing arguments to min; returning Inf
#2: In max(lat_indices) : no non-missing arguments to max; returning -Inf
#3: In min(lat_indices) : no non-missing arguments to min; returning Inf
#4: ! Warning: no data found in file system for any observational dataset. 

#---------------------About lon-----------------------
#---------------------Workable cases-----------------------

## 8 (lon no prob)
lons.min <- 0
lons.max <- 360
lats.min <- 0
lats.max <- 10

range(as.vector(attr(obs_start, 'Variables')$dat1$longitude))
#[1]   0.0000 359.7188
str(attr(attr(obs_start, 'Variables')$dat1$longitude, 'variables')$longitude$dim[[1]]$vals)
# num [1:1280(1d)] 0 0.281 0.562 0.844 1.125 ...

range(as.vector(obs_s2dv$lon))
#[1]   0.0000 359.7188


#---------------------Prob lon-----------------------
## 9 (prob lon: lons.max is unlimited!!! Will be 360 anyway)
lons.min <- 0
lons.max <- 1000
lats.min <- 0
lats.max <- 10

range(as.vector(attr(obs_start, 'Variables')$dat1$longitude))
#[1]   0.0000 359.7188

#Error in try({ : 
#  Error: parameter 'lonmax' must be in the range [-360, 360]


## 10 (prob lon: lons.min cannot >= 360)
lons.min <- 360
lons.max <- 1000
lats.min <- 0
lats.max <- 10

#Error in sub_array_of_indices[[1]]:sub_array_of_indices[[2]] : 
#  NA/NaN argument


## 11 (prob lon: lons.min < 0 does not have effect. the minimun will be 0 anyway)
lons.min <- -350
lons.max <- 0
lats.min <- 0
lats.max <- 10

range(as.vector(attr(obs_start, 'Variables')$dat1$longitude))
#[1] 0 0


## 12 (prob lon: lons.max cannot be < 0)
lons.min <- -350
lons.max <- -10
lats.min <- 0
lats.max <- 10
#Error in sub_array_of_indices[[1]]:sub_array_of_indices[[2]] : 
#  NA/NaN argument


## 13 (prob lon: lon can reverse. not cross greenwich)
lons.min <- 10
lons.max <- 0
lats.min <- 0
lats.max <- 10

range(as.vector(attr(obs_start, 'Variables')$dat1$longitude))
#[1] 0.00000 9.84375  #from 10 to 0
range(as.vector(obs_s2dv$lon))
##[1]   0.0000 359.7188  #from 20 to 0 cross greenwich

