
#-----Test the number rule for range--------
lons.min <- -10  #350
lons.max <- 20
lats.min <- 60
lats.max <- 70

#--------------------
#  Start
#--------------------
obs_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc'
var_name <- 'tas'#'sfcWind'
sdate <- '201811'

obs_start1 <- Start(dat = obs_path,
                   var = var_name,
                   sdate = '201811',
                   time = indices(1),
                   latitude = values(list(lats.min, lats.max)),
                   latitude_reorder = Sort(),
                   longitude = values(list(lons.min, lons.max)),
                   #longitude = indices(1270:1280),  #[0, 360]
                   #longitude_reorder = Sort(),
                   longitude_reorder = CircularSort(-180, 180),
                   synonims = list(longitude = c('lon', 'longitude'),
                                   latitude = c('lat', 'latitude')),
                   return_vars = list(time = NULL,
                                      latitude = 'dat',
                                      longitude = 'dat'),
                   split_multiselected_dims = TRUE,
                   merge_across_dims = TRUE,
                   num_procs = 4,
                   retrieve = T)


obs_start2 <- Start(dat = obs_path,
                   var = var_name,
                   sdate = '201811',
                   time = indices(1),
                   latitude = values(list(lats.min, lats.max)),
                   latitude_reorder = Sort(),
                   longitude = values(list(lons.min, lons.max)),
                   #longitude_reorder = Sort(),
                   longitude_reorder = CircularSort(0, 360),
                   synonims = list(longitude = c('lon', 'longitude'),
                                   latitude = c('lat', 'latitude')),
                   return_vars = list(time = NULL,
                                      latitude = 'dat',
                                      longitude = 'dat'),
                   split_multiselected_dims = TRUE,
                   merge_across_dims = TRUE,
                   num_procs = 4,
                   retrieve = T)

#--------------------
#  Load
#--------------------
pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')
sdates <- '20181101'

obs_s2dv <- Load(var = var_name,
                 obs = list(list(path = pobs)),
#                 sdates = paste0(y1:y2, '1201'),
                 sdates = '20181101',
                 leadtimemin = 1,
                 leadtimemax = 1,
                 output = 'lonlat',
                 storefreq = 'monthly',
                 latmin = lats.min,
                 latmax = lats.max,
                 lonmin = lons.min,
                 lonmax = lons.max)


#------------------
#  Results
#------------------

#---------------------About lat-----------------------
#---------------------Sort() and without Sort()------------------
## 1 
lons.min <- 10 #0
lons.max <- 20
lats.min <- 0
lats.max <- 10
### without 'latitude_reorder = Sort()'
dim(obs_start1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1       640        36 
### with 'latitude_reorder = Sort()'
dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36        36 
as.vector(attr(obs_start2, 'Variables')$dat1$latitude)
#  [1] 0.1405152 ... [36] 9.9765777

### Load
as.vector(obs_s2dv$lat)
# [1] 9.9765777 ... [36] 0.1405152

## 2 
lons.min <- 10
lons.max <- 20
lats.min <- 10
lats.max <- 0
### without 'latitude_reorder = Sort()'
dim(obs_start1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1       640        36 
### with 'latitude_reorder = Sort()'
dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36        36 
as.vector(attr(obs_start2, 'Variables')$dat1$latitude)
# [1] 9.9765777 ... [36] 0.1405152



#---------------------About lon-----------------------
#---------------------Sort() and without Sort()----------------------

## 3 
lons.min <- 10
lons.max <- 20
lats.min <- 0
lats.max <- 10
### with or without Sort() are the same
identical(obs_start1, obs_start2)
#[1] TRUE
dim(obs_start1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36        36 
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
# [1] 10.12500 ... [36] 19.96875

###Load
as.vector(obs_s2dv$lon)
# [1] 10.12500 ... [36] 19.96875

## 4
lons.min <- 20
lons.max <- 10
lats.min <- 0
lats.max <- 10
### with or without Sort() are the same
identical(obs_start1, obs_start2)
#[1] TRUE
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
# [1] 19.96875 ... [36] 10.12500


#---------------------CirlularSort() and without CirlularSort()----------------------

## 5 longitude_reorder = CircularSort(lons.min,lons.max)
lons.min <- 10
lons.max <- 20
lats.min <- 0
lats.max <- 10

dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36         4 
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
# [1] 10 10 10 10


## 6 longitude_reorder = CircularSort(lons.max,lons.min)
lons.min <- 10
lons.max <- 20
lats.min <- 0
lats.max <- 10

dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36         4 
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
# [1] 20 20 20 20


## 7 longitude_reorder = CircularSort(lons.min,lons.max)
lons.min <- 0
lons.max <- 360
lats.min <- 0
lats.max <- 10

dim(obs_start1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36      1280 
as.vector(attr(obs_start1, 'Variables')$dat1$longitude)
#[1] 0.00000 ... [1280] 359.71875
dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36         1 
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
# [1] 0


## 8 longitude_reorder = CircularSort(-180, 180)
lons.min <- 0
lons.max <- 180
lats.min <- 0
lats.max <- 10

dim(obs_start1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36       641 
as.vector(attr(obs_start1, 'Variables')$dat1$longitude)
# [1] 0 ... [641] 180.00000
dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36       641 
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
# [1] 0 ... [641] -180.00000


## 9 longitude_reorder = CircularSort(0, 360)
lons.min <- 0
lons.max <- 180
lats.min <- 0
lats.max <- 10
###same


## 9 longitude_reorder = CircularSort(0, 360)
lons.min <- 180
lons.max <- 0
lats.min <- 0
lats.max <- 10

dim(obs_start1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36       641 
as.vector(attr(obs_start1, 'Variables')$dat1$longitude)
# [1] 180.00000 ... [641] 0
dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36       641 
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
#  [1]   0.0000 180.0000 ... [641] 359.7188


## 10 longitude_reorder = CircularSort(0, 360)
lons.min <- 180
lons.max <- 1
lats.min <- 0
lats.max <- 10
dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36       645 
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
#  [1]   0.00000   0.28125   0.56250   0.84375   1.12500 180.00000 ... [645] 359.71875



## 11 longitude_reorder = CircularSort(-180, 180)
lons.min <- 180
lons.max <- 0
lats.min <- 0
lats.max <- 10
dim(obs_start2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        36      1921 
as.vector(attr(obs_start2, 'Variables')$dat1$longitude)
#   [1] -180.00000  -179.71875 ... [1279]  179.43750  179.71875



