library(startR)
library(s2dverification)
path_exp <- paste0('/esarchive/exp/ncar/cesm-dple/daily_mean/',
                   '$var$_f24h/$var$_$sdate$.nc')

variable<-'tasmax'
y1<-1961
y2<-2014

monini<-'11'
sdates_exp<-paste0(y1:y2,monini,'01')
nmemb<-2

lats.min=25
lats.max=30
lons.min=40
lons.max=44

data_exp <- Start(dat = list(list(path=path_exp)),
                  var = variable,
                  member = indices(1:nmemb),
                  time = indices(62:426),
                  sdate = sdates_exp,
                  latitude = 'all',  #values(list(lats.min, lats.max)),
                  longitude = 'all',  #values(list(lons.min, lons.max)),
                  longitude_reorder = CircularSort(0, 360),
                  synonims = list(latitude=c('lat','latitude'),
                                  longitude=c('lon','longitude'),
                                  member=c('ensemble','realization')),
                  return_vars = list(latitude = NULL,
                                     longitude = NULL,
                                     time = 'dat'),
                  retrieve = F)

hmd_fun <- function(x) {
  # Expected inputs:
  #   x: array with dimensions ('ensemble', 'time', 'sdate')
#  source("/home/Earth/bsolaraj/Documents/R/DecadalScript/JRC/DailyData/hmd_accum_mod.R")
   source("/home/bsc32/bsc32734/tmp/hmd_accum_mod.R")  # run in power9
  tasmax <- drop(x)
  memb <- dim(tasmax)[1]
  ftime_a <- dim(tasmax)[2]
  sdate_a <- dim(tasmax)[3]
  dim_days <- ftime_a*sdate_a
  dim(tasmax) <- c(memb,dim_days)
  
  Tref <- tasmax[,7300:18979]
  Temp <- tasmax
  
  hmd_all <- hmd_x(1980, Tref, 1961, Temp)
  return(hmd_all)
}

step <- Step(hmd_fun, 
             target_dims = c('member', 'time','sdate'), 
             output_dims = c('dim1','dim2', 'dim3'))

wf <- AddStep(data_exp, step)

# To use Compute() on Power9
#-------------------user-defined---------------------
  queue_host <- 'p1'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------
#  res1 <- Compute(wf,
#                 chunks = list(latitude = 2,
#                               longitude = 4),#$output1
#                 threads_load = 1,
#                 threads_compute = 4,
#
#                 cluster = list(queue_host = queue_host,
#                                queue_type = 'slurm',
#                                temp_dir = temp_dir,
#                                #temp_dir =  '/esarchive/scratch/aho/ecFlow', 
#                                lib_dir = '/gpfs/projects/bsc32/share/R_libs/3.5/',
#                                r_module = 'R/3.5.0-foss-2018b',
#                                job_wallclock = '03:30:00',
#                                cores_per_job = 4,
#                                max_jobs = 4, 
#                                bidirectional = FALSE,
#                                polling_period = 50
#                               ),
#                 ecflow_suite_dir = ecflow_suite_dir,
#                #ecflow_suite_dir = '/esarchive/scratch/aho/ecFlow/',
#                 wait = FALSE
#                 )  #$output1
##  res1 <- Collect(res1, wait = TRUE)

#  res2 <- Compute(wf,
#                 chunks = list(latitude = 2,
#                               longitude = 4),#$output1
#                 threads_load = 2,
#                 threads_compute = 4,
#
#                 cluster = list(queue_host = queue_host,
#                                queue_type = 'slurm',
#                                temp_dir = temp_dir,
#                                #temp_dir =  '/esarchive/scratch/aho/ecFlow', 
#                                lib_dir = '/gpfs/projects/bsc32/share/R_libs/3.5/',
#                                r_module = 'R/3.5.0-foss-2018b',
#                                job_wallclock = '03:30:00',
#                                cores_per_job = 4,
#                                max_jobs = 4,
#                                bidirectional = FALSE,
#                                polling_period = 50
#                               ),
#                 ecflow_suite_dir = ecflow_suite_dir,
#                #ecflow_suite_dir = '/esarchive/scratch/aho/ecFlow/',
#                 wait = FALSE
#                 )  #$output1
##  res2 <- Collect(res2, wait = TRUE)

#  res3 <- Compute(wf,
#                 chunks = list(latitude = 2,
#                               longitude = 4),#$output1
#                 threads_load = 3,
#                 threads_compute = 4,
#
#                 cluster = list(queue_host = queue_host,
#                                queue_type = 'slurm',
#                                temp_dir = temp_dir,
#                                #temp_dir =  '/esarchive/scratch/aho/ecFlow', 
#                                lib_dir = '/gpfs/projects/bsc32/share/R_libs/3.5/',
#                                r_module = 'R/3.5.0-foss-2018b',
#                                job_wallclock = '03:30:00',
#                                cores_per_job = 4,
#                                max_jobs = 4,
#                                bidirectional = FALSE,
#                                polling_period = 50
#                               ),
#                 ecflow_suite_dir = ecflow_suite_dir,
#                #ecflow_suite_dir = '/esarchive/scratch/aho/ecFlow/',
#                 wait = FALSE
#                 )  #$output1
##  res3 <- Collect(res3, wait = TRUE)


  res4 <- Compute(wf,
                 chunks = list(latitude = 2,
                               longitude = 4),#$output1
                 threads_load = 4,
                 threads_compute = 4,

                 cluster = list(queue_host = queue_host,
                                queue_type = 'slurm',
                                temp_dir = temp_dir,
                                #temp_dir =  '/esarchive/scratch/aho/ecFlow', 
                                lib_dir = '/gpfs/projects/bsc32/share/R_libs/3.5/',
                                r_module = 'R/3.5.0-foss-2018b',
                                job_wallclock = '03:30:00',
                                cores_per_job = 4,
                                max_jobs = 4,
                                bidirectional = FALSE,
                                polling_period = 50
                               ),
                 ecflow_suite_dir = ecflow_suite_dir,
                #ecflow_suite_dir = '/esarchive/scratch/aho/ecFlow/',
                 wait = FALSE
                 )  #$output1
#  res4 <- Collect(res4, wait = TRUE)

