library(startR)

var_name <- 'sfcWind'
path.exp <- '/esarchive/exp/ecmwf/s2s-monthly_ensforhc/daily/$var$_f24h/$sdate$/$var$_$syear$.nc'
#syear <- list(c('19961222', '19971222'), c('19961222', '19971222'))

data <- Start(dat = path.exp,
              var =  var_name,
              sdate = c('20161222', '20161223'),
              syear = indices(1), #syear,  #indices(1:2), 
              time = indices(1:11),             
              latitude = indices(1),
              longitude = indices(1),
              ensemble = indices(1), 
              syear_depends = 'sdate',
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = c('sdate','syear')
                                 ),
              retrieve = T)


not_found <- attr(data, "NotFoundFiles")
dim(not_found)
#  dat   var sdate syear 
#    1     1     2     1 


#///////////////////

hcst.path <- '/esarchive/exp/ecmwf/s2s-monthly_ensforhc/weekly_mean/$var$_f24h/20200305/$var$_$file_date$.nc'
file_dates <- c('20190305', '20200305')
variable <- 'sfcWind'

ltmin <- 1
ltmax <- 2
lats.min <- 10
lats.max <- 15
lons.min <- 10
lons.max <- 12
hcst.nmember <- 2

 hcst <- Start(dat = hcst.path,
              var = variable,
              file_date = file_dates,
              time = indices(ltmin:ltmax),
              latitude = values(list(lats.min, lats.max)),
              longitude = values(list(lons.min, lons.max)),
              synonims = list(latitude=c('lat','latitude'),
                              longitude=c('lon','longitude'),
                              member=c('ensemble')),
              member = indices(1:hcst.nmember),
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = 'file_date'),
              split_multiselected_dims = TRUE,
              retrieve = T)

not_found <- attr(hcst, "NotFoundFiles")
dim(not_found)
#      dat       var file_date 
#        1         1         2 





dates <- attr(hcst, 'Variables')$common$time
dates_file <- sapply(dates, format, '%Y%m%d')
dates_file <- format(as.Date(dates_file, '%Y%m%d'), "%Y%m%d")
dim(dates_file) <- dim(Subset(hcst,
                                along=c('dat','var',
                                'latitude', 'longitude', 'member'),
                                list(1,1,1,1,1), drop=T))
dim(dates_file)
#file_date      time 
#        2         2 
dates_file
#     [,1]       [,2]      
#[1,] "20190309" "20190316"
#[2,] NA         NA  

obs.path <- '/esarchive/recon/ecmwf/era5/weekly_mean/$var$_f1h-240x121/$var$_$file_date$.nc'
obs <- Start(dat = obs.path,
               var = variable,
             file_date = dates_file, #c('20190309', '20190332'), 
             latitude = values(list(lats.min, lats.max)),
             longitude = values(list(lons.min, lons.max)),
               synonims = list(latitude=c('lat','latitude'),
                               longitude=c('lon','longitude')),
             return_vars = list(latitude = 'dat',
                                  longitude = 'dat',
                                  time = 'file_date'),
             split_multiselected_dims = TRUE,
             retrieve = TRUE)
not_found2 <- attr(obs, "NotFoundFiles")
dim(not_found2)
#      dat       var file_date 
#        1         1         4 

# NOTE: when file dim is not vector (ie., multidim), it remains $file_date$

