library(startR)

# exp
repos_exp <- paste0('/esarchive/exp/ecearth/a1tr/cmorfiles/CMIP/EC-Earth-Consortium/',
                    'EC-Earth3/historical/r24i1p1f1/Amon/$var$/gr/v20190312/',
                    '$var$_Amon_EC-Earth3_historical_r24i1p1f1_gr_$sdate$01-$sdate$12.nc')

exp <- Start(dat = repos_exp,
             var = 'tas',
             sdate = as.character(c(2005:2008)),
             time = indices(1:3),
             lat = 'all',
             lon = 'all',
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'sdate'),
             retrieve = FALSE)


lats <- attr(exp, 'Variables')$common$lat
# The 'time' attribute is dependent on 'sdate'. You can see the dimension below.
dates <- attr(exp, 'Variables')$common$time

######## Andrea
file_date <- sapply(dates, format, '%Y%m')
dim(file_date) <- dim(dates)
########

repos_obs <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$file_date$.nc'

obs <- Start(dat = repos_obs,
             var = 'tas',
             file_date = file_date,
#             date = unique(format(dates, '%Y%m')),
#             time = values(dates),  #dim: [sdate = 4, time = 3]
             lat = values(lats),
             lon = 'all',
#             time_across = 'date',
#             merge_across_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = 'dat',
                                lat = 'dat',
                                time = 'file_date'),
             split_multiselected_dims = TRUE,
             retrieve = FALSE)

wrap_cal <- function(obs) {
print(dim(obs))
  return(obs)
}

step <- Step(wrap_cal,
             target_dims = c('time'),
             output_dims = c('time')
        )

wf <- AddStep(obs, step)

# split_dim
# $time
# sdate  time 
#    27    61 
# The operational dim must have sdate and time, or it crashes
res_fn <- Compute(wf, chunks = list(sdate = 1)) #, longitude = 2))



