
pkns <- c("ncdf4", "multiApply", "s2dv", "easyNCDF",
          "easyVerification", "zeallot", "startR", "CSTools")

lapply(pkns, library, character.only = TRUE)

# exp
pname_exp <- "/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc"

# Define a region
lons.w <- 342
lons.e <- 345
lats.s <- 32
lats.n <- 35

yearini <- 1993
yearend <- 2019

exp <- Start(dat = pname_exp, var = "tas",
             sdate = paste0(yearini:yearend, "0401"),
             time = indices(1:(30 + 31)),
             ensemble = "all",
             longitude_reorder = CircularSort(0, 360),
             longitude = values(list(lons.w, lons.e)),
             latitude_reorder = Sort(),
             latitude = values(list(lats.s, lats.n)),
             return_vars = list(latitude = 'dat', #NULL,
                                longitude = 'dat',
                                time = c("sdate")),
             retrieve = FALSE)

# Retrieve the attributes for obs
dates <- attr(exp, "Variables")$common$time
dates_file <- sort(unique(gsub('-', '', sapply(as.character(dates),
                                               substr, 1, 7))))


pname_obs <- "/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h/$var$_$file_date$.nc"
obs <- Start(dat = pname_obs, 
             var = "tas", 
             file_date = dates_file,
             time = values(dates),
               longitude_reorder = CircularSort(0, 360),
               longitude = values(list(lons.w, lons.e)),
               latitude_reorder = Sort(),
               latitude = values(list(lats.s, lats.n)),
             time_var = "time",
             time_tolerance = as.difftime(1, units = "hours"),
             time_across = "file_date",
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c("file_date")),
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
               transform = CDORemapper, 
               transform_extra_cells = 2,
               transform_params = list(grid = "r360x181", 
                                       method = "conservative",
                                       crop = c(lons.w, lons.e, lats.s, lats.n)),
               transform_vars = c("latitude", "longitude"),
               synonims = list(latitude = c('lat', 'latitude'),
                               longitude = c('lon', 'longitude')),
               retrieve = FALSE)

#====== Function to be applied (deprecated)==========
wrap_cal <- function(exp, obs) {
  # Bias correction
  source('/esarchive/scratch/cchou/MEDGOLD/others/functions/MG_BiasCorrectionNA.R')
  obs1 <- InsertDim(obs, 1, 1)
  exp1 <- t(exp)
  names(dim(obs1)) <- names(dim(exp1)) <- c('member', 'sdate')
  bc <- CST_BiasCorrectionNA(obs = obs1, exp = exp1)
  print('dim(bc)')
  print(dim(bc))
  return(bc)
}
step <- Step(wrap_cal,
             target_dims = list(c('sdate', 'time'), 
                                c('sdate', 'ensemble')
                           ),
             output_dims = c('ensemble', 'sdate')
        )
wf <- AddStep(list(obs, exp), step)
#===============================


#===== This works!!! ==========
wrap_cal <- function(obs, exp) {
# dim(exp) is (ensemble, sdate, time)
# dim(obs) is (sdate, time)

#print(dim(exp))
#print(dim(obs))
 bc <- multiApply::Apply(list(var_obs = obs, var_exp = exp),       
                         target_dims = list(c('sdate'), 
                                            c('ensemble', 'sdate')),
             fun = CSTools:::.sbc)$output1
}

step <- Step(wrap_cal,
             target_dims = list(c('sdate', 'time'),
                                c('ensemble', 'sdate', 'time')
                           ),
             output_dims = c('ensemble', 'sdate', 'time')
        )

wf <- AddStep(list(obs, exp), step)


#===== New fix works!!!==========
wrap_cal <- function(obs, exp) {
# dim(exp) is (ensemble, sdate)
# dim(obs) is (sdate)
 bc <- CSTools:::.sbc(var_obs = obs, var_exp = exp)
}

step <- Step(wrap_cal,
             target_dims = list(c('sdate'),
                                c('ensemble', 'sdate')
                           ),
             output_dims = c('ensemble', 'sdate')
        )

wf <- AddStep(list(obs, exp), step)
res_fn_new <- Compute(wf, chunks = list(time = 2))


#===== simple test ============

wrap_cal <- function(obs) {
#print(dim(obs))
  return(obs)
}

step <- Step(wrap_cal,
             target_dims = c('sdate'),
             output_dims = c('sdate')
        )
step2 <- Step(wrap_cal,
             target_dims = c('time', 'sdate'),
             output_dims = c('time', 'sdate')
        )

wf <- AddStep(obs, step)
wf_2 <- AddStep(obs, step2)
wf4_2 <- AddStep(obs4, step2)

 wf1 <- AddStep(obs1, step)
wf2 <- AddStep(data, step)

# split_dim
# $time
# sdate  time 
#    27    61 
# The operational dim must have sdate and time, or it crashes
res_fn <- Compute(wf, chunks = list(time = 1)) 

#===============================
res3 <- Compute(wf2, chunks = list(longitude = 2))
res2 <- Compute(wf1, chunks = list(time = 2))
res22 <- Compute(wf1, chunks = list(longitude = 2))
res_fn2 <- Compute(wf, chunks = list(longitude = 2))
res_fn3 <- Compute(wf, chunks = list(time = 2))
res_fn_2 <- Compute(wf_2, chunks = list(longitude = 2))
res4_fn_2 <- Compute(wf4_2, chunks = list(longitude = 2))

#Error in multiApply::Apply(data, target_dims = attr(step_fun, "TargetDims"),  : 
#  Found one or more margin dimensions with the same name and different length in some of the input objects in 'data'.





selectors <- array(1:12, dim = c(a=3, b=4))
selectors <- array(1:12, dim = c(a=3, b=4))
chunk <- dim(array(dim = c(a = 1, b = 1)))
n_chunks <- dim(array(dim = c(a = 1, b = 1)))

res <- Subset(selectors, 1:length(chunk), 
                        lapply(1:length(chunk), 
                          function(x) {
                            n_indices <- dim(selectors)[x]
                            chunk_sizes <- rep(floor(n_indices / n_chunks[x]), n_chunks[x])
                            chunks_to_extend <- n_indices - chunk_sizes[1] * n_chunks[x]
                            if (chunks_to_extend > 0) {
                              chunk_sizes[1:chunks_to_extend] <- chunk_sizes[1:chunks_to_extend] + 1
                            }
                            chunk_size <- chunk_sizes[chunk[x]]
                            offset <- 0
                            if (chunk[x] > 1) {
                              offset <- sum(chunk_sizes[1:(chunk[x] - 1)])
                            }
                            1:chunk_sizes[chunk[x]] + offset
                          }))

