
pkns <- c("ncdf4", "multiApply", "s2dv", "easyNCDF",
          "easyVerification", "zeallot", "startR", "CSTools")

lapply(pkns, library, character.only = TRUE)

# exp
pname_exp <- "/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc"

# Define a region
lons.w <- 340
lons.e <- 359
lats.s <- 30
lats.n <- 45

yearini <- 1993
yearend <- 2019

exp <- Start(dat = pname_exp, var = "tas",
             sdate = paste0(yearini:yearend, "0401"),
             time = indices(1:(30 + 31)),
             ensemble = "all",
             longitude_reorder = CircularSort(0, 360),
             longitude = values(list(lons.w, lons.e)),
             latitude_reorder = Sort(),
             latitude = values(list(lats.s, lats.n)),
             return_vars = list(latitude = 'dat', #NULL,
                                longitude = 'dat',
                                time = c("sdate")),
             retrieve = FALSE)

# Retrieve the attributes for obs
dates <- attr(exp, "Variables")$common$time
#dates_file <- sort(unique(gsub('-', '', sapply(as.character(dates),
#                                               substr, 1, 7))))

######## Andrea
file_date <- sapply(dates, format, '%Y%m')
dim(file_date) <- dim(dates)
########

pname_obs <- "/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h/$var$_$file_date$.nc"
obs <- Start(dat = pname_obs, 
             var = "tas", 
#             sdate =  dates_file,
             file_date = file_date,
##             time = values(dates),
             longitude_reorder = CircularSort(0, 360),
             longitude = values(list(lons.w, lons.e)),
             latitude_reorder = Sort(),
             latitude = values(list(lats.s, lats.n)),
#             time_var = "time",
#             time_tolerance = as.difftime(1, units = "hours"),
#             time_across = "sdate",
#             time_across = "file_date",
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c("file_date")),
##             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             transform = CDORemapper, 
             transform_extra_cells = 2,
             transform_params = list(grid = "r360x181", 
                                     method = "conservative",
                                     crop = c(lons.w, lons.e, lats.s, lats.n)),
             transform_vars = c("latitude", "longitude"),
             synonims = list(latitude = c('lat', 'latitude'),
                             longitude = c('lon', 'longitude')),
             retrieve = FALSE)

# Function to be applied
wrap_cal <- function(exp, obs) {
  # Bias correction
  source('/esarchive/scratch/cchou/MEDGOLD/others/functions/MG_BiasCorrectionNA.R')
  obs1 <- InsertDim(obs, 1, 1)
  exp1 <- t(exp)
  names(dim(obs1)) <- names(dim(exp1)) <- c('member', 'sdate')
  bc <- CST_BiasCorrectionNA(obs = obs1, exp = exp1)
  print('dim(bc)')
  print(dim(bc))
  return(bc)
}


step <- Step(wrap_cal,
             target_dims = list(c('sdate', 'time'), 
                                c('sdate', 'ensemble')
                           ),
             output_dims = c('ensemble', 'sdate')
        )


wf <- AddStep(list(obs, exp), step)

#===============
wrap_cal <- function(obs, exp) {
# dim(exp) is (ensemble, sdate, time)
# dim(obs) is (sdate, time)
print(dim(exp))
print(dim(obs))
 bc <- multiApply::Apply(list(var_obs = obs, var_exp = exp),                         target_dims = list(c('sdate'), 
                         target_dims = list(c('sdate'), 
                                            c('ensemble', 'sdate')),
             fun = CSTools:::.scb)$output1
}

step <- Step(wrap_cal,
             target_dims = list(c('sdate', 'time'),
                                #c('ensemble', 'sdate', 'time'),
                                c('ensemble', 'sdate', 'time')
                           ),
             output_dims = c('ensemble', 'sdate', 'time')
        )


wf <- AddStep(list(obs, exp), step)

#=================

wrap_cal <- function(obs) {
print(dim(obs))
  return(obs)
}

step <- Step(wrap_cal,
             target_dims = c('time'),
             output_dims = c('time')
        )

wf <- AddStep(obs, step)

# split_dim
# $time
# sdate  time 
#    27    61 
# The operational dim must have sdate and time, or it crashes
res_fn <- Compute(wf, chunks = list(latitude = 1)) 

######################


# split_dim
# $time
# sdate  time 
#    27    61 
# The operational dim must have sdate and time, or it crashes
res_fn <- Compute(wf, 
                  chunks = list(latitude = 1, longitude = 2))

                  threads_load = 2, threads_compute = 4)


