
pkns <- c("ncdf4", "multiApply", "s2dv", "easyNCDF",
          "easyVerification", "zeallot", "startR", "CSTools")

lapply(pkns, library, character.only = TRUE)

# exp
pname_exp <- "/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc"

# Define a region
lons.w <- 340
lons.e <- 359
lats.s <- 30
lats.n <- 45

yearini <- 1993
yearend <- 2019

exp <- Start(dat = pname_exp, var = "tas",
             sdate = paste0(yearini:yearend, "0401"),
             time = indices(1:(30 + 31)),
             ensemble = "all",
             longitude_reorder = CircularSort(0, 360),
             longitude = values(list(lons.w, lons.e)),
             latitude_reorder = Sort(),
             latitude = values(list(lats.s, lats.n)),
             return_vars = list(latitude = 'dat', #NULL,
                                longitude = 'dat',
                                time = c("sdate")),
             retrieve = FALSE)

# Retrieve the attributes for obs
dates <- attr(exp, "Variables")$common$time
dates_file <- sort(unique(gsub('-', '', sapply(as.character(dates),
                                               substr, 1, 7))))

pname_obs <- "/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h/$var$_$file_date$.nc"
obs <- Start(dat = pname_obs, 
             var = "tas", 
             file_date = dates_file,
             time = values(dates),
             longitude_reorder = CircularSort(0, 360),
             longitude = values(list(lons.w, lons.e)),
             latitude_reorder = Sort(),
             latitude = values(list(lats.s, lats.n)),
             time_var = "time",
             time_tolerance = as.difftime(1, units = "hours"),
             time_across = "file_date",
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c("file_date")),
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             transform = CDORemapper, 
             transform_extra_cells = 2,
             transform_params = list(grid = "r360x181", 
                                     method = "conservative",
                                     crop = c(lons.w, lons.e, lats.s, lats.n)),
             transform_vars = c("latitude", "longitude"),
             synonims = list(latitude = c('lat', 'latitude'),
                             longitude = c('lon', 'longitude')),
             retrieve = FALSE)

# Function to be applied
wrap_cal <- function(exp) {
#  obs1 <- InsertDim(obs, 1, 1)
  exp1 <- t(exp)
#  names(dim(obs)) <- 'asd'
names(dim(exp1)) <- c('member', 'sdate')
  #member  sdate 
  #    3      4 
  return(exp1)
}

wrap_cal <- function(obs, exp) {
#  obs1 <- InsertDim(obs, 1, 1)
  exp1 <- t(exp)
#names(dim(obs1)) <- c('member', 'sdate')
names(dim(exp1)) <- c('member', 'sdate')
  #member  sdate 
  #    3      4 
  return(exp1)
}

wrap_cal <- function(obs) {
print(dim(obs))
  return(obs)
}

step <- Step(wrap_cal,
             target_dims = list(c('sdate'), 
                                c('sdate', 'ensemble')
                           ),
             output_dims = c('ensemble', 'sdate')
        )

step <- Step(wrap_cal,
             target_dims = c('time','sdate') 
                               # c('sdate', 'ensemble')
                           ,
             output_dims = c('time','sdate')
        )

wf <- AddStep(list(obs, exp), step)
wf <- AddStep(obs, step)

# split_dim
# $time
# sdate  time 
#    27    61 
# The operational dim must have sdate and time, or it crashes
res_fn <- Compute(wf, 
                  chunks = list(latitude = 1, longitude = 2))

                  threads_load = 2, threads_compute = 4)


