library(startR)
library(multiApply)
library(CSTools)

load_multi_DWD_calibration <- function(variable, sdates, fyears, grid, n_cores = 8){
  
  ## Already anomalies
  ## Calibrated with the DWD method (Pasternack et al., 2018)
  ## 10 members for bsc, dwd, ukmo and 6 members for cmcc
  ## 5o grid
  ## Annual means (1 file per fyear with 1 value)
  ## Only for tas, pr, psl
  
  data <- startR::Start(dat = '/esarchive/scratch/cdelgado/data/DWD_recalibration/$auxcenter$_dcpp_$var$/id_number/crossval/decadalfc/output/$auxcenter$/$auxmodel$/dcppa$sdate$/yr/atmos/$var$/$member$/$var$_yr_$auxmodel$_dcppa$sdate$_$member$_$fyear$.nc',
                        var = 'tas', #variable,
                        sdate = '1999', #paste0(sdates),
                        fyear = indices(1:2), #indices(fyears),
                        time = 'all',
                        lat = 'all',
                        lon = 'all',
                        member = 'all',
                        auxcenter = c('bsc','cmcc','dwd','ukmo'),
                        auxmodel = list(bsc='ec-earth3',cmcc='dps-cmcc-cm2',dwd='mpi-esm-hr',ukmo='hadgem3'),
                        auxmodel_depends = 'auxcenter',
                        fyear_depends = 'sdate',
                        return_vars = list(lat='dat',lon='dat'),
                        lat_reorder = Sort(decreasing = F),
                        lon_reorder = CircularSort(0,360),
                        num_procs = 1, retrieve = TRUE)
  
  lon = as.vector(attr(data,'Variables')$dat1[['lon']])
  lat = as.vector(attr(data,'Variables')$dat1[['lat']])
  
  data <- multiApply::Apply(data = data, target_dims = 'fyear', fun = mean, na.rm = FALSE, n_cores = n_cores)$output1
  data <- drop(data)
  data <- CSTools::MergeDims(data = data, merge_dims = c('member','auxcenter'), rename_dim = 'member', na.rm = TRUE)
  
  return(list(data = data, lat = lat, lon = lon))
}

