## Example on how to load monthly data from decadal prediction systems that are initialised in different dates
## HadGEM3 (initialised in November)
## IPSL-CM6A-LR (initialised in January)
## NorCPM1 (initialised in October)

library(startR)

variable <- 'tas'
sdate <- 2000

## Paths
path_hadgem3 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-dcppA-hindcast_i1p1/DCPP/',
                       'MOHC/HadGEM3-GC31-MM/dcppA-hindcast/r1i1p1f2/Amon/$var$/gr/v20200316/',
                       '$var$_*_s$sdate$_r1i1p1f2_gr_$fyear$.nc')
path_ipsl <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/ipsl-cm6a-lr/cmip6-dcppA-hindcast_i1p1/DCPP/', 
                    'IPSL/IPSL-CM6A-LR/dcppA-hindcast/r1i1p1f1/Amon/$var$/gr/v20200504/',
                    '$var$_Amon_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-r1i1p1f1_gr_$fyear$.nc')
path_norcpm1 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/norcpm1/cmip6-dcppA-hindcast_i1p1/DCPP/',
                       'NCC/NorCPM1/dcppA-hindcast/r1i1p1f1/Amon/$var$/gn/v20200320/',
                       '$var$_Amon_NorCPM1_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')

## Forecast year (to complete the path)
fyear_hadgem3 <- paste0(sdate,'11-',sdate+11,'03')
fyear_ipsl <- paste0(sdate+1,'01-',sdate+10,'12')
fyear_norcpm1 <- paste0(sdate,'10-',sdate+10,'12')

## Forecast months: six indices (from October to March)
time_hadgem3 <- indices(c(NA, 1, 2, 3, 4, 5))
time_ipsl <- indices(c(NA, NA, NA, 1, 2, 3))
time_norcpm1 <- indices(c(1, 2, 3, 4, 5, 6))



res <- multiStart(
  dat = list(list(name = 'hadgem3', path = path_hadgem3),
             list(name = 'ipsl', path = path_ipsl),
             list(name = 'norcpm1', path = path_norcpm1)),
  var = variable, 
  sdate = paste0(sdate),
  fyear = list(list(name = 'hadgem3', fyear = fyear_hadgem3),
               list(name = 'ipsl', fyear = fyear_ipsl),
               list(name = 'norcpm1', fyear = fyear_norcpm1)),
  time = list(list(name = 'hadgem3', time = time_hadgem3),
              list(name = 'ipsl', time = time_ipsl),
              list(name = 'norcpm1', time = time_norcpm1)),
  lat = values(list(-30, 30)), 
  lon = values(list(0, 50)),
  synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
  transform = CDORemapper,
  transform_params = list(grid = 'r180x90',
                          method = 'conservative',
                          crop = c(0, 50, -30, 30)),
  transform_vars = c('lat', 'lon'),
  return_vars = list(lat = NULL, lon = NULL, time = 'sdate'),
  retrieve = TRUE)

