library(startR)

#-------------1. one data in one call-------------------
# Define selectors
  # HadGEM3 daily data (360_day)
  path_hadgem3 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/',
                         'dcppA-hindcast/r1i1p1f2/day/$var$/gn/v20200101/',
                         '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-r1i1p1f2_gn_$fyear$.nc')
  ## tasmax_day_HadGEM3-GC31-MM_dcppA-hindcast_s2000-r1i1p1f2_gn_20001101-20101230.nc

  # MPI-ESM1.2-HR (standard)
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/',
                         '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')
  ## tasmax_day_MPI-ESM1-2-HR_dcppA-hindcast_s2000-r1i1p1f1_gn_20001101-20101231.nc

  # Selectors
  var <- 'tasmax'
  sdate <- '2000'
  fyear_hadgem3 <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1230')
  fyear_mpi_esm <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1231')
  time_hadgem3 <- indices(59:62) #'all'
  time_mpi_esm <- indices(59:62)

test_2 <- list(test_hadgem3 = list(name = 'hadgem3', path = path_hadgem3, 
                                   fyear = fyear_hadgem3, time = time_hadgem3),
               test_mpi_esm = list(name = 'mpi_esm', path = path_mpi_esm,
                                   fyear = fyear_mpi_esm, time = time_mpi_esm))

res_list <- lapply(test_2, function(x) {
  list2env(x, env = .GlobalEnv)
  suppressWarnings(
    data <- Start(dat = path,
                  var = 'tasmax', sdate = '2000',
                  fyear = fyear,
                  time = time,
                  lat = indices(1), lon = indices(1),
                  #fyear_depends = 'sdate',
                  return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'), #'dat'),
                  num_procs = 1,
#                  debug = TRUE,
                  retrieve = TRUE)
  )
#  dates <- attr(data, 'Variables')$dat$time
  dates <- attr(data, 'Variables')$common$time
print(paste0('The first two dates of ', name))
print(dates[1:2])
print('done!!')
  return(dates)
})

str(res_list)
res_list$test_hadgem3
[1] "2000-12-29 12:00:00 UTC" "2000-12-30 12:00:00 UTC"
[3] "2001-01-01 12:00:00 UTC" "2001-01-02 12:00:00 UTC"
res_list$test_mpi_esm
[1] "2000-12-29 12:00:00 UTC" "2000-12-30 12:00:00 UTC"
[3] "2000-12-31 12:00:00 UTC" "2001-01-01 12:00:00 UTC"



#-------------2. two data in one call-------------------
# Read two datasets together (one sdate)

  # HadGEM3 daily data
  path_hadgem3 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/',
                         'dcppA-hindcast/r1i1p1f2/day/$var$/gn/v20200101/')

  file_hadgem3 <- '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s2000-r1i1p1f2_gn_20001101-20101230.nc'
  path_file_hadgem3 <- paste0(path_hadgem3, file_hadgem3)
  ## tasmax_day_HadGEM3-GC31-MM_dcppA-hindcast_s2000-r1i1p1f2_gn_20001101-20101230.nc

#  #=== BCC-CSM2-MR daily data ===
#  path_bcc_csm2 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/bcc-csm2-mr/',
#                          'cmip6-dcppA-hindcast_i1p1/DCPP/BCC/BCC-CSM2-MR/',
#                          'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200408/')
#  file_bcc_csm2 <- '$var$_day_BCC-CSM2-MR_dcppA-hindcast_s1980-r1i1p1f1_gn_19800101-19891231.nc'
#  ## tasmax_day_BCC-CSM2-MR_dcppA-hindcast_s1980-r1i1p1f1_gn_19800101-19891231.nc
#  path_file_bcc_csm2 <- paste0(path_bcc_csm2, file_bcc_csm2)


  #=== MPI-ESM1.2-HR ===
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/')
  file_mpi_esm <- '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s2000-r1i1p1f1_gn_20001101-20101231.nc'
  ## tasmax_day_MPI-ESM1-2-HR_dcppA-hindcast_s2000-r1i1p1f1_gn_20001101-20101231.nc
  path_file_mpi_esm <- paste0(path_mpi_esm, file_mpi_esm)


      data <- Start(dat = list(list(name = 'Hadgem3', path = path_file_hadgem3),
#                               list(name = 'BCC_CSM2', path = path_file_bcc_csm2),
                               list(name = 'MPI_ESM', path = path_file_mpi_esm)),
                    var = 'tasmax',
#                    sdate = sdate,
#                    fyear = c(fyear_bcc_csm2, fyear_mpi_esm),
                    time = indices(1:100),
                    lat = indices(1), lon = indices(1),
                    return_vars = list(lat = 'dat', lon = 'dat', time = 'dat'),
                    num_procs = 1,
                    retrieve = TRUE)
attr(data, 'Variables')$Hadgem3$time
attr(data, 'Variables')$MPI_ESM$time


#-------------3. one data one call; separate day and month-------------------
  # HadGEM3 daily data (360_day)
  path_hadgem3 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/',
                         'dcppA-hindcast/r1i1p1f2/day/$var$/gn/v20200101/',
                         '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-r1i1p1f2_gn_$fyear$.nc')
  # Selectors
  fyear_hadgem3 <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1230')
  time_hadgem3 <- indices(59:62) #'all'
  # Create an array for time then split them
  dates1 <- paste0('2000-11-', sprintf('%02d', 1:30), ' 12:00:00')
  dates2 <- paste0('2000-12-', sprintf('%02d', 1:30), ' 12:00:00')
  dates <- c(as.POSIXct(dates1, tz = 'UTC'), as.POSIXct(dates2, tz = 'UTC'))
  attr(dates, 'tzone') <- 'UTC'
  dim(dates) <- c(time = 30, month = 2)

  # split
  data <- Start(dat = path_hadgem3,
                var = 'tasmax', 
                sdate = '2000',
                fyear = fyear_hadgem3,
                time = values(dates),
                time_across = 'sdate',  #without this line, an error shows which is out-of-date
                split_multiselected_dims = TRUE,
                lat = indices(1), lon = indices(1),
                #fyear_depends = 'sdate',
                return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'),
#                debug = TRUE,
                retrieve = TRUE)

dim(attr(data, 'Variables')$common$time)
#sdate  time 
#    1    60 
dim(data)
#  dat   var sdate fyear  time month   lat   lon 
#    1     1     1     1    30     2     1     1 


  # non-split
  dates1 <- paste0('2000-11-', sprintf('%02d', 1:30), ' 12:00:00')
  dates2 <- paste0('2000-12-', sprintf('%02d', 1:30), ' 12:00:00')
  dates <- c(as.POSIXct(dates1, tz = 'UTC'), as.POSIXct(dates2, tz = 'UTC'))

  data_nonsplit <- Start(dat = path_hadgem3,
                var = 'tasmax',
                sdate = '2000',
                fyear = fyear_hadgem3,
                time = values(dates),
                time_across = 'sdate',  #without this line, an error shows which is out-of-date
                lat = indices(1), lon = indices(1),
#                fyear_depends = 'sdate',
                return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'),
#                  debug = TRUE,
                  retrieve = TRUE)

dim(data_nonsplit)
#  dat   var sdate fyear  time   lat   lon 
#    1     1     1     1    60     1     1 

all.equal(as.vector(data[1,1,1,1,,,1,1]), data_nonsplit[1,1,1,1,,1,1])
#[1] TRUE

#-----------------------------------
  # MPI-ESM1.2-HR (standard)
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/', 
                         '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')
  # Selectors
  fyear_mpi_esm <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1231')
  # Create an array for time then split them
  dates1 <- paste0('2000-11-', sprintf('%02d', 1:30), ' 12:00:00')
  dates2 <- paste0('2000-12-', sprintf('%02d', 1:31), ' 12:00:00')
  dates_nonsplit <- c(as.POSIXct(dates1, tz = 'UTC'), NA, as.POSIXct(dates2, tz = 'UTC'))
  attr(dates_nonsplit, 'tzone') <- 'UTC'
  dates <- dates_nonsplit
  dim(dates) <- c(time = 31, month = 2)

  # split
  data <- Start(dat = path_mpi_esm,
                var = 'tasmax',
                sdate = '2000',
                fyear = '20001101-20101231',
                time = values(dates),
                time_across = 'sdate',  #without this line, an error which I don't understand shows
                merge_across_dims = TRUE,
                merge_across_dims_narm = FALSE,
                split_multiselected_dims = TRUE,
                lat = indices(1), lon = indices(1),
                #fyear_depends = 'sdate',
                return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'),
                retrieve = TRUE)

dim(attr(data, 'Variables')$common$time)
#sdate  time 
#    1    61 
dim(data)
#  dat   var  fyear  time month   lat   lon 
#    1     1      1    31     2     1     1 
data[1, 1, 1, , , 1, 1]


  # non-split
  data_nonsplit <- Start(dat = path_mpi_esm,
                var = 'tasmax',
                sdate = '2000',
                fyear = '20001101-20101231',
                time = values(dates_nonsplit),
                time_across = 'sdate',  #without this line, an error shows which is out-of-date
                split_multiselected_dims = TRUE,
                lat = indices(1), lon = indices(1),
                #fyear_depends = 'sdate',
                return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'),
                retrieve = TRUE)

dim(data_nonsplit)
#  dat   var sdate fyear  time   lat   lon 
#    1     1     1     1    61     1     1 

all.equal(as.vector(data[1,1,1,1,,,1,1]), data_nonsplit[1,1,1,1,,1,1])
# data is incorret. The last one repeats the first one, and 31st is not NA.









#-------------4. two data one call; separate day and month-------------------
  # HadGEM3 daily data (360_day)
  path_hadgem3 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/',
                         'dcppA-hindcast/r1i1p1f2/day/$var$/gn/v20200101/',
                         '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-r1i1p1f2_gn_$fyear$.nc')
  # MPI-ESM1.2-HR (standard)
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/',
                         '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')
  # Selectors
  var <- 'tasmax'
  sdate <- '2000'
  fyear_hadgem3 <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1230')
  fyear_mpi_esm <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1231')
  # Create an array for time then split them
  dates1 <- paste0('2000-11-', sprintf('%02d', 1:31), ' 12:00:00')
  dates2 <- paste0('2000-12-', sprintf('%02d', 1:31), ' 12:00:00')
  dates <- c(as.POSIXct(dates1, tz = 'UTC'), as.POSIXct(dates2, tz = 'UTC'))
  attr(dates, 'tzone') <- 'UTC'
  dim(dates) <- c(time = 31, month = 2)

  # split
  data <- Start(dat = path_hadgem3,
                var = 'tasmax',
                sdate = '2000',
                fyear = fyear_hadgem3,
                time = values(dates),
                time_across = 'sdate',  #without this line, an error shows which is out-of-date
                split_multiselected_dims = TRUE,
                lat = indices(1), lon = indices(1),
                #fyear_depends = 'sdate',
                return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'),
#                debug = TRUE,
                retrieve = TRUE)

dim(attr(data, 'Variables')$common$time)
