library(startR)

#--------------- Test 2: Use values() to define time selector, one dataset ----------------
# Same idea as using exp time attributes to retrieve corresponding obs data.
#------------------------------------------------------------------------------------------


#/////////////// Define selectors //////////////////
#  #=== HadGEM3 daily data ===
#  path_hadgem3 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
#                         'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/',
#                         'dcppA-hindcast/r1i1p1f2/day/$var$/gn/v20200101/',
#                         '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-r1i1p1f2_gn_$fyear$.nc')
#  ## tasmax_day_HadGEM3-GC31-MM_dcppA-hindcast_s2000-r1i1p1f2_gn_20001101-20101230.nc

  #=== MPI-ESM1.2-HR ===
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/',
                         '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')
  ## tasmax_day_MPI-ESM1-2-HR_dcppA-hindcast_s2000-r1i1p1f1_gn_20001101-20101231.nc

  #=== Selectors ===
  var <- 'tasmax'
  sdate <- '2000' #c('2000', '2001')
#  fyear_hadgem3 <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1230')
  fyear_mpi_esm <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1231')
  # This is necessary if 'sdate' is more than 1. Make fyear as a list and put sdate as name
  fyear_mpi_esm <- setNames(as.list(fyear_mpi_esm), sdate)


#//////////////////// (1) Usual call. time = 'all'//////////////////////
  #=== MPI-ESM1.2-HR ===
  data <- Start(dat = path_mpi_esm,
                var = var, 
                sdate = sdate,
                fyear = fyear_mpi_esm,
                fyear_depends = 'sdate',
                time = 'all',
                lat = indices(1),
                lon = indices(1),
                return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'),
                retrieve = TRUE)

  dates <- attr(data, 'Variables')$common$time
  dates[1, 1:5]
#[1] "2000-11-01 12:00:00 UTC" "2000-11-02 12:00:00 UTC"
#[3] "2000-11-03 12:00:00 UTC" "2000-11-04 12:00:00 UTC"
#[5] "2000-11-05 12:00:00 UTC"

  dates[2, 1:5]
#[1] "2001-11-01 12:00:00 UTC" "2001-11-02 12:00:00 UTC"
#[3] "2001-11-03 12:00:00 UTC" "2001-11-04 12:00:00 UTC"
#[5] "2001-11-05 12:00:00 UTC"


#////////////// (2) Use the time values we get above to retrieve data ////////////////// 

  #NOTE: time2 and time3 are incorrect!!
  time1 <- dates[c(1, 2, 31)]
  time2 <- c(as.POSIXct('2000-11-01', tz = 'UTC'), 
             as.POSIXct('2000-11-02', tz = 'UTC'),
             as.POSIXct('2000-12-01', tz = 'UTC'))
  time3 <- time2
  attr(time3, "tzone") <- "UTC"
  time4 <- c(as.POSIXct('2000-11-01 12:00:00', tz = 'UTC'), 
             as.POSIXct('2000-11-02 12:00:00', tz = 'UTC'),
             as.POSIXct('2000-12-01 12:00:00', tz = 'UTC'))
  time5 <- time4
  attr(time5, "tzone") <- "UTC"
  # before 1st time step
  time6 <- as.POSIXct('2000-10-01 12:00:00', tz = 'UTC')
  # exceed the last time step
  time7 <- as.POSIXct('2020-11-01 12:00:00', tz = 'UTC')
  # Error!! Different time class
  time8 <- as.PCICt(result*3600*24, cal = '360_day', origin = '1850-01-01', tz = 'UTC')

#  time_list <- list(time1, time2, time3, time4, time5, time6, time7, time8)
  time_list <- list(time1, time4, time5, time6, time7)

  func_test_list <- function (time_list) {
    len <- length(time_list)
    a <- vector('list', length = len)
    names(a) <- paste0('test', 1:len)
    for (i in 1:len) {
      a[[i]] <- list(name = paste0('test', i), time = time_list[[i]])
    }
    return(a)
  }
  
  test_time_value <- func_test_list(time_list)
  
  res_time_value <- lapply(test_time_value, function (x) {
    list2env(x, env = .GlobalEnv)
    suppressWarnings(
      data <- Start(dat = path_mpi_esm,
                    var = var, sdate = sdate,
                    fyear = fyear_mpi_esm,
                    time = time,
                    lat = indices(1), lon = indices(1),
                    fyear_depends = 'sdate',
                    return_vars = list(lat = 'dat', lon = 'dat'),
                    num_procs = 1,
                    retrieve = TRUE)
    )
    dates <- attr(data, 'Variables')$common$time
    print(paste0('The input time of ', name))
    print(time)
    print(paste0('The output time of ', name))
    print(dates)
    print('done!!')
  
    return(list(input = time, output = dates, data = as.vector(data)))
  
  })
  
  str(res_time_value)


# Note that when time is assigned with values(), time = 'sdate' should be removed from ‘return_vars’ or an error is returned (why?).
# Error: Provided selectors for the dimension 'time' must have as many file dimensions as the variable the dimension is defined along, 'time', with the exceptions of the file pattern dimension ('dat') and any depended file dimension (if specified as depended dimension in parameter 'inner_dims_across_files' and the depending file dimension is present in the provided selector array).

