library(startR)

#--------- Test 3: Use values() to define time selector, TWO datasets, ONE sdate--------------
# Same idea as using exp time attributes to retrieve corresponding obs data.
#------------------------------------------------------------------------------------------


#//////////////////// (1) Usual call. time = 'all'//////////////////////
#NOTE: The part is not important. It is just for checking the correctness of data.

#/////////////// Define selectors //////////////////
  #=== BCC-CSM2-MR daily data (365_day) ===
  path_bcc_csm2 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/bcc-csm2-mr/',
                          'cmip6-dcppA-hindcast_i1p1/DCPP/BCC/BCC-CSM2-MR/',
                          'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200408/',
                          '$var$_day_BCC-CSM2-MR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')
  ## tasmax_day_BCC-CSM2-MR_dcppA-hindcast_s1980-r1i1p1f1_gn_19800101-19891231.nc

  #=== MPI-ESM1.2-HR ===
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/',
                         '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')
  ## tasmax_day_MPI-ESM1-2-HR_dcppA-hindcast_s2000-r1i1p1f1_gn_20001101-20101231.nc

  #=== Selectors ===
  var <- 'tasmax'
  sdate <- '1980' #c('1980', '1981')
  fyear_bcc_csm2 <- paste0(sdate, '0101-', as.numeric(sdate) + 9, '1231')
  fyear_mpi_esm <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1231')
  # This is necessary if 'sdate' is more than 1. Make fyear as a list and put sdate as name
  fyear_bcc_csm2 <- setNames(as.list(fyear_bcc_csm2), 'dat1') #sdate)
  fyear_mpi_esm <- setNames(as.list(fyear_mpi_esm), 'dat2') #sdate)



  #=== BCC-CSM2-MR ===
  data_bcc_csm2 <- Start(dat = path_bcc_csm2,
                var = var, 
                sdate = sdate,
                fyear = fyear_bcc_csm2,
                time = 'all',
                lat = indices(1),
                lon = indices(1),
                return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'),
                retrieve = TRUE)

  dates_bcc_csm2 <- attr(data_bcc_csm2, 'Variables')$common$time
  dates_bcc_csm2[1, 1:5]
#[1] "1980-01-01 12:00:00 UTC" "1980-01-02 12:00:00 UTC"
#[3] "1980-01-03 12:00:00 UTC" "1980-01-04 12:00:00 UTC"
#[5] "1980-01-05 12:00:00 UTC"


  #=== MPI-ESM1.2-HR ===
  data_mpi_esm <- Start(dat = path_mpi_esm,
                var = var, 
                sdate = sdate,
                fyear = fyear_mpi_esm,
                time = 'all',
                lat = indices(1),
                lon = indices(1),
                return_vars = list(lat = 'dat', lon = 'dat', time = 'sdate'),
                retrieve = TRUE)

  dates_mpi_esm <- attr(data_mpi_esm, 'Variables')$common$time
  dates_mpi_esm[1, 1:5]
#[1] "1980-11-01 12:00:00 UTC" "1980-11-02 12:00:00 UTC"
#[3] "1980-11-03 12:00:00 UTC" "1980-11-04 12:00:00 UTC"
#[5] "1980-11-05 12:00:00 UTC"






#////////////// (2) Read two datasets together (one sdate) /////////////////////
# (2-1) Directly define sdate and fyear in the path
  #=== BCC-CSM2-MR daily data ===
  path_bcc_csm2 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/bcc-csm2-mr/',
                          'cmip6-dcppA-hindcast_i1p1/DCPP/BCC/BCC-CSM2-MR/',
                          'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200408/')
  file_bcc_csm2 <- '$var$_day_BCC-CSM2-MR_dcppA-hindcast_s1980-r1i1p1f1_gn_19800101-19891231.nc'
  ## tasmax_day_BCC-CSM2-MR_dcppA-hindcast_s1980-r1i1p1f1_gn_19800101-19891231.nc
  path_file_bcc_csm2 <- paste0(path_bcc_csm2, file_bcc_csm2)


  #=== MPI-ESM1.2-HR ===
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/')
  file_mpi_esm <- '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s1980-r1i1p1f1_gn_19801101-19901231.nc'
  ## tasmax_day_MPI-ESM1-2-HR_dcppA-hindcast_s2000-r1i1p1f1_gn_20001101-20101231.nc
  path_file_mpi_esm <- paste0(path_mpi_esm, file_mpi_esm)


#--------------------------------------------------------------------------------- 
  # Read two dat together. Request time is 1980-11-01, 1980-11-02, and 1980-12-01.
  # (2-1-1) use values()
  data <- Start(dat = list(list(name = 'BCC_CSM2', path = path_file_bcc_csm2),
                           list(name = 'MPI_ESM', path = path_file_mpi_esm)),
                var = var,
#                sdate = sdate,
#                fyear = c(fyear_bcc_csm2, fyear_mpi_esm),
                time = dates_mpi_esm[c(1, 2, 31)], # equal to dates_bcc_csm2[305, 306, 335]
                lat = indices(1), lon = indices(1),
                return_vars = list(lat = 'dat', lon = 'dat'), #time = 'dat'),
                num_procs = 1,
                retrieve = TRUE)

  # Comparison
  dates	 <- attr(data, 'Variables')$common$time
  identical(data_bcc_csm2[c(305, 306, 335)], data[1, 1, , 1, 1])
  identical(data_mpi_esm[c(1, 2, 31)], data[2, 1, , 1, 1]) #WRONG!!!

  data[1:6]
#[1] 232.6879 213.6568 236.5945 218.4152 241.8504 222.5171
  data_bcc_csm2[c(305, 306, 335)]
#[1] 233.1345 232.6879 243.8640
  which(data_mpi_esm == data[2])
#[1] 305
  which(data_mpi_esm == data[4])
#[1] 306
  which(data_mpi_esm == data[6])
#[1] 335

#NOTE: The first dataset is correct; the second is wrong because Start uses the time "indices" of 
# the first dataset to find the 2nd dataset even if we provide "values".


#--------------------------------------------------------------------------
  # Add time = 'dat' into return_vars
  # (2-1-1) use values()
  data2 <- Start(dat = list(list(name = 'BCC_CSM2', path = path_file_bcc_csm2),
                            list(name = 'MPI_ESM', path = path_file_mpi_esm)),
                 var = var,
#                 sdate = sdate,
#                 fyear = c(fyear_bcc_csm2, fyear_mpi_esm),
                 time = dates_mpi_esm[c(1, 2, 31)], # equal to dates_bcc_csm2[305, 306, 335]
                 lat = indices(1), lon = indices(1),
                 return_vars = list(lat = 'dat', lon = 'dat', time = 'dat'),
                 num_procs = 1,
                 retrieve = TRUE)

  # Comparison
  dates_BCC_CSM2  <- attr(data2, 'Variables')$BCC_CSM2$time
  dates_MPI_ESM  <- attr(data2, 'Variables')$MPI_ESM$time
  identical(data_bcc_csm2[c(305, 306, 335)], data2[1, 1, , 1, 1])
  identical(data_mpi_esm[c(1, 2, 31)], data2[2, 1, , 1, 1]) #CORRECT


#---------------------------------------------
  # (2-1-2) Use list of indices to define time selector. FAIL!!
  names <- c('BCC_CSM2', 'MPI_ESM')
  times <- list(c(305, 306, 335), c(1, 2, 31))
  times_list <- setNames(times, names)

  data3 <- Start(dat = list(list(name = 'BCC_CSM2', path = path_file_bcc_csm2),
                            list(name = 'MPI_ESM', path = path_file_mpi_esm)),
                 var = var,
##                 model = names,
#                 sdate = sdate,
#                 fyear = c(fyear_bcc_csm2, fyear_mpi_esm),
#                 time = list(BCC_CSM2 = indices(c(305, 306, 335)), 
#                             MPI_ESM = indices(c(1, 2, 31))),
                 time = times_list,
                 time_var = 'time',
                 lat = indices(1), lon = indices(1),
                 return_vars = list(lat = 'dat', lon = 'dat', time = 'dat'),
                 num_procs = 1,
                 retrieve = TRUE)


#----------------------------------------------------------
  # (2-1-3) Use list of values

  data4 <- Start(dat = list(list(name = 'BCC_CSM2', path = path_file_bcc_csm2),
                            list(name = 'MPI_ESM', path = path_file_mpi_esm)),
                 var = var,
#                 sdate = sdate,
#                 fyear = c(fyear_bcc_csm2, fyear_mpi_esm),
                 time = list(path_file_bcc_csm2 = dates_bcc_csm2[c(305, 306, 335)],
                             path_file_mpi_esm = dates_mpi_esm[c(1, 2, 31)]),
#                 time_var = 'time',
                 lat = indices(1), lon = indices(1),
                 return_vars = list(lat = 'dat', lon = 'dat', time = 'dat'),
                 num_procs = 1,
                 retrieve = TRUE)

#----------------------------------------------------------
# (2-2) Use sdate and fyear tags in the path (undone)
  #=== BCC-CSM2-MR daily data ===
  path_bcc_csm2 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/bcc-csm2-mr/',
                          'cmip6-dcppA-hindcast_i1p1/DCPP/BCC/BCC-CSM2-MR/',
                          'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200408/')
  file_bcc_csm2 <- '$var$_day_BCC-CSM2-MR_dcppA-hindcast_s1980-r1i1p1f1_gn_19800101-19891231.nc'
#  file_bcc_csm2 <- '$var$_day_BCC-CSM2-MR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc'
  ## tasmax_day_BCC-CSM2-MR_dcppA-hindcast_s1980-r1i1p1f1_gn_19800101-19891231.nc
  path_file_bcc_csm2 <- paste0(path_bcc_csm2, file_bcc_csm2)


  #=== MPI-ESM1.2-HR ===
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/')
  file_mpi_esm <- '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s1980-r1i1p1f1_gn_19801101-19901231.nc'
#  file_mpi_esm <- '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc'
  ## tasmax_day_MPI-ESM1-2-HR_dcppA-hindcast_s2000-r1i1p1f1_gn_20001101-20101231.nc
  path_file_mpi_esm <- paste0(path_mpi_esm, file_mpi_esm)





#////////////// (3) Request time steps are not common ////////////////// 
# Read two datasets in one single Start() call, BUT the request time steps are not common ones.
# MPI_ESM is standard calendar, and BCC_CSM2 is 365_day calendar.
dates_mpi_esm[1216]
#[1] "1984-02-29 12:00:00 UTC"
dates_bcc_csm2[1519]
#[1] "1984-02-28 12:00:00 UTC"
dates_bcc_csm2[1520]
#[1] "1984-03-01 12:00:00 UTC"

  # time = "1984-02-28 12:00:00 UTC", "1984-02-29 12:00:00 UTC"
  data4 <- Start(dat = list(list(name = 'BCC_CSM2', path = path_file_bcc_csm2),
                            list(name = 'MPI_ESM', path = path_file_mpi_esm)),
                 var = var,
#                 sdate = sdate,
#                 fyear = c(fyear_bcc_csm2, fyear_mpi_esm),
                 time = dates_mpi_esm[c(1215, 1216)], # bcc doesn't have the 2nd one
                 lat = indices(1), lon = indices(1),
                 return_vars = list(lat = 'dat', lon = 'dat', time = 'dat'),
                 num_procs = 1,
                 retrieve = TRUE)

data4[1:4]
[1] 236.9087 232.8797 236.9087 231.4135
# NOTE: the 2nd time step of bcc is still 02-28.

  # Comparison
  attr(data4, 'Variables')$BCC_CSM2$time
[1] "1984-02-28 12:00:00 UTC"
  attr(data4, 'Variables')$MPI_ESM$time
[1] "1984-02-28 12:00:00 UTC" "1984-02-29 12:00:00 UTC"

  identical(data_bcc_csm2[c(1519)], data4[1, 1, 1, 1, 1])
  identical(data_mpi_esm[c(1215, 1216)], data4[2, 1, , 1, 1]) #CORRECT









# DEPRECATED
#////////////// (3) Use the time values we get above to retrieve data ////////////////// 

  time1 <- dates_mpi_esm[c(1, 2, 31)] # equal to dates_bcc_csm2[306, 307, 336]
  time4 <- c(as.POSIXct('1980-11-01 12:00:00', tz = 'UTC'), 
             as.POSIXct('1980-11-02 12:00:00', tz = 'UTC'),
             as.POSIXct('1980-12-01 12:00:00', tz = 'UTC'))
  time5 <- time4
  attr(time5, "tzone") <- "UTC"
  # bcc_csm2 1st time step (not exist in mpi_esm)
  time6 <- as.POSIXct('1980-01-01 12:00:00', tz = 'UTC')
  # mpi_esm last time step (not exist in bcc_csm2)
  time7 <- dates_mpi_esm[3713]  #  "1990-12-31 12:00:00 UTC"

  time_list <- list(time1, time4, time5, time6, time7)

  func_test_list <- function (time_list) {
    len <- length(time_list)
    a <- vector('list', length = len)
    names(a) <- paste0('test', 1:len)
    for (i in 1:len) {
      a[[i]] <- list(name = paste0('test', i), time = time_list[[i]])
    }
    return(a)
  }
  
  test_time_value <- func_test_list(time_list)
  
  res_time_value <- lapply(test_time_value, function (x) {
    list2env(x, env = .GlobalEnv)
    suppressWarnings(
      data <- Start(dat = list(list(name = 'BCC_CSM2', path = path_bcc_csm2),
                               list(name = 'MPI_ESM', path = path_mpi_esm)),
#dat = c(path_bcc_csm2, path_mpi_esm),
                    var = var,
                    sdate = sdate,
                    fyear = c(fyear_bcc_csm2, fyear_mpi_esm),

                    fyear_depends = 'dat',
                    time = time,
                    lat = indices(1), lon = indices(1),
                    return_vars = list(lat = 'dat', lon = 'dat'),
                    num_procs = 1,
                    retrieve = TRUE)
    )
    dates <- attr(data, 'Variables')$common$time
    print(paste0('The input time of ', name))
    print(time)
    print(paste0('The output time of ', name))
    print(dates)
    print('done!!')
  
    return(list(input = time, output = dates, data = as.vector(data)))
  
  })
  
  str(res_time_value)
