# case 1
path_list <- list(list(name = 'EC-Earth',
                  path = paste0('/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/', 
                                'EC-Earth3/dcppA-hindcast/$member$/day/$var$/$grid$/$version$/',
                                '$var$_day_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_$grid$_$fyear$.nc')),
                  list(name = 'HadGEM3', 
                  path = paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/dcppA-hindcast/',
                         '$member$/day/$var$/$grid$/$version$/',
                         '$var$_day_HadGEM3-GC31-MM_dcppA-hidcast_s$sdate$_$member$_$grid$_$fyear$.nc')))

data <- Start(dataset = path_list,
             var = 'tasmin',
             grid = c('gr', 'gn'),
             version = c('v20190713', 'v20200101'),
             sdate = paste0(2018),
             fmonth = 'all',
             lat = values(list(0, 14)),
             lon = values(list(0, 28)),
             fyear = indices(1:2),
             member = c('r1i1p1f1', 'r1i1p1f2'),
             fyear_depends = 'sdate',
             fmonth_across = 'fyear',
             merge_across_dims = TRUE,
             synonims = list(fmonth = c('fmonth','time'), 
                             lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r200x100', method = 'conservative', crop = c(0,28,0,14)),
             transform_vars = c('lat', 'lon'),
             return_vars = list(lat = 'dataset', lon = 'dataset'),
             lat_reorder = Sort(),
             num_procs = 1, 
             retrieve = TRUE)

# case 2
path_list <- list(list(name = 'EC-Earth',
                  path = paste0('/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/',
                                'EC-Earth3/dcppA-hindcast/r1i1p1f1/day/$var$/gr/v20190713/',
                                '$var$_day_EC-Earth3_dcppA-hindcast_s$sdate$-r1i1p1f1_gr_$fyear$.nc')),
                  list(name = 'HadGEM3',
                  path = paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/dcppA-hindcast/',
                         'r1i1p1f2/day/$var$/gn/v20200101/',
                         '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$_r1i1p1f2_gn_$fyear$.nc')))

data <- Start(dataset = path_list,
             var = 'tasmin',
#             grid = c('gr', 'gn'),
#             version = c('v20190713', 'v20200101'),
             sdate = paste0(2018),
             fmonth = 'all',
             lat = values(list(0, 14)),
             lon = values(list(0, 28)),
             fyear = '20181101-20191031', #indices(1),
#             member = c('r1i1p1f1', 'r1i1p1f2'),
             fyear_depends = 'sdate',
             fmonth_across = 'fyear',
             merge_across_dims = TRUE,
             synonims = list(fmonth = c('fmonth','time'),
                             lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r200x100', method = 'conservative', crop = c(0,28,0,14)),
             transform_vars = c('lat', 'lon'),
             return_vars = list(lat = 'dataset', lon = 'dataset'),
             lat_reorder = Sort(),
             num_procs = 1,
             retrieve = FALSE)

# PROBLEM: The second file does not have $fyear$ defined
attr(data, 'ExpectedFiles')
, , 1, 1

     [,1]                                                                                                                                                                                                                            
[1,] "/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/day/tasmin/gr/v20190713/tasmin_day_EC-Earth3_dcppA-hindcast_s2018-r1i1p1f1_gr_20181101-20191031.nc"                           
[2,] "/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/dcppA-hindcast/r1i1p1f2/day/tasmin/gn/v20200101/tasmin_day_HadGEM3-GC31-MM_dcppA-hindcast_s2018_r1i1p1f2_gn_$fyear$.nc"





#--------------------
path_list <- list(ECE = list(name = 'EC-Earth',
                  path = paste0('/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/',
                                'EC-Earth3/dcppA-hindcast/$member$/day/$var$/gr/v20190713/',
                                '$var$_day_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$chunk$.nc')),
                  Had = list(name = 'HadGEM3', 
                  path = paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
                                'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/',
                                'dcppA-hindcast/$member$/day/$var$/gn/v20200101/',
                                '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-$member$_gn_$chunk$.nc')))

chunk_ecearth <- c('20181101-20191031', '20191101-20201031', '20201101-20211031', '20211101-20221031', '20221101-20231031', '20231101-20241031')
chunk_hadgem <- c('20181101-20281230', NA, NA, NA, NA, NA)
data <- Start(dataset = path_list,
             var = 'tasmin',
             member = list('r1i1p1f1', 'r1i1p1f2'),
             sdate = paste0(2018),
             chunk = list(r1i1p1f1 = chunk_ecearth, r1i1p1f2 = chunk_hadgem),
             time = 'all',
             lat = values(list(0, 14)),
             lon = values(list(0, 28)),
             chunk_depends = 'member',
             time_across = 'chunk',
              merge_across_dims = TRUE, 
             synonims = list(time = c('fmonth','time'), 
                             lon = c('lon', 'longitude'), 
                             lat = c('lat', 'latitude')),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r200x100', 
                  method = 'conservative', crop = c(0,28,0,14)),
             transform_vars = c('lat', 'lon'),
             return_vars = list(lat = 'dataset', lon = 'dataset'),
             lat_reorder = Sort(),
             num_procs = 1, retrieve = F)


#---------------------
# a more simple case
 repos <- "/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc"
  repos2 <- "/esarchive/exp/ecmwf/system4_m1/monthly_mean/$var$_f2h/$var$_$sdate$.nc"
  var <- 'tas'

  data <- Start(dat = list(list(name = 'system4_m1', path = repos2),
                           list(name = 'system5_m1', path = repos)),
                var = var,
                sdate = '20170101',
                ensemble = indices(1),
                time = indices(1),
                lat = 'all',
                lon = 'all',
                synonims = list(lat = c('lat', 'latitude'),
                                lon = c('lon', 'longitude')),
                return_vars = list(time = 'sdate',
                                   longitude = 'dat',
                                   latitude = 'dat'),
                metadata_dims = 'dat',  # it can be omitted since it is automatically specified as 'dat'
                retrieve = T
               )


#--------------------
# FAILED after the fix

path_list <- list(
              MPI = list(name = 'MPI-ESM',
                         path = paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',                                       'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',                                       'dcppA-hindcast/$member$/day/$var$/gn/v20200101/',                                       '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-$member$_gn_$chunk$.nc')),
              Had = list(name = 'HadGEM3',       
                         path = paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',                                      'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/',                                      'dcppA-hindcast/$member$/day/$var$/gn/v20200101/',                                      '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-$member$_gn_$chunk$.nc')))

data <- Start(dataset = path_list,
              var = 'tasmin',
              member = list(c('r1i1p1f1', 'r2i1p1f2')),  #indices(2) #work!
              sdate = paste0(2018),
              chunk = list(c('20181101-20281231', '20181101-20281230')), #indices(1) #work!
              time = indices(1), #'all',
              lat = values(list(0, 14)),
              lon = values(list(0, 28)),
             #chunk_depends = 'member',
             #time_across = 'chunk',
             # merge_across_dims = TRUE, 
              synonims = list(time = c('fmonth','time'), 
                              lon = c('lon', 'longitude'), 
                              lat = c('lat', 'latitude')),
              transform = CDORemapper,
              transform_extra_cells = 2,
              transform_params = list(grid = 'r200x100', 
                                      method = 'conservative', crop = c(0,28,0,14)),
              transform_vars = c('lat', 'lon'),
              return_vars = list(lat = 'dataset', lon = 'dataset'),
              lat_reorder = Sort(),
              num_procs = 1, 
              retrieve = T)
