# experimental data
# May to December 1994, monthly file with 6-hourly frequency
  repos <- paste0('/esarchive/exp/ecmwf/system4_m1/6hourly/',
                  '$var$/$var$_$sdate$.nc')
  sdate <- sapply(1994, function(x) paste0(x, sprintf('%02d', 5:12), '01'))

  system4 <- Start(dat = repos,
                   var = 'tas',
                   sdate = sdate,
                   time = indices(seq(1, 124, 4)),  #first time step per day
                   ensemble = 'all',
                   latitude = indices(1:10),
                   longitude = indices(1:10),
                   return_vars = list(latitude = NULL,
                                      longitude = NULL,
                                      time = c('sdate')))

#-------- Check exp data -----------
 attr(system4, 'Dimensions')
#      dat       var     sdate      time  ensemble  latitude longitude 
#        1         1         8        31        51        10        10 
#-----------------------------------

# ------- retrieve the attributes for obs load-in ----------
  dates <- attr(system4, 'Variables')$common$time
# dim(dates)
#sdate  time 
#    8    31 

# NOTE: Even though June, September, and November only have 30 days, it reads
#       31 days for each month. Therefore, the last day of these months is the
#       first day of the following month. No NA in this dates array.

# Manually retrieve the observation dates in the required format 
  dates_file <- sort(unique(gsub('-', '', sapply(as.character(dates),
  substr, 1, 7))))
# dates_file
#[1] "199405" "199406" "199407" "199408" "199409" "199410" "199411" "199412"
# -----------------------------------------------------------
  
# observational data
# May to December 1994, monthly file with 6-hourly frequency
  repos_obs <- paste0('/esarchive/recon/ecmwf/erainterim/6hourly/',
                      '$var$/$var$_$file_date$.nc')
  erai <- Start(dat = repos_obs,
                var = 'tas',
                file_date = dates_file,
                time = values(dates), 
                latitude = indices(1:10),
                longitude = indices(1:10),
                time_var = 'time',
                #because time is assigned by 'values', set the tolerance to avoid too distinct match
                time_tolerance = as.difftime(1, units = 'hours'), 
                #time values are across all the files
                time_across = 'file_date',  
                #combine time and file_date dims 
                merge_across_dims = TRUE,
                #exclude the additional NAs generated by merge_across_dims
                merge_across_dims_narm = TRUE,
                #split time dim, because it is two-dimensional
                split_multiselected_dims = TRUE,
                return_vars = list(latitude = NULL,
                                   longitude = NULL,
                                   time = 'file_date'),
                retrieve = TRUE)

dim(erai)
#      dat       var     sdate      time  latitude longitude 
#        1         1         8        31        10        10 

 dim(attr(erai, 'Variables')$common$time)

