library(startR)

#--------------monthly---------------
# exp
repos_exp <- paste0('/esarchive/exp/ecearth/a1tr/cmorfiles/CMIP/EC-Earth-Consortium/',
                    'EC-Earth3/historical/r24i1p1f1/Amon/$var$/gr/v20190312/',
                    '$var$_Amon_EC-Earth3_historical_r24i1p1f1_gr_$sdate$01-$sdate$12.nc')
exp <- Start(dat = repos_exp,
             var = 'tas',
             sdate = as.character(c(1978:1979)),
             time = indices(1:3),
             lat = 'all',
             lon = 'all',
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'sdate'),
             retrieve = FALSE)

lats <- attr(exp, 'Variables')$common$lat
dates <- attr(exp, 'Variables')$common$time
dim(dates)
#sdate  time 
#    2     3 


repos_obs <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$date$.nc'

obs <- Start(dat = repos_obs,
             var = 'tas',
             date = unique(format(dates, '%Y%m')),
             time = values(dates),  #dim: [sdate = 2, time = 3]
             lat = values(lats),
             lon = 'all',
             time_across = 'date',
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'date'),
             retrieve = TRUE)

dim(obs)
  dat   var sdate  time   lat   lon 
    1     1     2     3   256   512

obs[1,1,,,1,1]
         [,1]     [,2]     [,3]
[1,] 250.3432 250.3432 250.3432
[2,] 250.3432 237.9695       NA
# Problems here may be:
# 1. choose the closest value. So the first row (1978) is all 1979 1st values.
# 2. If don't use 'merge_across_dims' and 'split_multiselected_dims', the returned dim 
#    will be [date = 3, time = 4]. time = 4 because all 1978 values are coarsed to 197901,
#    so file_197901 has 4 values. Start() decides the dimension by the longest file, so time = 4.
#    As for date = 3, it is the three 'time'. c(250.3432, 237.9695, 226.5861) 
#    is the three time steps in 1979. 
#obs[1,1,,,1,1]
#         [,1]     [,2]     [,3]     [,4]
#[1,] 250.3432 250.3432 250.3432 250.3432
#[2,] 237.9695       NA       NA       NA
#[3,] 226.5861       NA       NA       NA
#    Then, use 'merge_across_dims' and 'split_multiselected_dims', the final dim becomes
#    [sdate = 2, time = 3]. So only the first 6 values are included.


# date (file dim) is okay to have out-of-range values. The problem lies in time (inner dim).
# The following code has no problem because time is not assigned by values.
obs2 <- Start(dat = repos_obs,
             var = 'tas',
             date = unique(format(dates, '%Y%m')), # c('197801', '197802', '197803', '197901', '197902', '197903'), 
             time = c(1), #values(dates),  #dim: [sdate = 2, time = 3]
             lat = values(lats),
             lon = 'all',
#             time_across = 'date',
#             merge_across_dims = TRUE,
#             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'date'),
             retrieve = TRUE)

attr(obs2, 'Variables')$common$time
[1] NA                        "1979-01-31 18:00:00 UTC"
[3] NA                        "1979-02-28 18:00:00 UTC"
[5] NA                        "1979-03-31 18:00:00 UTC"
dim(attr(obs2, 'Variables')$common$time)
date time 
   6    1

obs2[1,1,,,1,1]
[1]       NA 250.3432       NA 237.9695       NA 226.5861
