library(startR)

  # Get time values
  repos <- '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc'
  data <- Start(dat = repos,
                var = 'tas',
                sdate = c('20170101', '20180101'),
                ensemble = indices(1),
                time = indices(1:4), #'all',
                latitude = indices(1),
                longitude = indices(1),
                return_vars = list(time = 'sdate'),
                retrieve = F)
  time_val <- attr(data, 'Variables')$common$time


basic_list <- list(
  dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
  var = 'tas',
  sdate = c('20170101', '20180101'),
  latitude = indices(1:3),
  longitude = indices(1:2),
  retrieve = F
)

tests <- list(
  # 1: ensemble and time are indices. no *_var assigned
  c(basic_list, list(ensemble = c(1, 3)), list(time = indices(1:4))),

  # 2: ensemble is values. ensemble_var assigned.
  c(basic_list, list(ensemble = values(c(1, 3))), list(time = indices(1:4)), 
    ensemble_var = 'realization'),

  # 3: ensemble and time are values. ensemble_var assigned. time_var will be added automatically.
  c(basic_list, list(ensemble = values(c(1, 3))), list(time = time_val),
    ensemble_var = 'realization'),

  # 4: same as 2 but time_var is assigned, and return_vars = list(time = NULL).
  c(basic_list, list(ensemble = values(c(1, 3))), list(time = indices(1:4)),
    ensemble_var = 'realization', time_var = 'time',
    list(return_vars = list(time = NULL))),

  # 5: same as 4 but return_var = list(time = 'sdate'). ERROR!!
  c(basic_list, list(ensemble = values(c(1, 3))), list(time = indices(1:4)),
    ensemble_var = 'realization', time_var = 'time',
    list(return_vars = list(time = 'sdate'))),

  # 6: same as 5 but time with dim.
  c(basic_list, list(ensemble = values(c(1, 3))), list(time = array(1:4, dim = c(time = 4, sdate = 2))), 
    ensemble_var = 'realization', time_var = 'time',
    list(return_vars = list(time = 'sdate'))),

  # 7: same as 1 but add latitude_var = 'latitude'
  c(basic_list, list(ensemble = c(1, 3)), list(time = indices(1:4)),
    latitude_var = 'latitude'),

  # 8: same as 1 but add latitude = NULL in return_vars
  c(basic_list, list(ensemble = c(1, 3)), list(time = indices(1:4)),
    list(return_vars = list(latitude = NULL)))
)

test_batteries <- list(tests[[1]])

for (battery_ind in 1:length(test_batteries)) {
  battery <- test_batteries[[battery_ind]]
  call <- list()
  cat(paste0("Test ", battery_ind, "...\n"))
  call[names(battery[[battery_ind]])] <- battery
  data <- do.call(Start, battery)
}

warnings()

# 19: compare with 6.
dat <- Start(
  dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = array(1:4, dim = c(time = 4, sdate = 2)),
  ensemble = values(c(1, 3)), 
  ensemble_var = 'realization')



# 20: To compare with 3. time is a vector without name. metadata is still returned.
dat <- Start(
  dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
  var = 'tas',
  sdate = '20170101',  #c('20170101', '20180101'),
  time = time_val[1, ],
  ensemble = values(c(1, 3)), 
  ensemble_var = 'realization',
  latitude = indices(1:3),
  longitude = indices(1:2),
  retrieve = F)

# 21: To compare wih 5. All the same but without time_var = 'time'. 
#time is indices and has no named dimension. return_vars time = 'sdate' works.
dat <- Start(
  dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = indices(1:4), 
  ensemble = values(c(1, 3)),     
  ensemble_var = 'realization',
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = 'sdate'),
  retrieve = F)

# 22: To compare with 20 and 21. time is values, and it must have dimname so return_vars can recognize. If time is not values (#21) or time is not in return_vars (#20), it works without dimname.
tmp <- array(time_val[1, ], dim = c(sdate = 1, time = 4))
tmp <- as.POSIXct(tmp, origin = '1970-01-01 UTC', tz = 'UTC')
dat <- Start(
  dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
  var = 'tas',
  sdate = c('20170101'),
  time = tmp,
  ensemble = values(c(1, 3)),
  ensemble_var = 'realization',
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = 'sdate'),
  retrieve = F)
