library(startR)

# ATL
lonmin <- -80
lonmax <-  50
latmin <- -60
latmax <-  50

# exp
repos_exp <-  paste0('/esarchive/scratch/eexarcho/Eleftheria/TRIATLAS/Analysis_a33d_a33e_a33f/',
                       'Data/a33d/tos/', 
                      'tos_Amon_EC-Earth3-CC_historical_S$sdate$_$member$_gr_$chunk$.nc'
				          )
sdates <-  paste0(c(2005:2006), '0501')

exp <- Start(dat = repos_exp,
             var = 'tos',
             member = 'all',
             sdate = sdates,
             chunk = 'all', 
          #   time = 'all',
             time = indices(1:12),  #first time step per day
             chunk_depends = 'sdate',
             time_across = 'chunk',
             merge_across_dims = TRUE,
             lat = values(list(latmin, latmax)),
             lat_reorder = Sort(), #Sort(decreasing = T),
             lon = values(list(lonmin, lonmax)),
             lon_reorder = CircularSort(-180, 180), #CircularSort(0, 360),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r360x180',
                                     method = 'conservative',
                                     crop = c(lonmin, lonmax, latmin, latmax)),
             transform_vars = c('lat', 'lon'),
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = 'dat',
                                lat = 'dat',
                                time = 'sdate'),
             retrieve = T)
lons <- attr(exp, 'Variables')$common$tos$dim[[1]]$vals
lats <- attr(exp, 'Variables')$common$tos$dim[[2]]$vals
dates <- attr(exp, 'Variables')$common$time
dim(dates)
#sdate  time 
#    2    12 

dates_file <- sort(unique(gsub('-', '', sapply(as.character(dates), substr, 1, 7))))

repos_obs <- '/esarchive/obs/ukmo/hadisst_v1.1/monthly_mean/$var$/$var$_$date$.nc'
obs <- Start(dat = repos_obs,
             var = 'tos',
             date = dates_file,
             time = values(dates),  #dim: [sdate = 2, time = 12]
             lat = values(lats),
             lon = values(lons),
             #because time is assigned by 'values', set the tolerance to avoid too distinct match
             time_var = 'time',
#             time_tolerance = as.difftime(372, units = 'hours'), 
             #time values are across all the files
             time_across = 'date',  
             #combine time and file_date dims 
             merge_across_dims = TRUE,
             #exclude the additional NAs generated by merge_across_dims
             merge_across_dims_narm = TRUE,
             #split time dim, because it is two-dimensional
             split_multiselected_dims = TRUE,

             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r360x180',
                                     method = 'conservative',
                                     crop = c(lonmin, lonmax, latmin, latmax)),
             transform_vars = c('lat', 'lon'),

             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lat = NULL,
                                lon = NULL,
                                time = 'date'),
                retrieve = TRUE)

#Error in while (indices_chunk[i + 1] == indices_chunk[i] & i < length(indices_chunk)) { : 
#  missing value where TRUE/FALSE needed


#----------Plotting-----------------
obs_lon <- as.vector(attr(obs, 'Variables')$common$lon)
obs_lat <- as.vector(attr(obs, 'Variables')$common$lat)
s2dv::PlotEquiMap(obs[1,1,1,1, , ], obs_lon, obs_lat,
            filled.continents = F, fileout = '/home/Earth/aho/issue85_obs.eps')




#================Part 2 (issue86)====================
obs <- Start(dat = repos_obs,
             var = 'tos',
             date = dates_file,
             time = 'all', 
             lat = values(lats),
             lon = values(lons),
             time_across = 'date',  
             #combine time and file_date dims 
             merge_across_dims = TRUE,
             #exclude the additional NAs generated by merge_across_dims
             merge_across_dims_narm = TRUE,

             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r360x180',
                                     method = 'conservative',
                                     crop = c(lonmin, lonmax, latmin, latmax)),
             transform_vars = c('lat', 'lon'),

             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lat = NULL,
                                lon = NULL,
                                time = 'date'),
             retrieve = TRUE)

attr(obs, 'Variables')$common$time  #WRONG!!!!





#////////////////////////////////////////////////////////
repos_exp <-  paste0('/esarchive/exp/ecearth/a33d/diags/CMIP/',
                     'EC-Earth-Consortium/EC-Earth3-CC/historical/',
                     '$member$/Omon/tos/r360x180/v20200915/',
                     'tos_Omon_EC-Earth3-CC_historical_$member$_r360x180_$chunk$.nc')

#sdates <-  paste0(c(2005:2006), '0501')

exp <- Start(dat = repos_exp,
             var = 'tos',
             member = 'all',
#             sdate = sdates,
             chunk = c('200505-200604', '200605-200704'), #'all', 
             time = indices(1:12),  
             lat = values(list(latmin, latmax)),
             lat_reorder = Sort(),
             lon = values(list(lonmin, lonmax)),
             lon_reorder = CircularSort(-180, 180),
             synonims = list(lat = c( 'latitude',  'lat'),
                             lon = c( 'longitude', 'lon') ),
             return_vars = list(lon = NULL, #'dat',
                                lat = NULL, #'dat',
                                time = 'chunk'),
             retrieve = F)

