library(startR)


repos <- '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc'

# Get time values
data <- Start(dat = repos,
              var = 'tas',
              sdate = c('20170101', '20180101'),
              ensemble = indices(1),
              time = indices(1:4),
              latitude = indices(1), longitude = indices(1),
              return_vars = list(time = 'sdate'),
              retrieve = F)
time_val <- attr(data, 'Variables')$common$time
dim(time_val)
#sdate  time 
#    2     4 
time_val
#[1] "2017-02-01 UTC" "2018-02-01 UTC" "2017-03-01 UTC" "2018-03-01 UTC"
#[5] "2017-04-01 UTC" "2018-04-01 UTC" "2017-05-01 UTC" "2018-05-01 UTC"

#----------------------------------------------------------------------

#=========== indices ===============

## It is correct.
test7 <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = indices(1:4),
#  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
#  return_vars = list(time = NULL),
  retrieve = TRUE)
attr(test7, 'Variables')$common$time
#NULL

## It is correct.
test9 <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = indices(1:4),
#  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = NULL),
  retrieve = TRUE)
attr(test9, 'Variables')$common$time
#[1] "2017-02-01 UTC" "2017-03-01 UTC" "2017-04-01 UTC" "2017-05-01 UTC"

## It is correct.
test10 <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = indices(1:4),
#  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = 'sdate'),
  retrieve = TRUE)
attr(test10, 'Variables')$common$time
#[1] "2017-02-01 UTC" "2018-02-01 UTC" "2017-03-01 UTC" "2018-03-01 UTC"
#[5] "2017-04-01 UTC" "2018-04-01 UTC" "2017-05-01 UTC" "2018-05-01 UTC"

all.equal(test7, test9, test10, check.attributes = F)
[1] TRUE


#========== values with dimensions ==============

## It is correct.
test4 <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = time_val,
#  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = 'sdate'),
  retrieve = TRUE)
attr(test4, 'Variables')$common$time
#[1] "2017-02-01 UTC" "2018-02-01 UTC" "2017-03-01 UTC" "2018-03-01 UTC"
#[5] "2017-04-01 UTC" "2018-04-01 UTC" "2017-05-01 UTC" "2018-05-01 UTC"

## It is correct.
test4a <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = time_val,
  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = 'sdate'),
  retrieve = TRUE)
attr(test4a, 'Variables')$common$time
#[1] "2017-02-01 UTC" "2018-02-01 UTC" "2017-03-01 UTC" "2018-03-01 UTC"
#[5] "2017-04-01 UTC" "2018-04-01 UTC" "2017-05-01 UTC" "2018-05-01 UTC"


## The time attributes were wrong. Data was correct but due to accident.
## Corrected now.
test6 <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = time_val,
#  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = NULL),
  retrieve = TRUE)
attr(test6, 'Variables')$common$time
#[1] "2017-02-01 UTC" "2018-02-01 UTC" "2017-03-01 UTC" "2018-03-01 UTC"
#[5] "2017-04-01 UTC" "2018-04-01 UTC" "2017-05-01 UTC" "2018-05-01 UTC"


## data and time attributes were wrong.
## Corrected now.
test6a <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = time_val,
  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = NULL),
  retrieve = TRUE)
attr(test6a, 'Variables')$common$time
#[1] "2017-02-01 UTC" "2018-02-01 UTC" "2017-03-01 UTC" "2018-03-01 UTC"
#[5] "2017-04-01 UTC" "2018-04-01 UTC" "2017-05-01 UTC" "2018-05-01 UTC"

all.equal(test7, test6, check.attributes = F)
all.equal(test4, test4a, test6, test6a, check.attributes = F)




#========== values without dimensions ==============
time_val_vector <- time_val[c(1, 3, 5, 7, 2, 4, 6, 8)]

## This call is logically wrong.
test14 <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = time_val_vector,
#  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = NULL),
  retrieve = TRUE)
attr(test14, 'Variables')$common$time


## data and time attributes were wrong.
## Corrected now.
test14a <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = time_val_vector,
  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = NULL),
  retrieve = TRUE)
attr(test14a, 'Variables')$common$time
#[1] "2017-02-01 UTC" "2018-02-01 UTC" "2017-03-01 UTC" "2018-03-01 UTC"
#[5] "2017-04-01 UTC" "2018-04-01 UTC" "2017-05-01 UTC" "2018-05-01 UTC"

## This call is logically wrong.
test15 <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = time_val_vector,
#  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = 'sdate'),
  retrieve = TRUE)
attr(test15, 'Variables')$common$time


## It is correct.
test15a <- Start(
  dat = repos,
  var = 'tas',
  sdate = c('20170101', '20180101'),
  time = time_val_vector,
  time_across = 'sdate',
  ensemble = c(1, 3),
  latitude = indices(1:3),
  longitude = indices(1:2),
  return_vars = list(time = 'sdate'),
  retrieve = TRUE)
attr(test15a, 'Variables')$common$time
#[1] "2017-02-01 UTC" "2018-02-01 UTC" "2017-03-01 UTC" "2018-03-01 UTC"
#[5] "2017-04-01 UTC" "2018-04-01 UTC" "2017-05-01 UTC" "2018-05-01 UTC"


all.equal(test7, test14a, test15a, check.attributes = F)
