library(startR)

obs.path <- "/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$file_date$.nc"
variable <- "prlr"
dates_file <- c("201311","201312","201411","201412")
dim(dates_file) <- c(time = 2, syear = 2)
lats.min <- 35.6
lats.max <- 44
longitude_reorder <- CircularSort(-180, 180)
lons.min <- 10
lons.max <- 20

obs <- Start(dat = obs.path,
             var = variable,
             file_date = dates_file,
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(decreasing = TRUE),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = longitude_reorder,
             synonims = list(latitude=c('lat','latitude'),
                             longitude=c('lon','longitude')), 
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = 'file_date'),
             split_multiselected_dims = TRUE,
             retrieve = TRUE)


obs <- Start(dat = obs.path,
             var = variable,
             file_date = dates_file,
             time = 'all',
             time_across = 'file_date',
             merge_across_dims = TRUE,
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(decreasing = TRUE),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = longitude_reorder,
             synonims = list(latitude=c('lat','latitude'),
                             longitude=c('lon','longitude')),
                            #  file_date = c('file_date', 'time')), 
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = 'file_date'),
             split_multiselected_dims = TRUE,
             retrieve = TRUE)




# -----------------------------------------------------------
# If time length > 1, it cannot be implicit.
#------------------------------------------------------------
tmp2 <- c('199406', '199407', '199408', '199409', '199410', '199411')
dim(tmp2) <- c(month = 3, season = 2)
 
# observational data
# May to December 1994, monthly file with 6-hourly frequency
  repos_obs <- paste0('/esarchive/recon/ecmwf/erainterim/6hourly/',
                      '$var$/$var$_$file_date$.nc')
  erai <- Start(dat = repos_obs,
                var = 'tas',
                file_date = tmp2, #dates_file,
#                time = 'all', 
                latitude = indices(1:10),
                longitude = indices(1:10),
#                time_across = 'file_date',  
                split_multiselected_dims = TRUE,
                return_vars = list(latitude = NULL,
                                   longitude = NULL,
                                   time = 'file_date'),
                retrieve = TRUE)

