#----------------------------------------------------
# Different time index bwtween files
library(startR)

variable <- 'tas'
sdates <- '2000'
time <- as.POSIXct("2000-11-16", tz = 'UTC')
## Paths
# HadGEM3 (initialised in November)
path_hadgem3 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/cmip6-dcppA-hindcast_i1p1/DCPP/',
                       'MOHC/HadGEM3-GC31-MM/dcppA-hindcast/r1i1p1f2/Amon/$var$/gr/v20200316/',
                       '$var$_*_s$sdate$_r1i1p1f2_gr_$fyear$.nc')
#NorCPM1 (initialised in October)
path_norcpm1 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/norcpm1/cmip6-dcppA-hindcast_i1p1/DCPP/',
                       'NCC/NorCPM1/dcppA-hindcast/r1i1p1f1/Amon/$var$/gn/v20200320/',
                       '$var$_Amon_NorCPM1_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')


tmp <- Start(dat = path_hadgem3,
             var = 'tas',
             sdate = sdates,
             fyear = indices(1), ## there is one file per sdate
             fyear_depends = 'sdate',
             time = indices(1),
             lat = values(list(0, 10)),
             lon = values(list(0, 10)),
             synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
             transform = CDORemapper,
             transform_params = list(grid = 'r180x90',
                                     method = 'conservative',
                                     crop = c(0, 10, 0, 10)),
             transform_vars = c('lat', 'lon'),
             return_vars = list(lat = NULL, lon = NULL, 
                                time = 'sdate'),
             retrieve = FALSE)

time <- attr(tmp, 'Variables')$common$time
#[1] "2000-11-16 UTC"

#------------------------------------------------------

# hadgem3 first, norcpm1 second. time is indices.
data <- Start(dat = list(list(name = 'hadgem3', path = path_hadgem3),
                         list(name = 'norcpm1', path = path_norcpm1)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = indices(1), #time,
                   lat = values(list(50, 60)),
                   lon = values(list(50, 60)),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   transform = CDORemapper,
                   transform_params = list(grid = 'r180x90',
                                           method = 'conservative',
                                           crop = c(50, 60, 50, 60)),
                   transform_vars = c('lat', 'lon'),
                   return_vars = list(lat = 'dat', lon = 'dat', 
                                      time = 'dat'),
                   retrieve = TRUE)

dim(data)
  dat   var sdate fyear  time   lat   lon 
    2     1     1     1     1     5     6 

# norcpm1 first, hadgem3 second. time is indices.
data_r <- Start(dat = list(list(name = 'norcpm1', path = path_norcpm1),
                         list(name = 'hadgem3', path = path_hadgem3)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = indices(1), #time,
                   lat = values(list(50, 60)),
                   lon = values(list(50, 60)),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   transform = CDORemapper,
                   transform_params = list(grid = 'r180x90',
                                           method = 'conservative',
                                           crop = c(50, 60, 50, 60)),
                   transform_vars = c('lat', 'lon'),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)

# hadgem3 first, norcpm1 second. time is value.
data_v <- Start(dat = list(list(name = 'hadgem3', path = path_hadgem3),
                         list(name = 'norcpm1', path = path_norcpm1)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = time,
                   lat = values(list(50, 60)),
                   lon = values(list(50, 60)),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   transform = CDORemapper,
                   transform_params = list(grid = 'r180x90',
                                           method = 'conservative',
                                           crop = c(50, 60, 50, 60)),
                   transform_vars = c('lat', 'lon'),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)

# norcpm1 first, hadgem3 second. time is value.
data_r_v <- Start(dat = list(list(name = 'norcpm1', path = path_norcpm1),
                         list(name = 'hadgem3', path = path_hadgem3)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = time,
                   lat = values(list(50, 60)),
                   lon = values(list(50, 60)),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   transform = CDORemapper,
                   transform_params = list(grid = 'r180x90',
                                           method = 'conservative',
                                           crop = c(50, 60, 50, 60)),
                   transform_vars = c('lat', 'lon'),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)

data1 <- Start(dat = list(list(name = 'hadgem3', path = path_hadgem3)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = indices(1), #time,
                   lat = values(list(50, 60)),
                   lon = values(list(50, 60)),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   transform = CDORemapper,
                   transform_params = list(grid = 'r180x90',
                                           method = 'conservative',
                                           crop = c(50, 60, 50, 60)),
                   transform_vars = c('lat', 'lon'),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)

data2 <- Start(dat = list(list(name = 'norcpm1', path = path_norcpm1)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = indices(1), #time,
                   lat = values(list(50, 60)),
                   lon = values(list(50, 60)),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   transform = CDORemapper,
                   transform_params = list(grid = 'r180x90',
                                           method = 'conservative',
                                           crop = c(50, 60, 50, 60)),
                   transform_vars = c('lat', 'lon'),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)

data1[1,1,1,1,1,,]
         [,1]     [,2]     [,3]     [,4]     [,5]     [,6]
[1,] 273.3338 272.7383 272.0932 271.3294 270.6414 270.8529
[2,] 273.2986 272.2289 271.3160 270.3497 269.0654 270.0157
[3,] 273.3144 272.4099 271.9105 271.2018 268.9900 269.7906
[4,] 273.0627 272.4097 272.0739 271.2791 270.0007 269.8668
[5,] 272.9395 272.0510 271.6794 271.3029 269.0366 268.9892

data2[1,1,1,1,1,,]
         [,1]     [,2]     [,3]     [,4]     [,5]     [,6]
[1,] 278.5213 278.8991 278.8683 278.7591 278.6861 278.8570
[2,] 276.1401 276.2737 276.3291 276.3971 276.5144 276.7959
[3,] 274.0771 273.9838 273.9749 274.0872 274.3201 274.6716
[4,] 271.2272 271.1368 271.1548 271.2510 271.4596 272.0138
[5,] 266.3823 266.2245 266.3170 266.5630 266.9609 267.6820



> data[1,1,1,1,1,,]  # CORRECT, indices(1) of hadgem3
         [,1]     [,2]     [,3]     [,4]     [,5]     [,6]
[1,] 273.3338 272.7383 272.0932 271.3294 270.6414 270.8529
[2,] 273.2986 272.2289 271.3160 270.3497 269.0654 270.0157
[3,] 273.3144 272.4099 271.9105 271.2018 268.9900 269.7906
[4,] 273.0627 272.4097 272.0739 271.2791 270.0007 269.8668
[5,] 272.9395 272.0510 271.6794 271.3029 269.0366 268.9892
> data[2,1,1,1,1,,]
     [,1] [,2] [,3] [,4] [,5] [,6]
[1,]   NA   NA   NA   NA   NA   NA
[2,]   NA   NA   NA   NA   NA   NA
[3,]   NA   NA   NA   NA   NA   NA
[4,]   NA   NA   NA   NA   NA   NA
[5,]   NA   NA   NA   NA   NA   NA


> data_r[1,1,1,1,1,,]  # CORRECT, indices(1) of norcpm1
         [,1]     [,2]     [,3]     [,4]     [,5]     [,6]
[1,] 278.5213 278.8991 278.8683 278.7591 278.6861 278.8570
[2,] 276.1401 276.2737 276.3291 276.3971 276.5144 276.7959
[3,] 274.0771 273.9838 273.9749 274.0872 274.3201 274.6716
[4,] 271.2272 271.1368 271.1548 271.2510 271.4596 272.0138
[5,] 266.3823 266.2245 266.3170 266.5630 266.9609 267.6820
> data_r[2,1,1,1,1,,] # Should be indices(1) of hadgem3
         [,1]     [,2]     [,3]     [,4]     [,5]     [,6]
[1,] 277.1243 277.0988 277.0956 277.0995 277.1097 277.1397
[2,] 277.6877 277.6640 277.6501 277.6362 277.6264 277.6459
[3,] 278.2408 278.2435 278.2434 278.2263 278.1976 278.1932
[4,] 278.8050 278.8283 278.8530 278.8607 278.8535 278.8620
[5,] 279.3870 279.4206 279.4523 279.4795 279.5048 279.5528


> data_v[1,1,1,1,1,,] # CORRECT, indices(1) of hadgem3
         [,1]     [,2]     [,3]     [,4]     [,5]     [,6]
[1,] 273.3338 272.7383 272.0932 271.3294 270.6414 270.8529
[2,] 273.2986 272.2289 271.3160 270.3497 269.0654 270.0157
[3,] 273.3144 272.4099 271.9105 271.2018 268.9900 269.7906
[4,] 273.0627 272.4097 272.0739 271.2791 270.0007 269.8668
[5,] 272.9395 272.0510 271.6794 271.3029 269.0366 268.9892
> data_v[2,1,1,1,1,,]
     [,1] [,2] [,3] [,4] [,5] [,6]
[1,]   NA   NA   NA   NA   NA   NA
[2,]   NA   NA   NA   NA   NA   NA
[3,]   NA   NA   NA   NA   NA   NA
[4,]   NA   NA   NA   NA   NA   NA
[5,]   NA   NA   NA   NA   NA   NA
> data_r_v[1,1,1,1,1,,]  # CORRECT, indices(2) of norcpm1
         [,1]     [,2]     [,3]     [,4]     [,5]     [,6]
[1,] 276.3354 275.7599 274.9832 274.3458 273.9036 273.3848
[2,] 274.7567 274.2432 273.4083 272.6837 272.1753 271.7393
[3,] 273.4592 273.2082 272.6087 271.8571 271.1251 270.8244
[4,] 272.2585 271.9230 271.4623 270.8910 270.3026 269.9609
[5,] 270.8980 270.4685 270.0259 269.4687 268.8188 268.1013
> data_r_v[2,1,1,1,1,,] # Should be indices(1) of hadgem3
         [,1]     [,2]     [,3]     [,4]     [,5]     [,6]
[1,] 277.1243 277.0988 277.0956 277.0995 277.1097 277.1397
[2,] 277.6877 277.6640 277.6501 277.6362 277.6264 277.6459
[3,] 278.2408 278.2435 278.2434 278.2263 278.1976 278.1932
[4,] 278.8050 278.8283 278.8530 278.8607 278.8535 278.8620
[5,] 279.3870 279.4206 279.4523 279.4795 279.5048 279.5528


#------------no transform (no problem)-----------------

# hadgem3 first, norcpm1 second. time is indices.
data <- Start(dat = list(list(name = 'hadgem3', path = path_hadgem3),
                         list(name = 'norcpm1', path = path_norcpm1)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = indices(1), #time,
                   lat = indices(1:3),
                   lon = indices(1:2),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)

# norcpm1 first, hadgem3 second. time is indices.
data_r <- Start(dat = list(list(name = 'norcpm1', path = path_norcpm1),
                         list(name = 'hadgem3', path = path_hadgem3)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = indices(1), #time,
                   lat = indices(1:3),
                   lon = indices(1:2),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)


# hadgem3 first, norcpm1 second. time is value.
data_v <- Start(dat = list(list(name = 'hadgem3', path = path_hadgem3),
                         list(name = 'norcpm1', path = path_norcpm1)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = time,
                   lat = indices(1:3),
                   lon = indices(1:2),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   return_vars = list(lat = 'dat', lon = 'dat',
                                      time = 'dat'),
                   retrieve = TRUE)

# norcpm1 first, hadgem3 second. time is value.
data_r_v <- Start(dat = list(list(name = 'norcpm1', path = path_norcpm1),
                         list(name = 'hadgem3', path = path_hadgem3)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = time,
                   lat = indices(1:3),
                   lon = indices(1:2),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)


data1 <- Start(dat = list(list(name = 'hadgem3', path = path_hadgem3)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = indices(1), #time,
                   lat = indices(1:3),
                   lon = indices(1:2),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)

data2 <- Start(dat = list(list(name = 'norcpm1', path = path_norcpm1)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = indices(1), #time,
                   lat = indices(1:3),
                   lon = indices(1:2),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)

data1[1,1,1,1,1,,]
         [,1]     [,2]
[1,] 244.4800 244.4780
[2,] 245.1499 245.1448
[3,] 245.5728 245.5693

data2[1,1,1,1,1,,]
         [,1]     [,2]
[1,] 228.5741 228.5114
[2,] 227.6276 227.4960
[3,] 229.3816 229.1060


#----------------use multiStart-------------------


data <- multiStart(dat = list(list(name = 'hadgem3', path = path_hadgem3),
                              list(name = 'norcpm1', path = path_norcpm1)),
                   var = 'tas',
                   sdate = sdates,
                   fyear = indices(1), ## there is one file per sdate
                   fyear_depends = 'sdate',
                   time = list(list(name = 'hadgem3', time = indices(1)),
                               list(name = 'norpcm1', time = indices(2))),
                   lat = values(list(50, 60)),
                   lon = values(list(50, 60)),
                   synonims = list(lon = c('lon', 'longitude'), lat = c('lat', 'latitude')),
                   transform = CDORemapper,
                   transform_params = list(grid = 'r180x90',
                                           method = 'conservative',
                                           crop = c(50, 60, 50, 60)),
                   transform_vars = c('lat', 'lon'),
                   return_vars = list(lat = NULL, lon = NULL,
                                      time = 'dat'),
                   retrieve = TRUE)
