library(startR)

# exp
repos_exp <- paste0('/esarchive/exp/ecearth/a1tr/cmorfiles/CMIP/EC-Earth-Consortium/',
                    'EC-Earth3/historical/r24i1p1f1/Amon/$var$/gr/v20190312/',
                    '$var$_Amon_EC-Earth3_historical_r24i1p1f1_gr_$sdate$01-$sdate$12.nc')

exp <- Start(dat = repos_exp,
             var = 'tas',
             sdate = as.character(c(2005:2008)),
             time = indices(1:3),
             lat = indices(1), #'all',
             lon = indices(1), #'all',
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'sdate'),
             retrieve = FALSE)
dates <- attr(exp, 'Variables')$common$time

dim(dates)
sdate  time 
    4     3 
dates
 [1] "2005-01-16 12:00:00 UTC" "2006-01-16 12:00:00 UTC"
 [3] "2007-01-16 12:00:00 UTC" "2008-01-16 12:00:00 UTC"
 [5] "2005-02-15 00:00:00 UTC" "2006-02-15 00:00:00 UTC"
 [7] "2007-02-15 00:00:00 UTC" "2008-02-15 12:00:00 UTC"
 [9] "2005-03-16 12:00:00 UTC" "2006-03-16 12:00:00 UTC"
[11] "2007-03-16 12:00:00 UTC" "2008-03-16 12:00:00 UTC"

# obs
repos_obs <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$date$.nc'

obs <- Start(dat = repos_obs,
             var = 'tas',
             date = unique(format(dates, '%Y%m')),
             time = values(dates),  #dim: [sdate = 4, time = 3]
             lat = indices(1), #values(lats),
             lon = indices(1), #'all',
             time_across = 'date',
             merge_across_dims = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'date'),
             retrieve = TRUE)


dim(obs)
  dat   var sdate  time   lat   lon 
    1     1     4     3     1     1 

## check with netCDF file
dates[1, 1]
#[1] "2005-01-16 12:00:00 UTC"
obs[1,1,1,1,1,1]
#[1] 250.7856
file200501 <- NcOpen('/esarchive/recon/ecmwf/erainterim/monthly_mean/tas_f6h/tas_200501.nc')
obs200501 <- NcToArray(file200501, vars_to_read = 'tas', dim_indices = list(lon = 1, lat = 1, time = 1))
NcClose(file200501)
obs200501[1]
#[1] 250.7856

dates[1, 2]
#[1] "2005-02-15 UTC"
obs[1,1,1,2,1,1]
#[1] 257.1013
file200502 <- NcOpen('/esarchive/recon/ecmwf/erainterim/monthly_mean/tas_f6h/tas_200502.nc')
obs200502 <- NcToArray(file200502, vars_to_read = 'tas', dim_indices = list(lon = 1, lat = 1, time = 1))
NcClose(file200502)
obs200502[1]
#[1] 247.7073

file200601 <- NcOpen('/esarchive/recon/ecmwf/erainterim/monthly_mean/tas_f6h/tas_200601.nc')
obs200601 <- NcToArray(file200601, vars_to_read = 'tas', dim_indices = list(lon = 1, lat = 1, time = 1))
NcClose(file200601)
obs200601[1]
#[1] 257.1013

#----------------------------------------------------------------------------
library(easyNCDF)
path <- paste0('/esarchive/recon/ecmwf/erainterim/monthly_mean/tas_f6h/tas_', 
               rep(2005:2008, each = 3), '0', 1:3, '.nc')
obs_netcdf <- rep(0, length(path))
for (i in 1:length(path)) {
  file <- NcOpen(path[i])
  obs_netcdf[i] <- NcToArray(file, vars_to_read = 'tas', dim_indices = list(lon = 1, lat = 1, time = 1))
  NcClose(file)
}

dim(obs_netcdf) <- c(time = 3, sdate = 4)
obs_netcdf
         [,1]     [,2]     [,3]     [,4]
[1,] 250.7856 257.1013 244.6135 240.0394
[2,] 247.7073 243.7262 249.1279 246.1091
[3,] 243.6829 244.7054 247.1041 246.2024

t(obs_netcdf)
         [,1]     [,2]     [,3]
[1,] 250.7856 247.7073 243.6829
[2,] 257.1013 243.7262 244.7054
[3,] 244.6135 249.1279 247.1041
[4,] 240.0394 246.1091 246.2024




#==========================================
# should we accept NA as input?? Probably not...
 
path_exp <- '/esarchive/exp/ecmwf/system5c3s/daily_mean/$var$_f6h/$var$_$sdate$.nc'
path_obs <- '/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h-r360x181/$var$_$date$.nc'
var <- 'tas'
sdate <- paste0(1993:1995, '1201')

exp <- Start(dat = path_exp,
             var = var,
             sdate = sdate,
             time = indices(1:91),
             lat = indices(1),
             lon = indices(1),
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'sdate'),
             retrieve = FALSE)

# Change 2/28 to NA
dates <- attr(exp, 'Variables')$common$time
dates[1:2, 91] <- NA

sorted_dates <- sort(unique(format(dates, '%Y%m')))
unsorted_dates <- unique(format(dates, '%Y%m'))

# unsorted dates
obs1 <- Start(dat = path_obs,
             var = var,
             date = unsorted_dates,
             time = values(dates),  #dim: [sdate = 3, time = 90]
             lat = indices(1),
             lon = indices(1),
             time_across = 'date',
             merge_across_dims = TRUE,
#             merge_across_dims_narm = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'date'),
             retrieve = TRUE)

# sorted_dates
obs2 <- Start(dat = path_obs,
             var = var,
             date = sorted_dates,
             time = values(dates),  #dim: [sdate = 3, time = 90]
             lat = indices(1),
             lon = indices(1),
             time_across = 'date',
             merge_across_dims = TRUE,
#             merge_across_dims_narm = TRUE,
             split_multiselected_dims = TRUE,
             synonims = list(lat = c('lat', 'latitude'),
                             lon = c('lon', 'longitude')),
             return_vars = list(lon = NULL,
                                lat = NULL,
                                time = 'date'),
             retrieve = TRUE)


