library(startR)

lon.min <- 0
lon.max <- 359.9
lat.min <- -90
lat.max <- 90

data <- Start(dat = "/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc",
              var = 'tas', 
              sdate = '20170101',
              ensemble = indices(1),
              time = indices(1), 
              latitude = values(list(lat.min, lat.max)), #'all',
              latitude_reorder = Sort(),
              longitude = values(list(lon.min, lon.max)), #'all',
              longitude_reorder = CircularSort(0, 360),
              transform = CDORemapper,
              transform_extra_cells = 2,  
              transform_params = list(grid = 'r360x180', 
                                      method = 'conservative', 
                                      crop = c(lon.min, lon.max, lat.min, lat.max)),
              transform_vars = c('latitude', 'longitude'),
              return_vars = list(time = 'sdate', longitude = NULL, latitude = NULL),
              retrieve = T)

lon <- as.vector(attr(data, 'Variables')$common$longitude)
lat <- as.vector(attr(data, 'Variables')$common$latitude)

# NEW
library(easyNCDF)
library(ncdf4)
library(ClimProjDiags)
source('/home/Earth/aho/aho-testtest/startR/issue/issue_CDORemap_irregular/CDORemap_irregular.R')
source('/home/Earth/aho/aho-testtest/startR/issue/issue_CDORemap_irregular/CDORemapper_irregular.R')
source('/home/Earth/aho/aho-testtest/startR/issue/issue_CDORemap_irregular/Start_irregular.R')

data2 <- Start(dat = "/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc",
              var = 'tas',
              sdate = '20170101',
              ensemble = indices(1),
              time = indices(1),
              latitude = values(list(lat.min, lat.max)), #'all',
              latitude_reorder = Sort(),
              longitude = values(list(lon.min, lon.max)), #'all',
              longitude_reorder = CircularSort(0, 360),
              transform = CDORemapper_irregular,
              transform_extra_cells = 2, 
              transform_params = list(grid = 'r360x180',
                                      method = 'conservative',
                                      crop = F),
#                                      crop = c(lon.min, lon.max, lat.min, lat.max)),
              transform_vars = c('latitude', 'longitude'),
              return_vars = list(time = 'sdate', longitude = NULL, latitude = NULL),
              retrieve = T)



# native ocean curvilinear grid

path <- '/esarchive/scratch/pbretonn/$var$_Omon_CMCC-CM2-SR5_dcppA-hindcast_s$sdate$-$member$_gn_$aux$-subset.nc'

data3 <- Start(dataset = path,
              var = 'tos',
              sdate = paste0(1960),
              aux = 'all',
              aux_depends = 'sdate',
              j = 'all', #values(list(-90,90)),
              i = 'all', #values(list(0,359.9)),
              time = indices(1:2),
              member = 'r1i1p1f1',
              return_vars = list(j = NULL, i = NULL, latitude = NULL, longitude = NULL),
#              synonims = list(latitude = c('latitude'), longitude = 'longitude'),
              num_procs = 1 , retrieve = T)
lons <- attributes(data3)$Variables$common$longitude
lats <- attributes(data3)$Variables$common$latitude

res <- CDORemap_irregular(data3, lons, lats, grid = 'r100x50', method = 'bil', crop = FALSE)




data4 <- Start(dataset = path,
               var = 'tos',
               sdate = paste0(1960),
               aux = 'all',
               aux_depends = 'sdate',
               j = 'all', #values(list(-90,90)),
               i = 'all', #values(list(0,359.9)),
#               latitude = #'all',
#               longitude = values(list(0, 10)), #'all',
               time = indices(1),
               member = 'r1i1p1f1',
              transform = CDORemapper_irregular,
              transform_extra_cells = 2,
              transform_params = list(grid = 'r360x180',
                                      method = 'bil',#'conservative',
                                      crop = F),
#                                      crop = c(lon.min, lon.max, lat.min, lat.max)),
              transform_vars = c('latitude', 'longitude'), #c('i', 'j'),
               return_vars = list(j = NULL, i = NULL, latitude = NULL, longitude = NULL),
               num_procs = 1 , retrieve = T)

#===========================================
# s2dverification issue259

library(startR)
library(easyNCDF)
library(ncdf4)
library(ClimProjDiags)
source('/home/Earth/aho/aho-testtest/startR/issue/issue_CDORemap_irregular/CDORemap_irregular.R')
source('/home/Earth/aho/startR/R/Utils.R')
source('/home/Earth/aho/s2dverification/R/ArrayToNetCDF.R')


path <- '/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Omon/$var$/gn/v20190713/$var$_*_s$sdate$-$member$_gn_$aux$.nc'

data <- startR::Start(dataset = path,
                      var = 'tos',
                      sdate = paste0(1960),
                      aux = indices(1),
                      aux_depends = 'sdate',
                      j = 'all', 
                      i = 'all',
                      time = indices(1),
                      member = 'r1i1p1f1',
                      return_vars = list(j = NULL, i = NULL, 
                                         latitude = NULL, longitude = NULL),
                      retrieve = T)
lons <- attributes(data)$Variables$common$longitude
lats <- attributes(data)$Variables$common$latitude

dim(data)
#dataset     var   sdate     aux       j       i    time  member 
#      1       1       1       1     292     362       1       1 

#------------ regrid -------------
## case 1: full dims
res1 <- CDORemap_irregular(data, lons, lats, grid = 'r360x180', method = 'bil', crop = FALSE)
dim(res1$data_array)
#dataset     var   sdate     aux     lon     lat    time  member 
#      1       1       1       1     360     180       1       1 

summary(as.vector(res1$data_array))
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max.    NA's 
# -1.890   0.288  14.243  13.375  25.075  30.741   23530 
s2dv::PlotEquiMap(t(drop(res1$data_array)), lon = res1$lon, lat = res1$lat, fileout = 'irregular_regrid_res1.png')

## case 2: two dims
res2 <- CDORemap_irregular(data[1,1,1,1,,,1,1], lons, lats, grid = 'r360x180', method = 'bil', crop = FALSE)
dim(res2$data_array)
#lat lon 
#180 360 
summary(as.vector(res2$data_array))
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max.    NA's 
#  -1.89   -0.75   10.83   12.61   26.88   29.99   63292 
s2dv::PlotEquiMap(res2$data_array, lon = res2$lon, lat = res2$lat, fileout = 'irregular_regrid_res2.png')

## case 3: two dims, switch i and j
tmp <- t(data[1,1,1,1,,,1,1])
names(dim(tmp)) <- c('i', 'j')  
res3 <- CDORemap_irregular(tmp, lons, lats, grid = 'r360x180', method = 'bil', crop = FALSE)
dim(res3$data_array)
# lon lat 
# 360 180
summary(as.vector(res3$data_array))
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max.    NA's 
# -1.890   0.288  14.243  13.375  25.075  30.741   23530 
s2dv::PlotEquiMap(t(res3$data_array), lon = res3$lon, lat = res3$lat, fileout = 'irregular_regrid_res3.png')

