library(startR)
library(easyNCDF)
library(ncdf4)
library(ClimProjDiags)
source('/home/Earth/aho/aho-testtest/startR/issue/issue_CDORemap_irregular/CDORemap_irregular.R')
source('/home/Earth/aho/startR/R/Utils.R')
source('/home/Earth/aho/s2dverification/R/ArrayToNetCDF.R')


path <- '/esarchive/exp/CMIP6/dcppA-hindcast/cmcc-cm2-sr5/cmip6-dcppA-hindcast_i1p1/DCPP/CMCC/CMCC-CM2-SR5/dcppA-hindcast/$member$/Omon/$var$/gn/v20210312/$var$_*_s$sdate$-$member$_gn_$aux$.nc'
#path <- '/esarchive/scratch/pbretonn/$var$_Omon_CMCC-CM2-SR5_dcppA-hindcast_s$sdate$-$member$_gn_$aux$-subset.nc'

data3 <- Start(dataset = path,
              var = 'tos',
              sdate = c('1960'),
              aux = 'all',
              aux_depends = 'sdate',
              j = 'all', #values(list(-90,90)),
              i = 'all', #values(list(0,359.9)),
              time = indices(1:12), #indices(1),
              member = 'r1i1p1f1',
              return_vars = list(j = NULL, i = NULL, latitude = NULL, longitude = NULL),
              retrieve = F)

func_regrid <- function(data) {
  lons <- attr(data, 'Variables')$common$longitude
  lats <- attr(data, 'Variables')$common$latitude
  data <- CDORemap_irregular(data, lons, lats, grid = 'r360x180', method = 'bil', crop = FALSE)
  lons_reg <- data[['lons']]
  lats_reg <- data[['lats']]
  return(list(data = data[[1]], lats = lats_reg, lons = lons_reg))
}

  
step <- Step(fun = func_regrid,
             target_dims = list(data = c('sdate', 'aux', 'j', 'i')),
             output_dims = list(data = c('sdate', 'aux', 'lon', 'lat'), lats = 'lat', lons = 'lon'),
             use_attributes = list(data = "Variables"))
wf <- AddStep(data3, step)
  

res <- Compute(workflow = wf$data,
               chunks = list(time = 2))

names(res)
[1] "data" "lats" "lons"
dim(res$data)
  sdate     aux     lon     lat dataset     var    time  member 
      1       1     360     180       1       1       1       1 

s2dv::PlotEquiMap(t(res$data[1, 1, , , 1, 1, 1, 1]), 
                  lon = res$lons[, 1, 1, 1, 1], 
                  lat = res$lats[, 1, 1, 1, 1],
                  fileout = 'startR_wf_irregular_regrid_t1.png')



#=====================================

data <- Start(dataset = path,
              var = 'tos',
              sdate = paste0(1960),
              aux = 'all',
              aux_depends = 'sdate',
              j = 'all', #values(list(-90,90)),
              i = 'all', #values(list(0,359.9)),
              time = indices(1),
              member = 'r1i1p1f1',
              return_vars = list(j = NULL, i = NULL, latitude = NULL, longitude = NULL),
              retrieve = T)

dim(data)
#dataset     var   sdate     aux       j       i    time  member 
#      1       1       1       1     360     290       1       1 

lons <- attr(data, 'Variables')$common$longitude
dim(lons)
#  j   i 
#360 290 
lats <- attr(data, 'Variables')$common$latitude

res1 <- CDORemap_irregular(data, lons, lats, grid = 'r360x180', method = 'bil', crop = FALSE)
dim(res1$data_array)
#dataset     var   sdate     aux     lon     lat    time  member 
#      1       1       1       1     360     180       1       1 
s2dv::PlotEquiMap(res1$data_array, lon = res1$lon, lat = res1$lat, fileout = 'irregular_regrid_full.png')

res2 <- CDORemap_irregular(data[1,1,1,1,,,1,1], lons, lats, grid = 'r360x180', method = 'bil', crop = FALSE)
dim(res2$data_array)
#lat lon 
#180 360 
s2dv::PlotEquiMap(res2$data_array, lon = res2$lon, lat = res2$lat, fileout = 'irregular_regrid_2dims.png')

# When the length of dim is 4, it works.
data_sub <- ClimProjDiags::Subset(data, c(1,2,7,8), list(1,1,1,1), drop = 'selected')
res3 <- CDORemap_irregular(data_sub, lons, lats, grid = 'r360x180', method = 'bil', crop = FALSE)
s2dv::PlotEquiMap(res3$data_array, lon = res3$lon, lat = res3$lat, fileout = 'irregular_regrid_4dims.png')

