library(s2dverification)
library(SpecsVerification)

  path_exp <- list(name = 'system4_m1',
              path = file.path('/esarchive/exp/ecmwf',
                               '$EXP_NAME$/monthly_mean',
                               '$VAR_NAME$_f6h/$VAR_NAME$_$START_DATE$.nc'))

  path_obs <- list(name = 'erainterim',
              path = file.path('/esarchive/recon/ecmwf',
                               '$OBS_NAME$/monthly_mean',
                               '$VAR_NAME$_f6h/$VAR_NAME$_$YEAR$$MONTH$.nc'))

  res <- array(NA, dim = c(year = 5, month = 2, dataset = 1, 
                           ftime = 1, lat = 256, lon = 512, member = 1))

  t_begin_cal <- Sys.time()

# Do chunking manually
  for (yr in 2012:2016) {
    for (mth in 0:1) {
      startDates <- as.vector(sapply(yr, function(x) paste0(x, 
                                           sprintf('%02d', (1 + 6 * mth):(6 + 6 * mth)), '01')))
    
      data <- Load(var = 'tas',
                   exp = list(path_exp),
                   obs = list(path_obs),
                   sdates = startDates,
                   storefreq = "monthly",
                   leadtimemax = 1,
                   output = 'lonlat')
    
      mod <- data$mod
      obs <- data$obs
    
     #Calculate the ensemble-adjusted Continuous Ranked Probability Score (CRPS)
      CRPS <- multiApply::Apply(data = list(mod, obs),
                                target_dims = list(c('sdate', 'member'),
                                                   c('sdate')),
                                fun = SpecsVerification::EnsCrps)$output1
      names(dim(CRPS))[1] <- c('sdate')
      CRPS <- multiApply::Apply(data = CRPS,
                                target_dims = c('sdate'),
                                fun = mean)$output1
    
      res[yr-2011, mth+1, , , , , ] <- CRPS
    } #mth for loop
  } #yr for loop

  CRPS <- multiApply::Apply(data = res,
                            target_dims = c('month'),
                            fun = mean)$output1
  CRPS <- multiApply::Apply(data = CRPS,
                            target_dims = c('year'),
                            fun = mean)$output1

  t_end_cal <- Sys.time()
  t_cal <- as.numeric(difftime(t_end_cal, t_begin_cal, units = 'mins'))
#[1] 8.041727 mins


#Plotting
  brks <- seq(-0.1, 4,length.out = 11)
  PlotEquiMap(CRPS[1, 1, , , 1], data$lon, data$lat,
              color_fun = clim.palette('yellowred'), brks = brks,
              filled.continents = FALSE, triangle_ends = c(TRUE,TRUE),
              toptitle = 'ECMWF monthly mean tas CRPS 2012-2016', title_scale = 0.6)


