library(startR)
library(s2dverification)

  repos <- paste0('/esarchive/exp/ecmwf/system4_m1/monthly_mean/',
                  '$var$_f6h/$var$_$sdate$.nc')
  var <- 'tas'
  lons.min <- 0
  lons.max <- 360 
  lats.min <- -90
  lats.max <- 90
  sdates <- (sapply(1987:2016, function(x) paste0(x, sprintf('%02d', 1:12), '01')))
  names(dim(sdates)) <- c('month','year')

# exp data
  data_exp <- Start(dat = repos,
                      var = var,
                      sdate = sdates,  #paste0(1981:2016, '1101'),
                      time = indices(1), #'all',  #7
                      ensemble = 'all',  #15
                      latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),
             longitude = values(list(lons.min, lons.max)),
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude')),
#             transform = CDORemapper,
#             transform_extra_cells = 2,
#             transform_params = list(grid = 'r360x181',
#                                     method = 'conservative',
#                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
#             transform_vars = c('latitude', 'longitude'),
             return_vars = list(time = NULL,
                                latitude = 'dat',
                                longitude = 'dat'),
             #split_multiselected_dims = TRUE,
             merge_across_dims = TRUE,
             retrieve = F)

# obs data
 repos_obs <- paste0('/esarchive/recon/ecmwf/erainterim/monthly_mean/',
                  '$var$_f6h/$var$_$sdate$.nc')
  sdates_obs <- (sapply(1987:2016, function(x) paste0(x, sprintf('%02d', 1:12))))
  names(dim(sdates_obs)) <- c('month', 'year')

  data_obs <- Start(dat = repos_obs,
                      var = var,
                      sdate = sdates_obs,  #paste0(1981:2016, '1101'),
                      time = indices(1), #'all',  #7
                      #ensemble = 'all',  #15
                      latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),
             longitude = values(list(lons.min, lons.max)),
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude')),
#             transform = CDORemapper,
#             transform_extra_cells = 2,
#             transform_params = list(grid = 'r360x181',
#                                     method = 'conservative',
#                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
#             transform_vars = c('latitude', 'longitude'),
             return_vars = list(time = NULL,
                                latitude = 'dat',
                                longitude = 'dat'),
             #split_multiselected_dims = TRUE,
             merge_across_dims = TRUE,
             retrieve = F)





#-------------------------------

  func <- function(var_exp, var_obs) {
    var_obs <- s2dverification::InsertDim(var_obs, 2, 1)
    names(dim(var_obs)[2])<- c('ensemble')

   clim <- s2dverification::Clim(var_exp, var_obs)
    clim_exp <- clim$clim_exp
    clim_obs <- clim$clim_obs
    # bias
    asd <- array(0, dim = dim(clim_exp))
    for (i in 1:15) {
    asd[, i, ] <- clim_obs
    }
    bias <- clim_exp - asd #clim_obs
    names(dim(bias)) <- 'ensemble'
    return(bias)
  }


  step <- Step(func,
               target_dims = list(data_exp = c('dat', 'ensemble', 'sdate', 'time'), 
                                  data_obs = c('dat', 'sdate', 'time')),
               output_dims = c('dat', 'ensemble', 'time'))

  wf <- AddStep(list(data_exp = data_exp, data_obs = data_obs), step)


#Compute() locally
#  res <- Compute(wf,
#                 chunks = list(latitude = 2,
#                               longitude = 2))

#Compute() on fatnodes
  res <- Compute(wf,
                 chunks = list(latitude = 2,
                               longitude = 2),#$output1
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(queue_host = 'fat',
                                queue_type = 'slurm',
                                cores_per_job = 2,
                                job_wallclock = '00:50:00',
                                max_jobs = 4,
                                bidirectional = TRUE
                 ),
                 ecflow_suite_dir = '/home/Earth/aho/startR_local/'
#                 wait = TRUE
         )


