# Load startR
library(startR)

# Define a region
lons.min <- 0
lons.max <- 360
lats.min <- -90
lats.max <- 90

# Declaration of data sources
exp <- Start(dat = '/esarchive/exp/ecmwf/system4_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = paste0(1981:2010, '0101'),
             ensemble = 'all',
             time = indices(1),
#             latitude = values(list(lats.min, lats.max)),
#             latitude_reorder = Sort(),
#             longitude = values(list(lons.min, lons.max)),
             latitude = 'all', #indices(129:142),
             longitude = 'all', #indices(1:15),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = FALSE)

obs <- Start(dat = '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = paste0(1981:2010, '01'),
             time = indices(1),
#             latitude = values(list(lats.min, lats.max)),
#             latitude_reorder = Sort(),
#             longitude = values(list(lons.min, lons.max)),
             latitude = 'all', #indices(129:142),
             longitude = 'all', #indices(1:15),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             split_multiselected_dims = TRUE,
             merge_across_dims = TRUE,
             retrieve = FALSE)

# Define of the workflow

# Function
wrap_cal <- function(obs, exp) {
    obs <- s2dverification::InsertDim(obs, 1, 1)
    names(dim(obs))<- c('member', 'sdate')
    exp <- t(exp)
    names(dim(exp))<- c('member', 'sdate')
    calibrated <- CSTools:::.cal(var_obs = obs, var_exp = exp) # CSTools version 1.0.1 or earlier
#    return(calibrated)

#ensemble mean
    asd <- apply(calibrated, c(which(names(dim(calibrated)) != 'member')), mean)
    dim(asd) <- c(sdate = length(asd))   #give it a name!!!!

#climatology
    qwe <- mean(asd)
    dim(qwe) <- c(sdate = 1)
   return(qwe)
}

step <- Step(wrap_cal,
             target_dims = list(obs = c('sdate'), exp = c('sdate', 'ensemble')),
             output_dims = c('sdate'))

# workflow of operations
wf <- AddStep(list(obs = obs, exp = exp), step)


# Execution
#res1 <- Compute(wf, chunks = list(latitude = 2, longitude = 2))
#               threads_load = 2, threads_compute = 4)

# Power 9
#-------------------user-defined---------------------
  queue_host <- 'p1'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------

  res <- Compute(wf,
                 chunks = list(latitude = 2,
                               longitude = 2),#$output1
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(queue_host = queue_host,
                                queue_type = 'slurm',
                                temp_dir = temp_dir,
                                #lib_dir = '/gpfs/projects/bsc32/share/R_libs/3.5/',
                                r_module = 'R/3.5.0-foss-2018b',
                                #r_module = 'startR/0.1.2-foss-2018b-R-3.5.0',
                                job_wallclock = '01:00:00',
                                cores_per_job = 4,
                                max_jobs = 4,
                                bidirectional = FALSE,
                                polling_period = 50
                               ),
                 ecflow_suite_dir = ecflow_suite_dir,
                #ecflow_suite_dir = '/esarchive/scratch/aho/ecFlow/',
                 wait = TRUE
                 )  #$output1

