# Define selectors
  # HadGEM3 daily data (360_day)
  path_hadgem3 <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/hadgem3-gc31-mm/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MOHC/HadGEM3-GC31-MM/',
                         'dcppA-hindcast/r1i1p1f2/day/$var$/gn/v20200101/',
                         '$var$_day_HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-r1i1p1f2_gn_$fyear$.nc')
  ## tasmax_day_HadGEM3-GC31-MM_dcppA-hindcast_s2000-r1i1p1f2_gn_20001101-20101230.nc

  # MPI-ESM1.2-HR (standard)
  path_mpi_esm <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/mpi-esm1-2-hr/',
                         'cmip6-dcppA-hindcast_i1p1/DCPP/MPI-M/MPI-ESM1-2-HR/',
                         'dcppA-hindcast/r1i1p1f1/day/$var$/gn/v20200101/',
                         '$var$_day_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-r1i1p1f1_gn_$fyear$.nc')
  ## tasmax_day_MPI-ESM1-2-HR_dcppA-hindcast_s2000-r1i1p1f1_gn_20001101-20101231.nc

  # Selectors
  sdate <- '2000'
  fyear_hadgem3 <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1230')
  fyear_mpi_esm <- paste0(sdate, '1101-', as.numeric(sdate) + 10, '1231')
#  time_hadgem3 <- indices(c(60, NA, 61)) #12/30, NA, 1/1
#  time_mpi_esm <- indices(60:62) #12/30, 12/31, 1/1

res1 <- Start(
         dat = list(list(name = 'hadgem3', path = path_hadgem3)),
         var = 'tasmax', sdate = '2000',
         fyear = fyear_hadgem3,
         time = indices(c(1:60)),
         lat = indices(1:192), #'all',         
         lon = indices(1:384), #'all',
         return_vars = list(lat = NULL, lon = NULL, time = 'sdate'),
         retrieve = FALSE)


res2 <- Start(
         dat = list(list(name = 'mpi_esm', path = path_mpi_esm)),
         var = 'tasmax', sdate = '2000',
         fyear = fyear_mpi_esm,
         time = indices(c(1:60)),
         lat = indices(1:192), #'all',         
         lon = indices(1:384), #'all',
         return_vars = list(lat = NULL, lon = NULL, time = 'sdate'),
         retrieve = FALSE)


#res <- Start(
#         dat = list(list(name = 'hadgem3', path = path_hadgem3),
#                    list(name = 'mpi_esm', path = path_mpi_esm)),
#         var = 'tasmax', 
#         sdate = '2000',
#         time = indices(c(1:60)),
#         lat = 'all', 
#         lon = 'all',
#         return_vars = list(lat = NULL, lon = NULL, time = 'sdate'),
#         retrieve = FALSE)

func <- function(x, y) {
  return(x + y)
}

step <- Step(func, target_dims = list(res1 = c('var', 'lon'), res2 = c('var', 'lon')),
                     output_dims = c('var', 'lon'))

wf <- AddStep(list(res1, res2), step)

#-------------------user-defined---------------------
  queue_host <- 'nord1'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------

  output <- Compute(wf,
                 chunks = list(lat = 2),
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(queue_host = queue_host,
                   queue_type = 'lsf',
lib_dir = '/.statelite/tmpfs/gpfs/home/bsc32/bsc32734/R/x86_64-pc-linux-gnu-library/3.6',
                   temp_dir = temp_dir,
                   cores_per_job = 2,
                   job_wallclock = '05:00',
                   max_jobs = 4,
                   extra_queue_params = list('#BSUB -q bsc_es'),
                   bidirectional = FALSE,
                   polling_period = 10
                 ),
                 ecflow_suite_dir = ecflow_suite_dir,
                 wait = TRUE
                 )  #$output1


