library(startR)
library(ClimProjDiags)
library(s2dv)

path <- '/esarchive/oper/thredds-dust/monarch-dustclim/monthly/$var$/$var$_$date$_av_an.nc'
var <- 'pm10du'
date <- sapply(2007:2016, function(x) paste0(x, sprintf('%02d', 1:12)))

data <- Start(dat = path,
              var = var,
              date = as.vector(date),
              stats = 1,
              lat = 'all',
              lon = 'all',
              return_vars = list(lat = NULL, lon = NULL),
              retrieve = FALSE)

lon <- as.vector(attr(data, 'Variables')$common$lon)
lat <- as.vector(attr(data, 'Variables')$common$lat)

# Define function
func <- function(x, lon, lat) {
  # x: [lat, lon]
  res <- ClimProjDiags::WeightedMean(x, lon = lon, lat = lat)  # a number
  return(res)
}

step <- Step(fun = func, 
             target_dims = c('lat', 'lon'), 
             output_dims = NULL)

wf <- AddStep(data, step, lat = lat, lon = lon)

# Run locally
res <- Compute(wf,
               chunks = list(date = 4))

dim(res)

# Smoothing
res_smooth <- s2dv::Smoothing(res, time_dim = 'ftime', runmeanlen = 5)

#--------------------------------------

data <- Start(dat = path,
              var = var,
              date = as.vector(date)[1],
              stats = 1,
              lat = 'all',
              lon = 'all',
              return_vars = list(lat = NULL, lon = NULL),
              retrieve = FALSE)

lon <- attr(data, 'Variables')$common$lon
lat <- attr(data, 'Variables')$common$lat

# Define function
func <- function(x) {
  # x: [date]
  res <- mean(x)  # a number
  return(res)
}

step <- Step(fun = func,
             target_dims = c('date'),
             output_dims = NULL)

wf <- AddStep(data, step)

# Run locally
res <- Compute(wf,
               chunks = list(lat = 2, lon = 2))

dim(res$output1)
#  dat   var stats   lat   lon 
#    1     1     1   701  1021 

#error
#s2dv::PlotEquiMap(res$output1[1, 1, 1, , ], lon = lon, lat = lat, 
#                  toptitle = "Climatology PM10 2007-2016 monthly data",
#                  fileout = './map_pm10.png')

s2dv::PlotEquiMap(res$output1[1, 1, 1, , ], lon = seq(-20, 60, length.out = 1021), 
                  lat = seq(-10, 0, length.out = 701), 
                  toptitle = "Climatology PM10 2007-2016 monthly data", fileout = 'fig1.png')
