# Apply an existing mask file on data
# Author: An-Chi Ho
# ------------------------------------------------------------------



# A) Load mask file
mask_path <- '/esarchive/autosubmit/con_files/mask.regions.HadISST.nc'

mask <- Start(dat = mask_path,
              var =  'Baffin_Bay', # c('Baffin_Bay', 'Labrador_Sea'), 
              lat = 'all', 
              lon = 'all', 
              synonims = list(lat = c('lat', 'y'),
                             lon = c('lon', 'x')),
              return_vars = list(var_names = NULL,
                                 lat = NULL, lon = NULL), 
              var_var = 'var_names', 
              retrieve = F)


# B) Load data
data_path <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc'
lons.min <- -179.9
lons.max <- 179.9
lats.min <- -90
lats.max <- 90

data <- Start(dat = data_path,
              var = 'tos',
              sdate = paste0(1991:2010, '0101'),
              ensemble = 'all',
              time = indices(1),
              lat = values(list(lats.min, lats.max)),
              lat_reorder = Sort(),
              lon = values(list(lons.min, lons.max)),
              lon_reorder = CircularSort(-180, 180),
              transform = CDORemapper,
              transform_extra_cells = 2,
              transform_params = list(grid = mask_path, 
                                      method = 'conservative',
                                      crop = c(lons.min, lons.max, lats.min, lats.max)),
              transform_vars = c('lat', 'lon'),
              synonims = list(lat = c('lat', 'latitude'),
                              lon = c('lon', 'longitude')), 
              return_vars = list(lat = NULL,
                                 lon = NULL,
                                 time = 'sdate'),
              retrieve = FALSE)

# C) Build the workflow
reg_mask <- function(data, mask) {
  # data: [sdate = 20, ensemble = 25]
  # mask: [var = 2]
  if (all(mask == 1)) {
    trend <- s2dv::Trend(data, time_dim = 'sdate')$trend[2, , , ]
    ind <- mean(data)
  } else {
    ind <-  NA
  }
  return(ind)

}

step_mask <- Step(fun = reg_mask,
                  target_dims = list(data = c('dat', 'sdate', 'ensemble', 'time'),
                                     mask = c('dat')),
                  output_dim = NULL)

wf_mask <- AddStep(list(data, mask), step_mask)

res <- Compute(workflow = wf_mask,
                chunks = list(lat = 4,
                              lon = 4))

